Package: SAMprior
Type: Package
Title: Self-Adapting Mixture (SAM) Priors
Version: 3.0.0
Authors@R: c(person(given = "Peng", family = "Yang", email = "pyang7@mdanderson.org", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-0630-9506")), 
             person(given = "Ying", family = "Yuan", email = "yyuan@mdanderson.org", 
                    role = "aut",
                    comment = c(ORCID = "0000-0003-3163-480X")))
Maintainer: Peng Yang <pyang7@mdanderson.org>
Description: Implementation of the SAM prior and generation of its
     operating characteristics for dynamically borrowing information
     from historical data. For details, please refer to Yang et al. (2023)
     <doi:10.1111/biom.13927>. 
Depends: R (>= 3.5.0), RBesT, MatchIt
Imports: Metrics, assertthat, checkmate, ggplot2
Suggests: rmarkdown, knitr, testthat (>= 2.0.0), foreach, purrr,
        rstanarm (>= 2.17.2), scales, tools, broom, tidyr, parallel
VignetteBuilder: knitr
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-27 20:54:08 UTC; pyang7
Author: Peng Yang [aut, cre] (ORCID: <https://orcid.org/0000-0002-0630-9506>),
  Ying Yuan [aut] (ORCID: <https://orcid.org/0000-0003-3163-480X>)
Repository: CRAN
Date/Publication: 2026-04-27 22:10:02 UTC
Built: R 4.5.2; ; 2026-04-28 02:02:24 UTC; unix

Package: SAMprior
Type: Package
Title: Self-Adapting Mixture (SAM) Priors
Version: 3.0.0
Authors@R: c(person(given = "Peng", family = "Yang", email = "pyang7@mdanderson.org", 
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-0630-9506")), 
             person(given = "Ying", family = "Yuan", email = "yyuan@mdanderson.org", 
                    role = "aut",
                    comment = c(ORCID = "0000-0003-3163-480X")))
Maintainer: Peng Yang <pyang7@mdanderson.org>
Description: Implementation of the SAM prior and generation of its
     operating characteristics for dynamically borrowing information
     from historical data. For details, please refer to Yang et al. (2023)
     <doi:10.1111/biom.13927>. 
Depends: R (>= 3.5.0), RBesT, MatchIt
Imports: Metrics, assertthat, checkmate, ggplot2
Suggests: rmarkdown, knitr, testthat (>= 2.0.0), foreach, purrr,
        rstanarm (>= 2.17.2), scales, tools, broom, tidyr, parallel
VignetteBuilder: knitr
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-27 20:54:08 UTC; pyang7
Author: Peng Yang [aut, cre] (ORCID: <https://orcid.org/0000-0002-0630-9506>),
  Ying Yuan [aut] (ORCID: <https://orcid.org/0000-0003-3163-480X>)
Repository: CRAN
Date/Publication: 2026-04-27 22:10:02 UTC
Built: R 4.6.0; ; 2026-05-01 22:28:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(SAM_prior,betaMix)
S3method(SAM_prior,default)
S3method(SAM_prior,gammaMix)
S3method(SAM_prior,normMix)
S3method(SAM_weight,betaMix)
S3method(SAM_weight,default)
S3method(SAM_weight,gammaMix)
S3method(SAM_weight,normMix)
S3method(calibrate_cutoff_2arm,betaMix)
S3method(calibrate_cutoff_2arm,default)
S3method(calibrate_cutoff_2arm,normMix)
S3method(get_OC,betaMix)
S3method(get_OC,default)
S3method(get_OC,normMix)
S3method(support,betaMix)
S3method(support,default)
S3method(support,gammaMix)
S3method(support,normMix)
export(PS_prior)
export(PS_prior.beta)
export(PS_prior.default)
export(PS_prior.norm)
export(SAM_prior)
export(SAM_weight)
export(calibrate_cutoff_2arm)
export(calibrate_cutoff_bin_2arm)
export(calibrate_cutoff_cont_2arm)
export(eval_oc_bin_2arm)
export(eval_oc_cont_2arm)
export(eval_scenario_bin_2arm)
export(eval_scenario_cont_2arm)
export(get_OC)
export(post_summary_bin_2arm)
export(post_summary_cont_2arm)
import(MatchIt)
import(Metrics)
import(RBesT)
import(assertthat)
import(checkmate)
import(ggplot2)
import(stats)

## ----include=FALSE------------------------------------------------------------
library(SAMprior)
library(knitr)
knitr::opts_chunk$set(
    fig.width = 1.62*4,
    fig.height = 4
    )
## setup up fast sampling when run on CRAN
is_CRAN <- !identical(Sys.getenv("NOT_CRAN"), "true")
## NOTE: for running this vignette locally, please uncomment the
## following line:
## is_CRAN <- FALSE
.user_mc_options <- list()
if (is_CRAN) {
    .user_mc_options <- options(RBesT.MC.warmup=250, RBesT.MC.iter=500, RBesT.MC.chains=2, RBesT.MC.thin=1, RBesT.MC.control=list(adapt_delta=0.9))
}

## ----results="D_h",echo=FALSE-------------------------------------------------
set.seed(123)
std <- function(x) sd(x)/sqrt(length(x))
df_1 <- rnorm(40, 0, 3); 
df_2 <- rnorm(50, 0, 3); 
df_3 <- rnorm(60, 0, 3); 
dat <- data.frame(study = c(1,2,3),
                  n = c(40, 50, 60),
                  mean = round(c(mean(df_1), mean(df_2), mean(df_3)), 3),
                  se = round(c(std(df_1), std(df_2), std(df_3)), 3))
kable(dat)

## ----message=FALSE------------------------------------------------------------
sigma = 3
# load R packages
library(ggplot2)
theme_set(theme_bw()) # sets up plotting theme
set.seed(22)
map_mcmc <- gMAP(cbind(mean, se) ~ 1 | study, 
                 weights=n,data=dat,
                 family=gaussian,
                 beta.prior=cbind(0, sigma),
                 tau.dist="HalfNormal",tau.prior=cbind(0,sigma/2))

map_automix <- automixfit(map_mcmc)
map_automix
plot(map_automix)$mix

## ----message=FALSE------------------------------------------------------------
set.seed(234)
sigma        <- 3 ## Standard deviation in the current trial
data.crt     <- rnorm(35, mean = 0.4, sd = sigma)
wSAM_LRT <- SAM_weight(if.prior = map_automix, 
                       delta = 0.5 * sigma,
                       data = data.crt)
                   
cat('SAM weight: ', wSAM_LRT)

## ----message=FALSE------------------------------------------------------------
wSAM_PPR <- SAM_weight(if.prior = map_automix, 
                       delta = 0.5 * sigma,
                       method.w = 'PPR',
                       prior.odds = 3/7,
                       data = data.crt)
                       
cat('SAM weight: ', wSAM_PPR)

## ----echo=FALSE, message=FALSE, warning=FALSE---------------------------------
weight_grid <- seq(-3, 3, by = 0.3)
weight_res  <- lapply(weight_grid, function(x){
  res <- c()
  for(s in 1:300){
    data.control <- rnorm(n = 35, mean = x, sd = sigma)
    res <- c(res, SAM_weight(if.prior = map_automix,
                             delta = 0.5 * sigma,
                             data = data.control))
    
  }
  mean(res)
})
df_weight <- data.frame(grid   = weight_grid,
                        weight = unlist(weight_res))
qplot(grid, weight, data = df_weight, geom = "line", main= "SAM Weight") +
  xlab('Sample mean from control trial')+ ylab('Weight') +
  geom_vline(xintercept = summary(map_automix)['mean'], linetype = 2, col = 'blue') 

## ----message=FALSE------------------------------------------------------------
unit_prior <- mixnorm(nf.prior = c(1, summary(map_automix)['mean'], sigma))
SAM.prior <- SAM_prior(if.prior = map_automix, 
                       nf.prior = unit_prior,
                       weight = wSAM_LRT, sigma = sigma)
SAM.prior

## ----message=FALSE------------------------------------------------------------
# weak_prior <- mixnorm(c(1, summary(map_automix)[1], 1e4))
TypeI <- get_OC(if.prior = map_automix,         ## MAP prior from historical data
                nf.prior = unit_prior,          ## Unit-informative prior for mixture prior
                prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                delta    = 0.5*sigma,         ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Treatment effect for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0,    -0.2, 2),
                theta.t  = c(summary(map_automix)['mean'], -0.1, -0.2, 2))
kable(TypeI)

## ----message=FALSE------------------------------------------------------------
Power <- get_OC(if.prior = map_automix,         ## MAP prior from historical data
                nf.prior = unit_prior,          ## Unit-information prior for mixture prior
                prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                delta    = 0.5 * sigma,       ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Treatment effect for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.1, 0.5,  -2),
                theta.t  = c(summary(map_automix)['mean'], 1.1, 2.0,  -0.5))
kable(Power)

## -----------------------------------------------------------------------------
## Calibrate the posterior probability cutoff 
PPC <- calibrate_cutoff_2arm(if.prior = map_automix,         ## MAP prior from historical data
                             nf.prior = unit_prior,          ## Unit-information prior for mixture prior
                             prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                             target = 0.05,                 ## Targeted Type I error rate
                             n.t = 70, n = 35,              ## Sample size for treatment and control arms, respectively
                             theta.t = summary(map_automix)['mean'],  ## The true effect for treatment arm
                             theta = summary(map_automix)['mean'],    ## The true effect for control arm
                             sigma.t = sigma, sigma = sigma, ## Standard deviation in the treatment and control arms, respectively
                             ## Method to determine the mixture weight for the SAM prior
                             method = 'SAM',    ## Method 
                             delta = 0.2,       ## CSD for SAM prior
                             ## Trial settings
                             alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less".
                             margin = 0.     ## Clinical margin.
                             )

## ----include=F----------------------------------------------------------------
cat("Calibrated posterior probability cutoff:", round(PPC$cutoff, 4), "\n")

## -----------------------------------------------------------------------------
## Simulate data for treatment arm
data.trt <- rnorm(70, mean = 3, sd = sigma)

## first obtain posterior distributions...
post_SAM <- postmix(priormix = SAM.prior,   ## SAM Prior
                    data = data.crt)
post_trt <- postmix(priormix = unit_prior,  ## Non-informative prior
                    data = data.trt)

## Define the decision function
decision = decision2S(PPC$cutoff, 0, lower.tail=FALSE)

## Decision-making
decision(post_trt, post_SAM)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include=FALSE------------------------------------------------------------
options(.user_mc_options)

## ----include=FALSE------------------------------------------------------------
library(SAMprior)
library(knitr)
knitr::opts_chunk$set(
    fig.width = 1.62*4,
    fig.height = 4
    )
## setup up fast sampling when run on CRAN
is_CRAN <- !identical(Sys.getenv("NOT_CRAN"), "true")
## NOTE: for running this vignette locally, please uncomment the
## following line:
## is_CRAN <- FALSE
.user_mc_options <- list()
if (is_CRAN) {
    .user_mc_options <- options(RBesT.MC.warmup=250, RBesT.MC.iter=500, RBesT.MC.chains=2, RBesT.MC.thin=1, RBesT.MC.control=list(adapt_delta=0.9))
}

## ----results="asis",echo=FALSE------------------------------------------------
## Current trial data
ASAS20 <- data.frame(study = c('Baeten (2013)', 'Deodhar (2016)', 'Deodhar (2019)',
                               'Erdes (2019)', 'Huang (2019)', 'Kivitz (2018)',
                               'Pavelka (2017)', 'Sieper (2017)', 'Van der Heijde (2018)'),
                     n = c(6, 122, 104, 23, 153, 117, 76, 74, 87),
                     r = c(1, 35, 31, 10, 56, 55, 28, 21, 35))
kable(ASAS20)

## -----------------------------------------------------------------------------
# load R packages
library(ggplot2)
theme_set(theme_bw()) # sets up plotting theme
set.seed(22)
map_ASAS20 <- gMAP(cbind(r, n-r) ~ 1 | study,
                   family = binomial,
                   data = ASAS20, 
                   tau.dist = "HalfNormal", 
                   tau.prior = 1,
                   beta.prior = 2)

map_automix <- automixfit(map_ASAS20)
map_automix
plot(map_automix)$mix

## ----message=FALSE------------------------------------------------------------
n <- 35; r = 10 
wSAM_LRT <- SAM_weight(if.prior = map_automix, 
                       delta = 0.2,
                       n = n, r = r)
                   
cat('SAM weight: ', wSAM_LRT)

## ----message=FALSE------------------------------------------------------------
wSAM_PPR <- SAM_weight(if.prior = map_automix, 
                       delta = 0.2,
                       method.w = 'PPR',
                       prior.odds = 3/7,
                       n = n, r = r)
cat('SAM weight: ', wSAM_PPR)

## ----echo=FALSE, message=FALSE------------------------------------------------
weight_grid <- seq(1, 34, by = 1)
weight_res  <- lapply(1:34, function(x) SAM_weight(if.prior = map_automix, 
                                                   delta = 0.2,
                                                   n = 35, r = x))
df_weight <- data.frame(grid   = weight_grid/35,
                        weight = unlist(weight_res))
qplot(grid, weight, data = df_weight, geom = "line", main= "SAM Weight") +
  xlab('Response Rate from Control trial')+ ylab('Weight') +
  geom_vline(xintercept = summary(map_automix)['mean'], linetype = 2, col = 'blue') 

## ----message=FALSE------------------------------------------------------------
SAM.prior <- SAM_prior(if.prior = map_automix, 
                       nf.prior = mixbeta(nf.prior = c(1,1,1)),
                       weight = wSAM_LRT)
SAM.prior

## ----message=FALSE------------------------------------------------------------
TypeI <- get_OC(if.prior = map_automix,       ## MAP prior from historical data
                nf.prior = mixbeta(c(1,1,1)), ## Non-informative prior for mixture prior
                prior.t = mixbeta(c(1,1,1)),  ## Non-informative prior for treatment arm
                delta    = 0.2,               ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Response rates for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.30, 0.40, 0.60),
                theta.t  = c(summary(map_automix)['mean'], 0.30, 0.38, 0.61))
kable(TypeI)

## ----message=FALSE------------------------------------------------------------
Power <- get_OC(if.prior = map_automix,       ## MAP prior from historical data
                nf.prior = mixbeta(c(1,1,1)), ## Non-informative prior for mixture prior
                prior.t = mixbeta(c(1,1,1)),  ## Non-informative prior for treatment arm
                delta    = 0.2,               ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin,
                ## Response rates for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.36, 0.42, 0.16),
                theta.t  = c(summary(map_automix)['mean'], 0.56, 0.62, 0.36))
kable(Power)

## -----------------------------------------------------------------------------
## Sample size and number of responses for treatment arm
n_t <- 70; x_t <- 22 
## Calibrate the posterior probability cutoff 
PPC <- calibrate_cutoff_2arm(if.prior = map_automix,        ## MAP prior from historical data
                             nf.prior = mixbeta(c(1,1,1)),  ## Non-informative prior for mixture prior
                             prior.t = mixbeta(c(1,1,1)),   ## Non-informative prior for treatment arm
                             target = 0.05,                 ## Targeted Type I error rate
                             n.t = n_t, n = n,              ## Sample size for treatment and control arms, respectively
                             theta.t = summary(map_automix)['mean'],  ## The true response rate for treatment arm
                             theta = summary(map_automix)['mean'],    ## The true response rate for control arm
                             ## Method to determine the mixture weight for the SAM prior
                             method = 'SAM',    ## Method 
                             delta = 0.2,       ## CSD for SAM prior
                             ## Trial settings
                             alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less".
                             margin = 0      ## Clinical margin
                             )

## ----include=F----------------------------------------------------------------
cat("Calibrated posterior probability cutoff:", round(PPC$cutoff, 4), "\n")

## -----------------------------------------------------------------------------
## first obtain posterior distributions...
post_SAM <- postmix(priormix = SAM.prior,         ## SAM Prior
                    r = r,   n = n)
post_trt <- postmix(priormix = mixbeta(c(1,1,1)), ## Non-informative prior
                    r = x_t, n = n_t)

## Define the decision function
decision = decision2S(PPC$cutoff, 0, lower.tail=FALSE)

## Decision-making
decision(post_trt, post_SAM)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include=FALSE------------------------------------------------------------
options(.user_mc_options)

#' Evaluate Multiple Scenarios for a Two-Arm Comparative Trial with Binary Endpoint
#'
#' The \code{eval_oc_bin_2arm} function is designed to evaluate
#' repeated-sampling operating characteristics for multiple scenarios in a
#' two-arm comparative trial with a binary endpoint under one or more
#' borrowing strategies: self-adapting mixture prior (SAM), robust MAP prior
#' with fixed weight (rMAP), or non-informative prior (NP).
#'
#' For each scenario, the function computes the repeated-sampling rejection
#' probability, bias, RMSE, and mean borrowing weight using
#' \code{\link{eval_scenario_bin_2arm}}.
#'
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a beta mixture prior.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param theta A vector of true control arm response rates.
#' @param theta.t A vector of true treatment arm response rates.
#' @param n.t Sample size for the treatment arm.
#' @param n Sample size for the control arm.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param method Borrowing methods to evaluate. Any subset of
#' \code{c("SAM", "rMAP", "NP")}.
#' @param cutoff Posterior probability cutoff specification. Either a single
#' numeric value applied to all methods, or a named numeric vector/list with
#' method-specific cutoffs, for example
#' \code{c(SAM = 0.94, rMAP = 0.96, NP = 0.95)}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param rel.tol Relative tolerance passed to scenario-level numerical
#' integration used in \code{\link{post_summary_bin_2arm}}.
#'
#' @return A data frame with one row per scenario-method combination and columns:
#' \describe{
#'   \item{scenario}{Scenario index.}
#'   \item{theta}{True control arm response rate.}
#'   \item{theta.t}{True treatment arm response rate.}
#'   \item{delta_true}{True treatment effect, \eqn{\tau = \theta_t - \theta}.}
#'   \item{method}{Borrowing method used.}
#'   \item{alternative}{Direction of the posterior decision.}
#'   \item{cutoff}{Posterior probability cutoff used.}
#'   \item{margin}{Clinical margin used for inference.}
#'   \item{reject_prob}{Repeated-sampling rejection probability.}
#'   \item{bias}{Bias of the posterior mean estimator of \eqn{\theta}.}
#'   \item{rmse}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
#'   \item{mean_weight}{Average borrowing weight under the specified method.}
#' }
#'
#' @details
#' The vectors \code{theta} and \code{theta.t} must have the same length.
#' Each pair \code{(theta[i], theta.t[i])} defines one scenario.
#'
#' The \code{cutoff} argument may be common across methods or method-specific.
#' This is useful when each borrowing method is calibrated separately before
#' operating characteristics are evaluated.
#'
#' @export
eval_oc_bin_2arm <- function(if.prior, nf.prior, prior.t = nf.prior,
                             theta, theta.t,
                             n.t, n,
                             delta,
                             method = c("SAM", "rMAP", "NP"),
                             cutoff,
                             alternative = c("greater", "less"),
                             margin = 0,
                             weight_rMAP = 0.5,
                             method.w = "LRT",
                             prior.odds = 1,
                             rel.tol = 1e-8) {
  alternative <- match.arg(alternative)
  method <- unique(method)

  valid_methods <- c("SAM", "rMAP", "NP")
  if (!all(method %in% valid_methods)) {
    stop("`method` must be a subset of c(\"SAM\", \"rMAP\", \"NP\").")
  }

  if (!is.numeric(theta) || !is.numeric(theta.t)) {
    stop("`theta` and `theta.t` must be numeric vectors.")
  }
  if (length(theta) != length(theta.t)) {
    stop("`theta` and `theta.t` must have the same length.")
  }
  if (length(theta) == 0) {
    stop("At least one scenario must be provided.")
  }
  if (any(!is.finite(theta)) || any(!is.finite(theta.t))) {
    stop("`theta` and `theta.t` must contain only finite values.")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }

  resolve_cutoff <- function(method_name, cutoff) {
    if (is.numeric(cutoff) && length(cutoff) == 1) {
      return(cutoff)
    }

    if (is.list(cutoff)) {
      cutoff <- unlist(cutoff, recursive = TRUE, use.names = TRUE)
    }

    if (!is.numeric(cutoff) || is.null(names(cutoff))) {
      stop("`cutoff` must be either a scalar or a named numeric vector/list by method.")
    }

    if (!(method_name %in% names(cutoff))) {
      stop(sprintf("Missing cutoff for method '%s'.", method_name))
    }

    out <- cutoff[[method_name]]

    if (!is.numeric(out) || length(out) != 1 || !is.finite(out)) {
      stop(sprintf("Cutoff for method '%s' must be a finite scalar.", method_name))
    }

    out
  }

  out_list <- vector("list", length(theta) * length(method))
  idx <- 1L

  for (i in seq_along(theta)) {
    for (m in method) {
      cutoff_m <- resolve_cutoff(m, cutoff)

      res <- eval_scenario_bin_2arm(
        if.prior = if.prior,
        nf.prior = nf.prior,
        prior.t = prior.t,
        n.t = n.t,
        n = n,
        theta.t = theta.t[i],
        theta = theta[i],
        cutoff = cutoff_m,
        delta = delta,
        method = m,
        alternative = alternative,
        margin = margin,
        weight_rMAP = weight_rMAP,
        method.w = method.w,
        prior.odds = prior.odds,
        rel.tol = rel.tol
      )

      res$scenario <- i
      res <- res[, c("scenario", "theta", "theta.t", "delta_true",
                     "method", "alternative", "cutoff", "margin",
                     "reject_prob", "bias", "rmse", "mean_weight")]

      out_list[[idx]] <- res
      idx <- idx + 1L
    }
  }

  out <- do.call(rbind, out_list)
  rownames(out) <- NULL
  out
}

#' Calculating SAM priors
#'
#' The \code{SAM_prior} function is designed to display the SAM prior, given
#' the informative prior (constructed from historical data), non-informative
#' prior, and the mixture weight calculated using \code{\link{SAM_weight}}
#' function (\emph{Yang, et al., 2023}).
#'
#' @param if.prior Informative prior constructed from historical data,
#' represented (approximately) as a mixture of conjugate distributions.
#' @param nf.prior Non-informative prior used for the mixture.
#' @param weight Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight} \eqn{\leq 1}), which should be determined by
#' \code{\link{SAM_weight}} function.
#' @param ... Additional parameters required for different endpoints.
#'
#' @details SAM prior is constructed by mixing an informative prior
#' \eqn{\pi_1(\theta)}, constructed based on historical data, with a
#' non-informative prior \eqn{\pi_0(\theta)} using the mixture weight
#' \eqn{w} determined by \code{\link{SAM_weight}} function to achieve the
#' degree of prior-data conflict (\emph{Schmidli et al., 2015, Yang et al., 2023}).
#'
#'  Let \eqn{\theta} and \eqn{\theta_h} denote the treatment effects
#'  associated with the current arm data \eqn{D} and historical data \eqn{D_h},
#'  respectively. Let \eqn{\delta} denote the clinically significant difference
#'  such that if \eqn{|\theta_h - \theta| \ge \delta}, then \eqn{\theta_h} is
#'  regarded as clinically distinct from \eqn{\theta}, and it is therefore
#'  inappropriate to borrow any information from \eqn{D_h}. Consider two
#'  hypotheses:
#'
#'  \deqn{H_0: \theta = \theta_h, ~ H_1: \theta = \theta_h + \delta ~ or ~  \theta = \theta_h - \delta.}
#'  \eqn{H_0} represents that \eqn{D_h} and \eqn{D} are consistent (i.e.,
#'  no prior-data conflict) and thus information borrowing is desirable,
#'  whereas \eqn{H_1} represents that the treatment effect of \eqn{D}
#'  differs from \eqn{D_h} to such a degree that no information should be
#'  borrowed.
#'
#'  The SAM prior uses the likelihood ratio test (LRT) statistics \eqn{R} to
#'  quantify the degree of prior-data conflict and determine the extent of
#'  information borrowing.
#'
#'  \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(D | \theta = \theta_h) / \max(P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta)) ,}
#'  where \eqn{P(D | \cdot)} denotes the likelihood function. An alternative
#'  Bayesian choice is the posterior probability ratio (PPR):
#'
#'  \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(H_0) / P( H_1) \times BF, }
#'  where \eqn{P(H_0)} and \eqn{P(H_1)} is the prior probabilities of \eqn{H_0}
#'  and \eqn{H_1} being true. \eqn{BF} is the Bayes Factor that in this case
#'  is the same as the LRT.
#'
#'  The SAM prior, denoted as \eqn{\pi_{sam}(\theta)}, is then defined
#'  as a mixture of an informative prior \eqn{\pi_1(\theta)}, constructed
#'  based on \eqn{D_h} and a non-informative prior \eqn{\pi_0(\theta)}:
#'
#'  \deqn{\pi_{sam}(\theta) = w\pi_1(\theta) + (1-w)\pi_0(\theta),}
#'  where the mixture weight \eqn{w} is calculated as:
#'
#'  \deqn{w = R / (1 + R).}
#'
#'  As the level of prior-data conflict increases, the likelihood ratio
#'  \eqn{R} decreases, resulting in a decrease in the weight \eqn{w}
#'  assigned to the informative prior and thus a decrease in information
#'  borrowing. As a result, \eqn{\pi_{sam}(\theta)} is data-driven and
#'  has the ability to self-adapt the information borrowing based on the
#'  degree of prior-data conflict.
#'
#' @return Displays the SAM prior as a mixture of an informative prior
#' (constructed based on the historical data) and a non-informative prior.
#'
#' @references Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
#' SAM: Self-adapting mixture prior to dynamically borrow information from
#' historical data in clinical trials. \emph{Biometrics} 2023; 79(4), 2857-2868.
#' @references Schmidli H, Gsteiger S, Roychoudhury S, O'Hagan A,
#' Spiegelhalter D, Neuenschwander B.  Robust meta-analytic-predictive
#' priors in clinical trials with historical control information.
#' \emph{Biometrics} 2014; 70(4):1023-1032.
#'
#' @seealso \code{\link{SAM_weight}}
#'
#' @examples
#' set.seed(123)
#' ## Examples for binary endpoints
#' ## Suppose that the informative prior constructed based on historical data is
#' ## beta(40, 60)
#' prior.historical <- mixbeta(c(1, 40, 60))
#' ## Data of the control arm
#' data.control     <- rbinom(60, size = 1, prob = 0.42)
#' ## Calculate the mixture weight of the SAM prior
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.15,        ## Clinically significant difference
#'                    data = data.control  ## Control arm data
#'                    )
#' ## Assume beta(1,1) as the non-informative prior used for mixture
#' nf.prior  <- mixbeta(nf.prior = c(1,1,1))
#' ## Generate the SAM prior
#' SAM.prior <- SAM_prior(if.prior = prior.historical, ## Informative prior
#'                        nf.prior = nf.prior,         ## Non-informative prior
#'                        weight = wSAM                ## Mixture weight of the SAM prior
#'                        )
#' plot(SAM.prior)
#'
#' ## Examples for continuous endpoints
#' ## Suppose that the informative prior constructed based on historical data is
#' ## N(0, 3)
#' sigma      <- 3
#' prior.mean <- 0
#' prior.se   <- sigma/sqrt(100)
#' prior.historical <- mixnorm(c(1, prior.mean, prior.se), sigma = sigma)
#' ## Data of the control arm
#' data.control <- rnorm(80, mean = 0, sd = sigma)
#' ## Calculate the mixture weight of the SAM prior
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.2 * sigma,    ## Clinically significant difference
#'                    data = data.control     ## Control arm data
#'                    )
#' ## Assume unit-information prior N(0,3) as the non-informative prior used
#' ## for the mixture
#' nf.prior         <- mixnorm(nf.prior = c(1,prior.mean, sigma),
#'                             sigma = sigma)
#' ## Generate the SAM prior
#' SAM.prior <- SAM_prior(if.prior = prior.historical, ## Informative prior
#'                        nf.prior = nf.prior,         ## Non-informative prior
#'                        weight = wSAM                ## Mixture weight of the SAM prior
#'                        )
#' plot(SAM.prior)
#'
#' @import Metrics
#' @import RBesT
#' @import assertthat
#' @import checkmate
#' @import ggplot2
#' @import stats
#' @export
SAM_prior <- function(if.prior, nf.prior, weight, ...) UseMethod("SAM_prior")

#' @export
SAM_prior.default <- function(if.prior, nf.prior, weight, ...) "Unknown density"

#' @describeIn SAM_prior The function calculates the SAM prior for beta
#' mixture distribution. The default \code{nf.prior} is set to be
#' \code{mixbeta(c(1,1,1))} which represents a uniform prior \code{Beta(1,1)}.
#' @export
SAM_prior.betaMix <- function(if.prior, nf.prior, weight, ...) {
  checkmate::assert_number(weight, lower=0, upper=1)
  # checkmate::assert_number(n, lower=0, finite=TRUE)
  if(missing(nf.prior)) {
    message("Using default uniform prior as non-informative prior.")
    nf.prior <- RBesT::mixbeta(nf.prior = c(1,1,1))
  }
  # checkmate::assert_number(mean, lower=0, upper=1)
  # rob <- mixbeta(robust=c(1, mean, n+1), param="mn")
  # mixcombine(if.prior, rob, weight=c(weight, 1-weight))
  mixcombine(if.prior, nf.prior, weight=c(weight, 1-weight))

}

#' @describeIn SAM_prior The function calculates the SAM prior for gamma
#' mixture distribution. The default \code{nf.prior} is set to be
#' \code{mixgamma(c(1,0.001,0.001))} which represents a vague gamma prior
#' \code{Gamma(0.001,0.001)}.
#' @export
SAM_prior.gammaMix <- function(if.prior, nf.prior, weight, ...) {
  checkmate::assert_number(weight, lower=0, upper=1)
  # checkmate::assert_number(n, lower=0, finite=TRUE)
  if(missing(nf.prior)) {
    message(paste("Using default flat gamma prior as non-informative prior"))
    nf.prior <- mixgamma(nf.prior = c(1,0.001,0.001),
                         param = 'ab', likelihood = 'exp')
  }
  # checkmate::assert_number(mean, lower=0, finite=TRUE)
  # rob <- mixgamma(robust=c(1, mean, n), param="mn", likelihood=likelihood(if.prior))
  # mixcombine(if.prior, rob, weight=c(weight, 1-weight))
  mixcombine(if.prior, nf.prior, weight=c(weight, 1-weight))
}

#' @describeIn SAM_prior The function calculates the SAM prior for normal
#' mixture distribution. The default \code{nf.prior} is set to be
#' \code{mixnorm(c(1,summary(if.prior)['mean'], sigma))} which represents a
#' unit-information prior.
#' @param sigma Variance used for constructing the non-informative prior for
#' continuous endpoints.
#' @export
SAM_prior.normMix <- function(if.prior, nf.prior, weight, ..., sigma) {
  checkmate::assert_number(weight, lower=0, upper=1)
  # checkmate::assert_number(n, lower=0, finite=TRUE)
  # if(missing(mean)) {
  #   s <- summary(if.prior)
  #   message(paste("Using default mean for robust component; the mean of the prior which is", s["mean"], "."))
  #   mean <- s["mean"]
  # }
  # checkmate::assert_number(mean, finite=TRUE)
  if(missing(sigma)) {
    message("Using default prior reference scale ", RBesT::sigma(if.prior))
    sigma <- RBesT::sigma(if.prior)
  }
  if(missing(nf.prior)) {
    message(paste("Using default unit-information prior as non-informative prior"))
    nf.prior <- mixnorm(nf.prior = c(1,summary(if.prior)['mean'],sigma),
                        param = 'ms')
  }
  # rob <- mixnorm(robust=c(1, mean, n), param="mn", sigma=sigma)
  # mixcombine(if.prior, rob, weight=c(weight, 1-weight))
  mixcombine(if.prior, nf.prior, weight=c(weight, 1-weight))
}

#' Update Control Posterior Under SAM Prior for a Binary Endpoint
#'
#' Internal helper function to update the control arm posterior distribution
#' under a self-adapting mixture (SAM) prior for a binary endpoint.
#'
#' The control prior is formed by mixing an informative prior and a
#' non-informative prior using an adaptive borrowing weight determined by a
#' user-supplied weight function. Each informative component and the
#' non-informative component are updated separately under conjugate
#' beta-binomial updating, and the posterior mixture weights are obtained by
#' Bayes theorem.
#'
#' @param if.prior Informative beta mixture prior for the control arm,
#' typically a MAP prior constructed from historical data.
#' @param nf.prior Non-informative beta prior used as the robustifying
#' component in the SAM prior.
#' @param x Observed number of responses in the control arm.
#' @param n Number of subjects in the control arm.
#' @param delta Clinically significant difference used in the SAM borrowing rule.
#' @param weight_fun Function used to compute the adaptive borrowing weight for
#' the SAM prior. The function must return a value in \eqn{[0,1]}.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is LRT (Likelihood Ratio Test), the alternative option can
#' be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
#' details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights for the control arm posterior.}
#'   \item{a}{Posterior first beta shape parameters.}
#'   \item{b}{Posterior second beta shape parameters.}
#'   \item{wSAM}{Adaptive borrowing weight used in the SAM prior.}
#' }
#'
#' @details
#' Let \code{if.prior} denote the informative prior \eqn{\pi_1(\theta)} and
#' \code{nf.prior} denote the non-informative prior \eqn{\pi_0(\theta)}. The
#' SAM prior for the control arm is formed as
#' \deqn{\pi_{\mathrm{SAM}}(\theta) = w \pi_1(\theta) + (1-w)\pi_0(\theta),}
#' where \eqn{w} is the adaptive borrowing weight returned by
#' \code{weight_fun}.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.control_sam_update_bin <- function(if.prior, nf.prior, x, n, delta, weight_fun,
                                    method.w = "LRT", prior.odds = 1) {
  if (!is.function(weight_fun)) {
    stop("`weight_fun` must be a function returning the SAM weight w(x).")
  }
  if (!is.numeric(x) || length(x) != 1 || !is.finite(x) || x < 0 || x > n) {
    stop("`x` must be a scalar between 0 and `n`.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 || !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  
  w_sam <- weight_fun(
    x = x,
    if.prior = if.prior,
    nf.prior = nf.prior,
    n = n,
    delta = delta,
    method.w = method.w,
    prior.odds = prior.odds
  )
  
  if (!is.finite(w_sam) || w_sam < 0 || w_sam > 1) {
    stop("`weight_fun` must return a finite value in [0, 1].")
  }
  
  inf_mix <- .extract_mixbeta(if.prior)
  nf_mix  <- .extract_mixbeta(nf.prior)
  
  inf_upd <- lapply(seq_len(inf_mix$K), function(k) {
    .beta_update(inf_mix$a[k], inf_mix$b[k], x = x, n = n)
  })
  
  nf_upd <- lapply(seq_len(nf_mix$K), function(k) {
    .beta_update(nf_mix$a[k], nf_mix$b[k], x = x, n = n)
  })
  
  inf_marg <- vapply(inf_upd, `[[`, numeric(1), "marg")
  nf_marg  <- vapply(nf_upd,  `[[`, numeric(1), "marg")
  
  prior_w_inf <- w_sam * inf_mix$w
  prior_w_nf  <- (1 - w_sam) * nf_mix$w
  
  post_w_unnorm <- c(prior_w_inf * inf_marg, prior_w_nf * nf_marg)
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.control_sam_update_bin()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_a <- c(
    vapply(inf_upd, `[[`, numeric(1), "a"),
    vapply(nf_upd,  `[[`, numeric(1), "a")
  )
  
  post_b <- c(
    vapply(inf_upd, `[[`, numeric(1), "b"),
    vapply(nf_upd,  `[[`, numeric(1), "b")
  )
  
  list(
    w = post_w,
    a = post_a,
    b = post_b,
    wSAM = w_sam
  )
}

#' Update Control Posterior Under Fixed-Weight Mixture Prior for a Binary Endpoint
#'
#' Internal helper function to update the control arm posterior distribution
#' under a fixed-weight robust MAP prior for a binary endpoint.
#'
#' The control prior is formed by mixing an informative prior and a
#' non-informative prior using a prespecified fixed borrowing weight. Each
#' informative component and the non-informative component are updated
#' separately under conjugate beta-binomial updating, and the posterior
#' mixture weights are obtained by Bayes theorem.
#'
#' @param if.prior Informative beta mixture prior for the control arm.
#' @param nf.prior Non-informative beta prior used as the robustifying
#' component in the fixed-weight prior.
#' @param x Observed number of responses in the control arm.
#' @param n Number of subjects in the control arm.
#' @param weight Fixed borrowing weight assigned to the informative prior.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights for the control arm posterior.}
#'   \item{a}{Posterior first beta shape parameters.}
#'   \item{b}{Posterior second beta shape parameters.}
#'   \item{weight}{Fixed borrowing weight used in the prior.}
#' }
#'
#' @noRd
.control_fixed_update_bin <- function(if.prior, nf.prior, x, n, weight) {
  if (!is.numeric(weight) || length(weight) != 1 || !is.finite(weight) ||
      weight < 0 || weight > 1) {
    stop("`weight` must be a finite scalar in [0, 1].")
  }
  if (!is.numeric(x) || length(x) != 1 || !is.finite(x) || x < 0 || x > n) {
    stop("`x` must be a scalar between 0 and `n`.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  
  inf_mix <- .extract_mixbeta(if.prior)
  nf_mix  <- .extract_mixbeta(nf.prior)
  
  inf_upd <- lapply(seq_len(inf_mix$K), function(k) {
    .beta_update(inf_mix$a[k], inf_mix$b[k], x = x, n = n)
  })
  
  nf_upd <- lapply(seq_len(nf_mix$K), function(k) {
    .beta_update(nf_mix$a[k], nf_mix$b[k], x = x, n = n)
  })
  
  inf_marg <- vapply(inf_upd, `[[`, numeric(1), "marg")
  nf_marg  <- vapply(nf_upd,  `[[`, numeric(1), "marg")
  
  prior_w_inf <- weight * inf_mix$w
  prior_w_nf  <- (1 - weight) * nf_mix$w
  
  post_w_unnorm <- c(prior_w_inf * inf_marg, prior_w_nf * nf_marg)
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.control_fixed_update_bin()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_a <- c(
    vapply(inf_upd, `[[`, numeric(1), "a"),
    vapply(nf_upd,  `[[`, numeric(1), "a")
  )
  
  post_b <- c(
    vapply(inf_upd, `[[`, numeric(1), "b"),
    vapply(nf_upd,  `[[`, numeric(1), "b")
  )
  
  list(
    w = post_w,
    a = post_a,
    b = post_b,
    weight = weight
  )
}

#' Compute SAM Borrowing Weight for a Beta Mixture Prior
#'
#' Internal helper function to compute the self-adapting mixture (SAM)
#' borrowing weight for a binary endpoint using the summary-statistic version
#' of the likelihood ratio test rule.
#'
#' @param x Observed number of responses in the control arm.
#' @param if.prior Informative beta mixture prior for the control arm.
#' @param nf.prior Non-informative prior. Included for interface consistency
#' but not used in the calculation.
#' @param n Number of subjects in the control arm.
#' @param delta Clinically significant difference used in the SAM borrowing rule.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is LRT (Likelihood Ratio Test), the alternative option can
#' be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
#' details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#'
#' @return A scalar borrowing weight in \eqn{[0,1]}.
#'
#' @details
#' Let \eqn{\theta_h} denote the posterior mean estimate from the informative
#' prior. The SAM borrowing weight is defined as
#' \deqn{w = \frac{1}{1 + R},}
#' where
#' \deqn{
#' R =
#' \frac{
#' \max\{\Pr(X=x \mid \theta = \min(\theta_h + \delta, 0.99)),
#'      \Pr(X=x \mid \theta = \max(\theta_h - \delta, 0.01))\}
#' }{
#' \Pr(X=x \mid \theta = \theta_h)
#' }.
#' }
#'
#' Under the PPR rule, the likelihood ratio is additionally divided by
#' \code{prior.odds}. See \code{\link{SAM_weight}} for more details.
#'
#' @noRd
weight_fun_betamix <- function(x, if.prior, nf.prior, n, delta,
                               method.w = "LRT", prior.odds = 1) {
  if (!is.numeric(x) || length(x) != 1 || !is.finite(x) || x < 0 || x > n) {
    stop("`x` must be a scalar between 0 and `n`.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(delta) || length(delta) != 1 || !is.finite(delta) || delta < 0) {
    stop("`delta` must be a non-negative scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 || !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  
  theta_h <- summary(if.prior)["mean"]
  
  p1 <- min(theta_h + delta, 0.99)
  p2 <- max(theta_h - delta, 0.01)
  
  num <- max(
    dbinom(x = x, size = n, prob = p1),
    dbinom(x = x, size = n, prob = p2)
  )
  den <- dbinom(x = x, size = n, prob = theta_h)
  
  if (!is.finite(den) || den <= 0) {
    stop("SAM weight calculation failed in `weight_fun_betamix()`.")
  }
  
  R <- num / den
  
  if (method.w == "PPR") {
    R <- R / prior.odds
  }
  
  1 / (1 + R)
}

#' Support of Distributions
#'
#' Returns the support of a distribution.
#'
#'
#' takes x and transforms it according to the defined link function of
#' the mixture
#' @keywords internal
mixlink <- function(mix, x)
  attr(mix, "link")$link(x)

mixinvlink <- function(mix, x)
  attr(mix, "link")$invlink(x)

mixJinv_orig <- function(mix, x)
  attr(mix, "link")$Jinv_orig(x)

mixlJinv_orig <- function(mix, x)
  attr(mix, "link")$lJinv_orig(x)

mixlJinv_link <- function(mix, l)
  attr(mix, "link")$lJinv_link(l)

is.dlink <- function(x)
  inherits(x, "dlink")

is.dlink_identity <- function(x)
  is.dlink(x) & x$name == "identity"

is.mixidentity_link <- function(mix, l)
  is.dlink_identity(attr(mix, "link"))
#'
#' @param mix Mixture distribution.
#'
#' @keywords internal
support <- function(mix) UseMethod("support")
#' @export
support.default <- function(mix) "Unknown mixture"
#' @export
support.betaMix <- function(mix) mixlink(mix, c(0,1))
#' @export
support.gammaMix <- function(mix) mixlink(mix, c(0,Inf))
#' @export
support.normMix <- function(mix) mixlink(mix, c(-Inf,Inf))

#' Posterior Summary for Two-Arm Comparative Trial with Continuous Endpoint
#'
#' The \code{post_summary_cont_2arm} function is designed to compute the
#' posterior summary for the treatment effect in a two-arm comparative trial
#' with a continuous endpoint under one of three borrowing strategies:
#' self-adapting mixture prior (SAM), robust MAP prior with fixed weight
#' (rMAP), or non-informative prior (NP).
#'
#' The treatment effect is defined as \eqn{\tau = \theta_t - \theta}, where
#' \eqn{\theta_t} and \eqn{\theta} denote the means in the treatment and
#' control arms, respectively. Inference is based on the posterior distribution
#' of \eqn{\tau} given the observed sample means from the two arms.
#'
#' @param ybar_t Observed sample mean in the treatment arm.
#' @param ybar Observed sample mean in the control arm.
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a normal mixture prior.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param n.t Sample size for the treatment arm.
#' @param n Sample size for the control arm.
#' @param sigma.t Known standard deviation in the treatment arm.
#' @param sigma Known standard deviation in the control arm.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param cutoff Posterior probability cutoff used for decision making.
#' The null hypothesis is rejected if the posterior tail probability
#' exceeds \code{cutoff}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#'
#' @details
#' The posterior for the treatment arm is obtained by updating
#' \code{prior.t} using the observed sample mean \code{ybar_t}. The posterior
#' for the control arm depends on the selected borrowing strategy:
#'
#' \itemize{
#' \item \code{SAM}: the control arm prior is a mixture of \code{if.prior}
#' and \code{nf.prior}, with adaptive mixture weight determined by the
#' SAM borrowing rule.
#' \item \code{rMAP}: the control arm prior is a fixed mixture of
#' \code{if.prior} and \code{nf.prior}, with fixed weight
#' \code{weight_rMAP}.
#' \item \code{NP}: the control arm prior is \code{nf.prior} only.
#' }
#'
#' When \code{alternative = "greater"}, inference is based on
#' \eqn{P(\theta_t - \theta > margin \mid \bar{Y}_t, \bar{Y})}. When
#' \code{alternative = "less"}, inference is based on
#' \eqn{P(\theta_t - \theta < -margin \mid \bar{Y}_t, \bar{Y})}.
#'
#' The posterior mean and posterior variance of \eqn{\tau} are defined as
#' are defined as
#' \deqn{E(\tau \mid \bar{Y}_t, \bar{Y}) = E(\theta_t \mid \bar{Y}_t) - E(\theta \mid \bar{Y}),}
#' and
#' \deqn{\mathrm{Var}(\tau \mid \bar{Y}_t, \bar{Y}) = \mathrm{Var}(\theta_t \mid \bar{Y}_t) + \mathrm{Var}(\theta \mid \bar{Y}),}
#' where independence between the treatment and control arm posteriors is
#' assumed conditional on the current and historical data.
#'
#' @return A list containing the posterior probability in the requested
#' direction, posterior mean and variance of \eqn{\tau}, decision indicator,
#' borrowing weight used for the control arm prior, and the corresponding
#' trial data and method information.
#'
#' @seealso \code{\link{SAM_weight}}, \code{\link{SAM_prior}}
#'
#' @export
post_summary_cont_2arm <- function(ybar_t, ybar,
                                   if.prior, nf.prior, prior.t = nf.prior,
                                   n.t, n,
                                   sigma.t, sigma,
                                   delta,
                                   cutoff,
                                   method = c("SAM", "rMAP", "NP"),
                                   alternative = c("greater", "less"),
                                   margin = 0,
                                   weight_rMAP = 0.5,
                                   method.w = "LRT",
                                   prior.odds = 1) {
  method <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!is.numeric(ybar_t) || length(ybar_t) != 1 || !is.finite(ybar_t)) {
    stop("`ybar_t` must be a finite scalar.")
  }
  if (!is.numeric(ybar) || length(ybar) != 1 || !is.finite(ybar)) {
    stop("`ybar` must be a finite scalar.")
  }
  if (!is.numeric(cutoff) || length(cutoff) != 1 || cutoff <= 0 || cutoff >= 1) {
    stop("`cutoff` must be a scalar in (0, 1).")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (!is.numeric(n.t) || length(n.t) != 1 || !is.finite(n.t) || n.t <= 0) {
    stop("`n.t` must be a positive scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(sigma.t) || length(sigma.t) != 1 || !is.finite(sigma.t) || sigma.t <= 0) {
    stop("`sigma.t` must be a positive scalar.")
  }
  if (!is.numeric(sigma) || length(sigma) != 1 || !is.finite(sigma) || sigma <= 0) {
    stop("`sigma` must be a positive scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }

  post_t <- .posterior_mix_update(
    prior = prior.t,
    ybar = ybar_t,
    n = n.t,
    sigma = sigma.t
  )

  if (method == "SAM") {
    post_c <- .control_sam_update(
      if.prior = if.prior,
      nf.prior = nf.prior,
      ybar = ybar,
      n = n,
      sigma = sigma,
      delta = delta,
      weight_fun = weight_fun_normmix,
      method.w = method.w,
      prior.odds = prior.odds
    )
    weight_used <- post_c$wSAM

  } else if (method == "rMAP") {
    if (is.null(weight_rMAP) || length(weight_rMAP) != 1 ||
        !is.finite(weight_rMAP) || weight_rMAP < 0 || weight_rMAP > 1) {
      stop("`weight_rMAP` must be a scalar in [0, 1] when `method = \"rMAP\"`.")
    }

    post_c <- .control_fixed_update(
      if.prior = if.prior,
      nf.prior = nf.prior,
      ybar = ybar,
      n = n,
      sigma = sigma,
      weight = weight_rMAP
    )
    weight_used <- weight_rMAP

  } else {
    post_c <- .posterior_mix_update(
      prior = nf.prior,
      ybar = ybar,
      n = n,
      sigma = sigma
    )
    weight_used <- 0
  }

  post_prob_from_margin <- function(m) {
    out <- 0
    for (j in seq_along(post_t$w)) {
      for (k in seq_along(post_c$w)) {
        z_jk <- (post_t$mu[j] - post_c$mu[k] - m) /
          sqrt(post_t$var[j] + post_c$var[k])
        out <- out + post_t$w[j] * post_c$w[k] * pnorm(z_jk)
      }
    }
    out
  }

  post_prob_greater <- post_prob_from_margin(margin)

  post_prob <- if (alternative == "greater") {
    post_prob_greater
  } else {
    1 - post_prob_from_margin(-margin)
  }

  mean_t <- sum(post_t$w * post_t$mu)
  var_t  <- sum(post_t$w * (post_t$var + post_t$mu^2)) - mean_t^2

  mean_c <- sum(post_c$w * post_c$mu)
  var_c  <- sum(post_c$w * (post_c$var + post_c$mu^2)) - mean_c^2

  post_mean <- mean_t - mean_c
  post_var  <- var_t + var_c

  list(
    post_prob = post_prob,
    post_prob_greater = post_prob_greater,
    post_mean = post_mean,
    post_var = post_var,
    decision = as.numeric(post_prob > cutoff),
    weight = weight_used,
    method = method,
    alternative = alternative,
    margin = margin,
    ybar_t = ybar_t,
    ybar = ybar
  )
}

#' @name PS_SAM_data
#'
#' @title Simulated Data for the Construction of Propensity Score-Integrated Informative Priors
#'
#' @description  This dataset demonstrates the construction of a Propensity
#' Score-Integrated (PS) SAM prior. It simulates a two-arm randomized
#' clinical trial (RCT) with a 2:1 randomization ratio between treatment and
#' control arms, considering both binary and continuous endpoints.
#'
#'
#' @docType data
#' @format A data frame with 600 observations.
#' \itemize{
#' \item "A" is the treatment assignment (1 = treated, 0 = control).
#' \item "G" is the study indicator (1 = current, 0 = historical).
#' \item "\eqn{X_1}" is a binary covariate.
#' \item "\eqn{X_2}" is a continuous covariate.
#' \item "\eqn{X_3}" is a continuous covariate.
#' \item "\eqn{Y_{binary}}" is binary outcome.
#' \item "\eqn{Y_{continuous}}" is continuous outcome.
#' }
#'
#' @details
#' The dataset includes:
#' \itemize{
#'   \item Sample size for treatment arm: \eqn{n_t = 200}.
#'   \item Sample size for control arm: \eqn{n_c = 100}.
#'   \item Sample size for historical control study: \eqn{n_h = 300}.
#' }
#'
#' Covariates for the control arm were generated from
#'
#' \deqn{X_1 \sim Ber(0.5), ~~ X_2 \sim N(0, 1), ~~ X_3 \sim N(0.5, 1),}
#'
#' where \eqn{Ber(\cdot)} stands for Bernoulli distribution. Covariates for the
#' historical controls were generated from a mixture distribution, with half
#' were generated the same as for the control arm, while the other half were
#' drawn from
#'
#' \deqn{X_1 \sim Ber(0.8), ~~ X_2 \sim N(-0.4, 1), ~~ X_3 \sim N(-0.2, 1).}
#'
#' For the binary endpoint, \eqn{y_i} were generated from the logit model:
#'
#' \deqn{logit(\Pr(y_i = 1 | X_{1i}, X_{2i}, X_{3i}, A_i)) = -1.4 - 0.5
#' X_{1i} + X_{2i} + 2 X_{3i} + \lambda A_i,}
#'
#' where \eqn{\lambda} is the treatment effect size, and we let \eqn{\lambda = 0.9}
#' to generate a moderate treatment effect size so that they study has a reasonable
#' power.
#'
#' For the continuous endpoint, \eqn{y_i} were generated from the following
#' normal model:
#'
#' \deqn{y_i = 1.8 X_{1i} + 0.9 X_{2i} - 2 X_{3i} + \lambda A_i + \epsilon_i,}
#'
#' where we let \eqn{\lambda = 1}, and \eqn{\epsilon_i \sim N(0, 3.5^2)}.
#'
#' This dataset enables evaluation of the PS-SAM prior's performance in addressing heterogeneity
#' between the RCT control arm and historical controls.
#'
#' @keywords datasets
#'
#' @examples
#' # Load the dataset
#' data(PS_SAM_data)
#'
#' # View the structure
#' str(PS_SAM_data)
#'
"PS_SAM_data"

#' Extract Components from a Beta Mixture Prior
#'
#' Internal helper function to extract mixture weights and beta shape
#' parameters from a beta mixture prior object.
#'
#' The function assumes the mixture prior stores its component matrix in
#' \code{prior[[1]]}, with row names \code{"w"}, \code{"a"}, and \code{"b"}
#' corresponding to the mixture weights and beta shape parameters.
#'
#' @param prior A beta mixture prior object, such as one created by
#' \code{\link[SAMprior]{mixbeta}} or returned by related functions in the
#' \pkg{SAMprior} workflow.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Numeric vector of normalized mixture weights.}
#'   \item{a}{Numeric vector of first beta shape parameters.}
#'   \item{b}{Numeric vector of second beta shape parameters.}
#'   \item{K}{Number of mixture components.}
#' }
#'
#' @details
#' Missing component entries are removed before returning the extracted
#' quantities. The mixture weights are normalized to sum to 1.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.extract_mixbeta <- function(prior) {
  comp <- prior[[1]]
  
  if (is.null(comp)) {
    stop("`prior[[1]]` is NULL.")
  }
  
  comp <- as.matrix(comp)
  
  rn <- rownames(comp)
  if (is.null(rn)) {
    stop("`prior[[1]]` must have row names 'w', 'a', and 'b'.")
  }
  
  w_idx <- match("w", rn)
  a_idx <- match("a", rn)
  b_idx <- match("b", rn)
  
  if (any(is.na(c(w_idx, a_idx, b_idx)))) {
    stop("Could not find rows 'w', 'a', 'b' in `prior[[1]]`.")
  }
  
  w <- as.numeric(comp[w_idx, , drop = TRUE])
  a <- as.numeric(comp[a_idx, , drop = TRUE])
  b <- as.numeric(comp[b_idx, , drop = TRUE])
  
  keep <- !(is.na(w) | is.na(a) | is.na(b))
  w <- w[keep]
  a <- a[keep]
  b <- b[keep]
  
  if (length(w) == 0) stop("No mixture components found.")
  if (any(a <= 0) || any(b <= 0)) {
    stop("All beta shape parameters must be positive.")
  }
  
  w <- w / sum(w)
  
  list(
    w = w,
    a = a,
    b = b,
    K = length(w)
  )
}

#' Update a Beta Prior Component
#'
#' Internal helper function to update a single beta prior component using
#' conjugate beta-binomial updating based on an observed number of responses.
#'
#' Suppose
#' \deqn{\theta \sim \mathrm{Beta}(a, b)}
#' and
#' \deqn{X \mid \theta \sim \mathrm{Binomial}(n, \theta).}
#' Then this function returns the posterior beta parameters and the
#' beta-binomial marginal probability of \eqn{X}.
#'
#' @param a First beta shape parameter.
#' @param b Second beta shape parameter.
#' @param x Observed number of responses.
#' @param n Number of subjects.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{a}{Posterior first beta shape parameter.}
#'   \item{b}{Posterior second beta shape parameter.}
#'   \item{marg}{Beta-binomial marginal probability of \code{x}.}
#' }
#'
#' @details
#' The marginal probability of \code{x} is
#' \deqn{
#' \Pr(X=x)=\binom{n}{x}\frac{B(a+x,b+n-x)}{B(a,b)},
#' }
#' where \eqn{B(\cdot,\cdot)} denotes the beta function.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.beta_update <- function(a, b, x, n) {
  a_post <- a + x
  b_post <- b + n - x
  
  marg <- choose(n, x) * beta(a_post, b_post) / beta(a, b)
  
  list(
    a = a_post,
    b = b_post,
    marg = marg
  )
}

#' Update a Beta Mixture Prior
#'
#' Internal helper function to update all components of a beta mixture prior
#' using conjugate beta-binomial updating based on an observed number of
#' responses.
#'
#' Each mixture component is updated separately, and the posterior mixture
#' weights are obtained by Bayes theorem using the component-specific
#' beta-binomial marginal probabilities.
#'
#' @param prior A beta mixture prior object.
#' @param x Observed number of responses.
#' @param n Number of subjects.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights.}
#'   \item{a}{Posterior first beta shape parameters.}
#'   \item{b}{Posterior second beta shape parameters.}
#'   \item{marg}{Beta-binomial marginal probabilities under the prior components.}
#' }
#'
#' @details
#' Let the prior be represented as a finite mixture of beta components. Each
#' component is updated using \code{.beta_update()}, and the posterior mixture
#' weights are proportional to the prior mixture weights times the
#' corresponding beta-binomial marginal probabilities.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.posterior_mixbeta_update <- function(prior, x, n) {
  mix <- .extract_mixbeta(prior)
  
  upd <- lapply(seq_len(mix$K), function(j) {
    .beta_update(mix$a[j], mix$b[j], x = x, n = n)
  })
  
  marg <- vapply(upd, `[[`, numeric(1), "marg")
  post_w_unnorm <- mix$w * marg
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.posterior_mixbeta_update()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_a <- vapply(upd, `[[`, numeric(1), "a")
  post_b <- vapply(upd, `[[`, numeric(1), "b")
  
  list(
    w = post_w,
    a = post_a,
    b = post_b,
    marg = marg
  )
}

#' Calculating the Propensity Score-Integrated Informative Priors
#'
#' The \code{PS_prior} function is designed to calculate the Propensity Score-Integrated
#' (PS) informative prior constructed based on historical data.
#'
#' @param formula A two-sided \code{\link{formula}} object containing the study
#' indicator and covariates to be used in creating the distance measure used
#' in the matching. This formula will be supplied to the functions that estimate
#' the distance measure. For example, the formula should be specified as
#' \code{G ~ X1 + X2 + \dots} where \code{G} represents the name of study indicator
#'  and \code{X1} and \code{X2} are covariates.
#' @param data A data frame containing the variables named in \code{formula}
#' and possible other arguments.
#' @param outcome The variable name of the outcome.
#' @param study The variable name of the study indicator.
#' @param treat The variable name of the treatment indicator.
#' @param method The matching method to be used. The allowed methods are
#' \code{"nearest"} for nearest neighbor matching (on
#' the propensity score by default), \code{"optimal"} [\link[MatchIt]{method_optimal}]
#' for optimal pair matching, \code{"full"} [\link[MatchIt]{method_full}] for optimal
#' full matching, \code{"genetic"} [\link[MatchIt]{method_genetic}] for genetic
#' matching, \code{"cem"} [\link[MatchIt]{method_cem}] for coarsened exact matching,
#' \code{"exact"} [\link[MatchIt]{method_exact}] for exact matching,
#' \code{"cardinality"} [\link[MatchIt]{method_cardinality}] for cardinality and
#' template matching, and \code{"subclass"} [\link[MatchIt]{method_subclass}] for
#' subclassification. When set to \code{"NULL"}, no matching will occur, but
#' propensity score estimation and common support restrictions will still occur
#' if requested. See the linked pages for each method for more details on what
#' these methods do, how the arguments below are used by each on, and what
#' additional arguments are allowed.
#' @param distance The distance measure to be used. Can be either the name of a
#' method of estimating propensity scores (e.g., \code{"glm"}), the name of a
#' method of computing a distance matrix from the covariates (e.g.,
#' \code{"mahalanobis"}), a vector of already-computed distance measures, or a
#' matrix of pairwise distances. See [\link[MatchIt]{distance}] for allowable
#' options. The default is \code{"glm"} for propensity scores estimated with
#' logistic regression using \code{glm()}. Ignored for some methods; see individual
#' methods pages for information on whether and how the distance measure is
#' used.
#' @param ratio For methods that allow it, how many historical control units should be
#' matched to each current control unit in \eqn{k:1} matching. Should be a single integer
#' value. See the individual methods pages for information on whether and how
#' this argument is used. The default is 1 for 1:1 matching.
#' @param ps.method PS method utilize to calculate an informative prior
#' based on historical data. The allowed methods are \code{"Weighting"}
#' or \code{"Matching"}. The default method is \code{"Weighting"}.
#' @param trim Lower and upper bound of trimming used in \code{"Weighting"}.
#' The default is [0.1,0.9].
#'
#'
#'
#' @details This function aims to calculate informative priors using historical
#'  data by incorporating covariate information to enhance borrowing strength
#'  and address prior-data conflicts.
#'
#'  Let \eqn{G} be the study indicator, where \eqn{G = 1} indicate patient is
#'  from current control study, and \eqn{G = 0} indicate patient is from
#'  historical control study. Given the covariates data \eqn{X}, the propensity
#'  score is defined as follows,
#'  \deqn{e(X) = \Pr(G = 1 | X),}
#'  where \link[MatchIt]{distance} allows different methods to estimate the
#'  propensity scores.
#'
#'  Calculate informative prior through PS matching is to identify a subset of
#'  historical data (\eqn{D_h^*}) that have similar PS as current control data
#'  (\eqn{D}). Various algorithms are available for PS matching, please refer
#'  to \code{method}. The informative prior can then be calculated based on
#'  the matched historical dataset.
#'
#'  Alternative, we can utilize the inverse probability of treatment weighting
#'  (IPTW) to adjust the distribution of \eqn{X} in historical data \eqn{D_h},
#'  making it similar to that in \eqn{D}. Specifically, for the \eqn{i}th
#'  subject, we assign a weight \eqn{\alpha_i} to the outcome \eqn{y_i} in
#'  \eqn{D_h} based on its PS \eqn{e(X_i)} and a fixed weight \eqn{\alpha_i = 1}
#'  to \eqn{X_i} in \eqn{D}, as follows:
#'  \deqn{\alpha_i = G_1 + (1 - G_i) \frac{e(X_i)}{1 - e(X_i)}.}
#'  To avoid extremely large weights that may compromise IPTW, symmetric
#'  trimming rule can be used to trim the tails of the PS distribution by
#'  input \code{trim} with default [0.1,0.9], that is to trim observations
#'  whose estimated PS is outside of this range.
#'
#'  To standardized \eqn{\alpha}, we compute the effective sample size (ESS),
#'  which approximately reflects the level of precision or equivalently its
#'  sample size, retained in the sample after weight as
#'  \eqn{n^{*}_h = (\sum \alpha_i)^2 / \sum{\alpha_i^2}}. The standardized weight
#'  is given by
#'  \deqn{\alpha_i^{*} = G_i + (1 - G_i)\frac{G_i}{\sum{\alpha_i} / n_h^{*}}.}
#'
#'  For binary endpoint \eqn{Y \sim Ber(\theta)}, the informative
#'  prior \eqn{\pi_1(\theta)} can be constructed as follows,
#'  \deqn{\pi_1(\theta) \propto L(\theta | D_h, \alpha^{*}) \pi_0(\theta)
#'   = Beta(a + \sum \alpha_i^{*}y_i, b + n_h^* - \sum \alpha_i^{*}y_i )\},}
#'  where \eqn{\pi_0(\theta)} is a non-informative prior, a natural choice is
#'  \eqn{Beta(a, b)}, with \eqn{a = b = 1}.
#'
#'  For continuous endpoint \eqn{Y \sim N(0, \sigma^2)}, suppose \eqn{\sigma^2}
#'  is unknown, with non-informative prior \eqn{p(\theta, \sigma^2) \propto 1/\sigma^2},
#'  \eqn{\pi_1(\theta)} follows a student-\eqn{t} distribution with degree of
#'  freedom \eqn{n_h^{*} - 1}. Given that \eqn{n_h^{*}} is moderate and large,
#'  it can be approximated by a normal distribution
#'  \eqn{N(\bar{y}^{*}, {s^{*}}^2 / n_h^{*})} with
#'  \deqn{\bar{y}^{*} = \sum \alpha_i^* y_i / \alpha_i^*, ~~ {s^{*}}^2 =
#'  \sum \alpha_i^* (y_i - \bar{y}^{*})^2 / (n_h^{*}  - 1).}
#'
#' @return Displays the informative prior calculated from historical data
#' based on the selected PS method.
#'
#' @references Zhao Y, Laird G, Chen J, Yuan Y. PS-SAM: doubly robust
#' propensity-score-integrated self-adapting mixture prior to dynamically
#' borrow information from historical data.
#'
#' @seealso \link[MatchIt]{matchit}
#'
#' @examples
#' ## Load example data
#' data('PS_SAM_data')
#' ## Subset the data to contain historical data and current control
#' dat <- PS_SAM_data[PS_SAM_data$A == 0, ]
#' str(dat)
#'
#' ## Examples for binary endpoints
#' ## Generate the informative prior based on historical data using PS Matching
#' summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
#'                  data = dat, ps.method = 'Matching', method = 'nearest',
#'                  outcome = 'Y_binary', study = 'G', treat = 'A'))
#'
#' ## Generate the informative prior based on historical data using PS Weighting
#' summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
#'                  data = dat, ps.method = 'Weighting',
#'                  outcome = 'Y_binary', study = 'G', treat = 'A'))
#'
#' ## Examples for continuous endpoints
#' ## Generate the informative prior based on historical data using PS Matching
#' summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
#'                  data = dat, ps.method = 'Matching', method = 'nearest',
#'                  outcome = 'Y_continuous', study = 'G', treat = 'A'))
#'
#' ## Generate the informative prior based on historical data using PS Weighting
#' summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
#'                  data = dat, ps.method = 'Weighting',
#'                  outcome = 'Y_continuous', study = 'G', treat = 'A'))
#'
#'
#' @import Metrics
#' @import RBesT
#' @import assertthat
#' @import checkmate
#' @import ggplot2
#' @import stats
#' @import MatchIt
#' @export
PS_prior <- function(formula, data, outcome, study, treat, method, distance, ratio, ps.method, trim){


  if(missing(outcome)){
    stop("Plase input the name of outcome variable.")
  }else if(!outcome %in% colnames(data)){
    stop("The name of outcome variable does not match any column names in the dataset.")
  }

  if(missing(study)){
    stop("Plase input the name of study indicator.")
  }else if(!study %in% colnames(data)){
    stop("The name of study indicator does not match any column names in the dataset.")
  }

  if(missing(treat)){
    stop("Plase input the name of treatment indicator.")
  }else if(!treat %in% colnames(data)){
    stop("The name of treatment indicator does not match any column names in the dataset.")
  }

  if(missing(distance)){
    message("Using default glm method to estimate the propensity score.")
    distance <- 'glm'
  }

  if(missing(ps.method)){
    message("Using default mehtod Weighting to construct PS prior.")
    ps.method <- 'Weighting'
  }else{
    if(sum(ps.method %in% c('Matching', 'Weighting')) == 0){
      stop("Plase input the ps.method as either Weighting or Matching.")
    }
  }

  if(missing(method) & ps.method == 'Matching'){
    message("Using default nearest neighbor matching method for propensity score matching.")
    method <- 'nearest'
  }else if(missing(method)){
    method <- 'nearest'
  }

  if(missing(ratio) & ps.method == 'Matching') {
    message("Using default ratio as 1 for propensity score matching.")
    ratio <- 1
  }else if(missing(ratio)){
    ratio <- NULL
  }

  if(missing(trim) & ps.method == 'Weighting'){
    message("Using default triming range as [0.1, 0.9].")
    trim <- c(0.1, 0.9)
  }else if(missing(trim)){
    trim <- c(0.1, 0.9)
  }
  checkmate::assert_number(trim[1], lower=0, upper=1)
  checkmate::assert_number(trim[2], lower=0, upper=1)

  ## Determine the outcome data
  outcome_data <- data[ , outcome]

  # Determine outcome type
  is_binary <- all(outcome_data %in% c(0, 1))
  is_continuous <- is.numeric(outcome_data) && !is_binary

  if (is_binary) {
    # message("Binary outcome detected.")
    out <- PS_prior.beta(formula   = formula,
                         data      = data,
                         outcome   = outcome,
                         study     = study,
                         treat     = treat,
                         method    = method,
                         distance  = distance,
                         ratio     = ratio,
                         ps.method = ps.method,
                         trim      = trim)
  } else if (is_continuous) {
    # message("Continuous outcome detected.")
    out <- PS_prior.norm(formula   = formula,
                         data      = data,
                         outcome   = outcome,
                         study     = study,
                         treat     = treat,
                         method    = method,
                         distance  = distance,
                         ratio     = ratio,
                         ps.method = ps.method,
                         trim      = trim)
  } else {
    stop("Unsupported outcome type. Outcome must be either binary or continuous.")
  }

  return(out)

}

#' @describeIn PS_prior The function calculates the Propensity Score-Integrated
#' informative prior based on historical data for binary and continuous endpoint.
#' @export
PS_prior.default <- function(formula, data, outcome, study, treat, method, distance, ratio, ps.method, trim) "Unknown density"

#' @describeIn PS_prior The function calculates the Propensity Score-Integrated
#' informative prior based on historical data for binary endpoint.
#' @export
PS_prior.beta <- function(formula, data, outcome, study, treat, method, distance, ratio, ps.method, trim) {


  ## PS Matching
  if(ps.method == 'Matching'){

    ## Estimate the propensity score via MatchIt
    m.out <- MatchIt::matchit(formula = as.formula(formula), data = data, method = method,
                              distance = distance, ratio = ratio, caliper = 0.2)
    out   <- match.data(m.out, data = data)

    ## Subset the external data from matching
    dat_h <- subset(out, out[ , treat] == 0 & out[ , study] == 0)
    dat_h <- dat_h[ , outcome]

    ## Summary statsitics of subsetted historical data
    n_h <- length(dat_h)
    x_h <- sum(dat_h)

    ## Construct the informative prior
    if.prior <- RBesT::mixbeta(c(1, x_h, n_h - x_h))

  }

  ## IPW
  if(ps.method == 'Weighting'){

    ## Estimate the propensity score via MatchIt
    m.out <- MatchIt::matchit(formula = as.formula(formula), data = data,
                              distance = distance)
    out   <- match.data(m.out, data = data)

    ## Triming
    data$distance <- m.out$distance
    data_trimed <- subset(data, data$distance >= trim[1] & data$distance <= trim[2])
    ## Subset to historical data
    data_trimed <- subset(data_trimed, data_trimed[ , study] == 0 & data_trimed[ , treat] == 0)

    ## IPTW-ATT weight
    w_alpha <- data_trimed$distance / (1 - data_trimed$distance)

    # ESS
    n_h0 <- ((sum(w_alpha))^2) / (sum((w_alpha)^2))

    ## Normalized weight
    w_c  <- w_alpha / sum(w_alpha) *  n_h0

    ## Sum weighted of historical data
    x_h0 <- sum(data_trimed[ , outcome] * w_c)

    ## Construct the informative prior
    if.prior <- RBesT::mixbeta(c(1, x_h0, n_h0 - x_h0))

  }

  return(if.prior)

}

#' @describeIn PS_prior The function calculates the Propensity Score-Integrated
#' informative prior based on historical data for continuous endpoint.
#' @export
PS_prior.norm <- function(formula, data, outcome, study, treat, method, distance, ratio, ps.method, trim) {

  ## PS Matching
  if(ps.method == 'Matching'){

    ## Estimate the propensity score via MatchIt
    m.out <- MatchIt::matchit(formula = as.formula(formula), data = data, method = method,
                              distance = distance, ratio = ratio, caliper = 0.2)
    out   <- match.data(m.out, data = data)

    ## Subset the external data from matching
    dat_h <- subset(out, out[ , treat] == 0 & out[ , study] == 0)
    dat_h <- dat_h[ , outcome]

    ## Summary statsitics of subsetted historical data
    n_h <- length(dat_h)
    x_h <- mean(dat_h)

    ## Construct the informative prior
    if.prior <- RBesT::mixnorm(c(1,  x_h , sqrt(var(dat_h)/n_h)), sigma = sqrt(var(dat_h)))

  }

  ## IPW
  if(ps.method == 'Weighting'){


    ## Estimate the propensity score via MatchIt
    m.out <- MatchIt::matchit(formula = as.formula(formula), data = data,
                              distance = distance)
    out   <- match.data(m.out, data = data)

    # Triming
    data$distance <- m.out$distance
    data_trimed <- subset(data, data$distance >= trim[1] & data$distance <= trim[2])
    ## Subset to historical data
    data_trimed <- subset(data_trimed, data_trimed[ , study] == 0 & data_trimed[ , treat] == 0)

    ## Subset to historical data
    data_trimed <- subset(data_trimed, data_trimed[ , study] == 0)

    ## IPTW-ATT weight
    w_alpha <- data_trimed$distance / (1 - data_trimed$distance)

    # ESS
    n_h0 <- ((sum(w_alpha))^2) / (sum((w_alpha)^2))

    ## Normalized weight
    w_c  <- w_alpha / sum(w_alpha) *  n_h0

    ## mean of historical data
    x_h0  <- sum(data_trimed[ , outcome] * w_c) / sum(w_c)
    var_h0 <- sum((data_trimed[ , outcome] - x_h0)^2 * w_c) / (n_h0 - 1)

    ## Construct the informative prior
    if.prior <- RBesT::mixnorm(c(1,  x_h0 , sqrt(var_h0 / n_h0)), sigma = sqrt(var_h0))

  }

  return(if.prior)
}

#' Generating Operating Characteristics of SAM Priors Using Analytical Engines
#'
#' The \code{get_OC} function generates operating characteristics of SAM
#' priors using the analytical operating characteristic engines for two-arm
#' trials with binary or continuous endpoints. As an option, the operating
#' characteristics of robust MAP priors can also be generated for comparison.
#'
#' Compared with the original simulation-based implementation, this function
#' does not rely on trial simulation. Instead, it first calibrates the
#' posterior probability cutoff for each borrowing method to achieve the
#' target type I error, and then evaluates operating characteristics
#' analytically across the requested scenarios.
#'
#' @param if.prior Informative prior constructed from historical data,
#' represented (approximately) as a mixture of conjugate distributions.
#' @param theta.h Estimate of the treatment effect based on historical data.
#' Included for interface compatibility. If missing, the default value is set
#' to the posterior mean estimate from \code{if.prior}.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is LRT (Likelihood Ratio Test), the alternative option can
#' be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
#' details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param delta Clinically significant difference used for the SAM prior.
#' @param n Sample size for the control arm.
#' @param n.t Sample size for the treatment arm.
#' @param target Target type I error used to calibrate the posterior probability
#' cutoff for each method. The default value is typically 0.05.
#' @param if.rMAP Whether to evaluate the operating characteristics of the
#' robust MAP prior for comparison. The default value is \code{FALSE}.
#' @param weight.rMAP Weight assigned to the informative prior component
#' (\eqn{0 \le} \code{weight.rMAP} \eqn{\le 1}) for the robust MAP prior.
#' @param theta A vector of the response rate (binary endpoints) or mean
#' (continuous endpoints) for the control arm.
#' @param theta.t A vector of the response rate (binary endpoints) or mean
#' (continuous endpoints) for the treatment arm.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param rel.tol Tolerance passed to numerical root finding.
#' @param oc_rel.tol Relative tolerance passed to operating characteristic
#' evaluation.
#' @param interval Search interval for the posterior probability cutoff.
#' @param n_sd_int Half-width of the numerical integration region for each arm,
#' expressed as a multiple of the corresponding standard error. Used for
#' continuous endpoints only.
#' @param ... Additional parameters. For continuous endpoints, this includes
#' \code{sigma}.
#'
#' @details
#' For each borrowing method, the function first calibrates the posterior
#' probability cutoff so that the repeated-sampling rejection probability under
#' the boundary null scenario equals the target type I error \code{target}.
#' Specifically, calibration is based on the first value of \code{theta}. Let
#' \eqn{\theta = \code{theta}[1]} denote the control-arm parameter under the
#' calibration scenario. Then the treatment-arm parameter is set to
#' \eqn{\theta_t = \theta + margin} when \code{alternative = "greater"}, and
#' to \eqn{\theta_t = \theta - margin} when \code{alternative = "less"}.
#' Thus, when \code{margin = 0}, calibration is performed under the null
#' scenario \eqn{\theta_t = \theta}, corresponding to no treatment effect
#' difference between the treatment and control arms. After the cutoff is
#' calibrated, the function evaluates the operating characteristics across all
#' requested scenarios using the analytical two-arm engines.
#'
#' @return A data frame with one row per scenario-method combination. The
#' columns are:
#' \describe{
#'   \item{Scenarios}{Scenario index.}
#'   \item{theta}{True control-arm parameter value.}
#'   \item{theta.t}{True treatment-arm parameter value.}
#'   \item{Methods}{Borrowing method, one of \code{"NP"}, \code{"rMAP"}, or \code{"SAM"}.}
#'   \item{Cutoffs}{Calibrated posterior probability cutoff for the method.}
#'   \item{Bias of theta}{Bias of the posterior mean estimator of \eqn{\theta}.}
#'   \item{RMSE of theta}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
#'   \item{Weight}{Average borrowing weight under the method.}
#'   \item{Probability of Rejection}{Repeated-sampling rejection probability.}
#' }
#'
#' @references
#' Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
#' SAM: Self-adapting mixture prior to dynamically borrow information from
#' historical data in clinical trials.
#' \emph{Biometrics} 2023;79(4):2857-2868.
#'
#' Schmidli H, Gsteiger S, Roychoudhury S, O'Hagan A, Spiegelhalter D,
#' Neuenschwander B.
#' Robust meta-analytic-predictive priors in clinical trials with historical
#' control information.
#' \emph{Biometrics} 2014;70(4):1023-1032.
#'
#' @seealso \code{\link{eval_oc_bin_2arm}},
#' \code{\link{eval_oc_cont_2arm}}, \code{\link{calibrate_cutoff_bin_2arm}},
#' \code{\link{calibrate_cutoff_cont_2arm}}
#'
#' @examples
#' ## Example: operating characteristics for a two-arm binary trial
#' ## using a SAM prior without rMAP comparison
#'
#' ## Informative prior constructed from historical data
#' if.prior <- mixbeta(c(1, 20, 40))
#'
#' ## Evaluate operating characteristics
#' OC <- get_OC(
#'   if.prior = if.prior,
#'   nf.prior = mixbeta(c(1, 1, 1)),   ## Non-informative prior for control mixture
#'   prior.t = mixbeta(c(1, 1, 1)),    ## Prior for treatment arm
#'   delta = 0.2,                      ## Clinically significant difference for SAM
#'   n = 50,                           ## Sample size for control arm
#'   n.t = 100,                        ## Sample size for treatment arm
#'   target = 0.05,                    ## Target type I error for cutoff calibration
#'   if.rMAP = FALSE,                  ## Do not include rMAP comparison
#'   theta = c(summary(if.prior)["mean"], summary(if.prior)["mean"]),
#'   theta.t = c(summary(if.prior)["mean"], 0.50),
#'   alternative = "greater",          ## Superiority test
#'   margin = 0                        ## Clinical margin
#' )
#'
#' OC
#'
#' @export
get_OC <- function(if.prior, theta.h, method.w, prior.odds, nf.prior,
                   prior.t = nf.prior,
                   delta, n, n.t, target = 0.05,
                   if.rMAP = FALSE, weight.rMAP = 0.5,
                   theta, theta.t,
                   alternative = c("greater", "less"),
                   margin = 0,
                   rel.tol = 1e-5,
                   oc_rel.tol = 1e-6,
                   interval = c(0.5, 0.999),
                   n_sd_int = 8, ...) {

  UseMethod("get_OC")
}

#' @export
get_OC.default <- function(if.prior, theta.h, method.w, prior.odds, nf.prior,
                           prior.t = nf.prior,
                           delta, n, n.t, target = 0.05,
                           if.rMAP = FALSE, weight.rMAP = 0.5,
                           theta, theta.t,
                           alternative = c("greater", "less"),
                           margin = 0,
                           rel.tol = 1e-5,
                           oc_rel.tol = 1e-6,
                           interval = c(0.5, 0.999),
                           n_sd_int = 8, ...) {

  "Unknown density"
}

.combine_oc_results_long_format <- function(res_list, cal_list) {
  preferred_order <- c("NP", "rMAP", "SAM")
  method_names <- preferred_order[preferred_order %in% names(res_list)]

  n_scen <- nrow(res_list[[method_names[1]]])
  out_list <- vector("list", n_scen * length(method_names))
  idx <- 1L

  for (s in seq_len(n_scen)) {
    for (m in method_names) {
      res <- res_list[[m]]
      cal <- cal_list[[m]]

      out_list[[idx]] <- data.frame(
        Scenarios = s,
        theta = res$theta[s],
        theta.t = res$theta.t[s],
        Methods = m,
        Cutoffs = cal$cutoff,
        `Bias of theta` = res$bias[s],
        `RMSE of theta` = res$rmse[s],
        Weight = res$mean_weight[s],
        `Probability of Rejection` = res$reject_prob[s]
      )

      idx <- idx + 1L
    }
  }

  out <- do.call(rbind, out_list)
  rownames(out) <- NULL

  num_cols <- sapply(out, is.numeric)
  num_cols["Scenarios"] <- FALSE
  out[num_cols] <- lapply(out[num_cols], function(x) round(x, 4))

  out
}

#' @export
get_OC.betaMix <- function(if.prior, theta.h, method.w, prior.odds, nf.prior,
                           prior.t = nf.prior,
                           delta, n, n.t, target = 0.05,
                           if.rMAP = FALSE, weight.rMAP = 0.5,
                           theta, theta.t,
                           alternative = c("greater", "less"),
                           margin = 0,
                           rel.tol = 1e-5,
                           oc_rel.tol = 1e-6,
                           interval = c(0.5, 0.999),
                           n_sd_int = 8, ...) {

  alternative <- match.arg(alternative)

  if (length(theta) != length(theta.t)) {
    stop("Theta under control and treatment should be the same length!")
  }
  if (missing(if.prior)) {
    stop("Please input the informative prior!")
  }
  if (missing(n)) {
    stop("Please input the sample size for control arm!")
  }
  if (missing(n.t)) {
    stop("Please input the sample size for treatment arm!")
  }
  if (missing(delta)) {
    stop("Please input clinically significant difference!")
  }
  if (missing(theta.h)) {
    theta.h <- summary(if.prior)["mean"]
  }
  if (missing(method.w)) {
    method.w <- "LRT"
  }
  if (missing(prior.odds)) {
    prior.odds <- 1
  }
  if (missing(nf.prior)) {
    nf.prior <- mixbeta(nf.prior = c(1, 1, 1))
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }

  theta_cal <- theta[1]
  theta.t_cal <- if (alternative == "greater") {
    theta_cal + margin
  } else {
    theta_cal - margin
  }

  if (theta.t_cal < 0 || theta.t_cal > 1) {
    stop("Calibration scenario for binary endpoint is outside [0, 1].")
  }

  cal_list <- list()
  res_list <- list()

  message("Calibrating the posterior probability cutoff for NP prior method ...")
  cal_list$NP <- calibrate_cutoff_bin_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    theta.t = theta.t_cal,
    theta = theta_cal,
    delta = delta,
    method = "NP",
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol
  )

  if (if.rMAP) {
    message("Calibrating the posterior probability cutoff for rMAP prior method ...")
    cal_list$rMAP <- calibrate_cutoff_bin_2arm(
      if.prior = if.prior,
      nf.prior = nf.prior,
      prior.t = prior.t,
      target = target,
      n.t = n.t,
      n = n,
      theta.t = theta.t_cal,
      theta = theta_cal,
      delta = delta,
      method = "rMAP",
      alternative = alternative,
      margin = margin,
      weight_rMAP = weight.rMAP,
      method.w = method.w,
      prior.odds = prior.odds,
      interval = interval,
      rel.tol = rel.tol,
      oc_rel.tol = oc_rel.tol
    )
  }

  message("Calibrating the posterior probability cutoff for SAM prior method ...")
  cal_list$SAM <- calibrate_cutoff_bin_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    theta.t = theta.t_cal,
    theta = theta_cal,
    delta = delta,
    method = "SAM",
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol
  )

  message("Evaluating operating characteristics for NP prior method ...")
  res_list$NP <- eval_oc_bin_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    theta = theta,
    theta.t = theta.t,
    n.t = n.t,
    n = n,
    delta = delta,
    method = "NP",
    cutoff = cal_list$NP$cutoff,
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    rel.tol = oc_rel.tol
  )

  if (if.rMAP) {
    message("Evaluating operating characteristics for rMAP prior method ...")
    res_list$rMAP <- eval_oc_bin_2arm(
      if.prior = if.prior,
      nf.prior = nf.prior,
      prior.t = prior.t,
      theta = theta,
      theta.t = theta.t,
      n.t = n.t,
      n = n,
      delta = delta,
      method = "rMAP",
      cutoff = cal_list$rMAP$cutoff,
      alternative = alternative,
      margin = margin,
      weight_rMAP = weight.rMAP,
      method.w = method.w,
      prior.odds = prior.odds,
      rel.tol = oc_rel.tol
    )
  }

  message("Evaluating operating characteristics for SAM prior method ...")
  res_list$SAM <- eval_oc_bin_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    theta = theta,
    theta.t = theta.t,
    n.t = n.t,
    n = n,
    delta = delta,
    method = "SAM",
    cutoff = cal_list$SAM$cutoff,
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    rel.tol = oc_rel.tol
  )

  .combine_oc_results_long_format(
    res_list = res_list,
    cal_list = cal_list
  )
}

#' @export
get_OC.normMix <- function(if.prior, theta.h, method.w, prior.odds, nf.prior,
                           prior.t = nf.prior,
                           delta, n, n.t, target = 0.05,
                           if.rMAP = FALSE, weight.rMAP = 0.5,
                           theta, theta.t,
                           alternative = c("greater", "less"),
                           margin = 0,
                           rel.tol = 1e-5,
                           oc_rel.tol = 1e-6,
                           interval = c(0.5, 0.999),
                           n_sd_int = 8, ..., sigma) {
  alternative <- match.arg(alternative)

  if (length(theta) != length(theta.t)) {
    stop("Theta under control and treatment should be the same length!")
  }
  if (missing(if.prior)) {
    stop("Please input the informative prior!")
  }
  if (missing(n)) {
    stop("Please input the sample size for control arm!")
  }
  if (missing(n.t)) {
    stop("Please input the sample size for treatment arm!")
  }
  if (missing(delta)) {
    stop("Please input clinically significant difference!")
  }
  if (missing(theta.h)) {
    theta.h <- summary(if.prior)["mean"]
  }
  if (missing(method.w)) {
    method.w <- "LRT"
  }
  if (missing(prior.odds)) {
    prior.odds <- 1
  }
  if (missing(sigma)) {
    sigma <- RBesT::sigma(if.prior)
  }
  if (missing(nf.prior)) {
    nf.prior <- mixnorm(
      nf.prior = c(1, summary(if.prior)["mean"], sigma),
      param = "ms"
    )
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  theta_cal <- theta[1]
  theta.t_cal <- if (alternative == "greater") {
    theta_cal + margin
  } else {
    theta_cal - margin
  }

  cal_list <- list()
  res_list <- list()

  message("Calibrating the posterior probability cutoff for NP prior method ...")
  cal_list$NP <- calibrate_cutoff_cont_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    sigma.t = sigma,
    sigma = sigma,
    theta.t = theta.t_cal,
    theta = theta_cal,
    delta = delta,
    method = "NP",
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol,
    n_sd_int = n_sd_int
  )

  if (if.rMAP) {
    message("Calibrating the posterior probability cutoff for rMAP prior method ...")
    cal_list$rMAP <- calibrate_cutoff_cont_2arm(
      if.prior = if.prior,
      nf.prior = nf.prior,
      prior.t = prior.t,
      target = target,
      n.t = n.t,
      n = n,
      sigma.t = sigma,
      sigma = sigma,
      theta.t = theta.t_cal,
      theta = theta_cal,
      delta = delta,
      method = "rMAP",
      alternative = alternative,
      margin = margin,
      weight_rMAP = weight.rMAP,
      method.w = method.w,
      prior.odds = prior.odds,
      interval = interval,
      rel.tol = rel.tol,
      oc_rel.tol = oc_rel.tol,
      n_sd_int = n_sd_int
    )
  }

  message("Calibrating the posterior probability cutoff for SAM prior method ...")
  cal_list$SAM <- calibrate_cutoff_cont_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    sigma.t = sigma,
    sigma = sigma,
    theta.t = theta.t_cal,
    theta = theta_cal,
    delta = delta,
    method = "SAM",
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol,
    n_sd_int = n_sd_int
  )

  message("Evaluating operating characteristics for NP prior method ...")
  res_list$NP <- eval_oc_cont_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    theta = theta,
    theta.t = theta.t,
    n.t = n.t,
    n = n,
    sigma.t = sigma,
    sigma = sigma,
    delta = delta,
    method = "NP",
    cutoff = cal_list$NP$cutoff,
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    rel.tol = oc_rel.tol,
    n_sd_int = n_sd_int
  )

  if (if.rMAP) {
    message("Evaluating operating characteristics for rMAP prior method ...")
    res_list$rMAP <- eval_oc_cont_2arm(
      if.prior = if.prior,
      nf.prior = nf.prior,
      prior.t = prior.t,
      theta = theta,
      theta.t = theta.t,
      n.t = n.t,
      n = n,
      sigma.t = sigma,
      sigma = sigma,
      delta = delta,
      method = "rMAP",
      cutoff = cal_list$rMAP$cutoff,
      alternative = alternative,
      margin = margin,
      weight_rMAP = weight.rMAP,
      method.w = method.w,
      prior.odds = prior.odds,
      rel.tol = oc_rel.tol,
      n_sd_int = n_sd_int
    )
  }

  message("Evaluating operating characteristics for SAM prior method ...")
  res_list$SAM <- eval_oc_cont_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    theta = theta,
    theta.t = theta.t,
    n.t = n.t,
    n = n,
    sigma.t = sigma,
    sigma = sigma,
    delta = delta,
    method = "SAM",
    cutoff = cal_list$SAM$cutoff,
    alternative = alternative,
    margin = margin,
    weight_rMAP = NULL,
    method.w = method.w,
    prior.odds = prior.odds,
    rel.tol = oc_rel.tol,
    n_sd_int = n_sd_int
  )

  .combine_oc_results_long_format(
    res_list = res_list,
    cal_list = cal_list
  )
}

#' Calibrate Posterior Probability Cutoff for a Two-Arm Comparative Trial with Continuous Endpoint
#'
#' The \code{calibrate_cutoff_cont_2arm} function is designed to calibrate the
#' posterior probability cutoff for a two-arm comparative trial with a
#' continuous endpoint under one borrowing strategy: self-adapting mixture prior
#' (SAM), robust MAP prior with fixed weight (rMAP), or non-informative prior
#' (NP).
#'
#' The calibrated cutoff is chosen so that the repeated-sampling rejection
#' probability under a specified scenario equals the target value.
#'
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a normal mixture prior.
#' @param nf.prior Non-informative prior used for the mixture.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param target Target rejection probability under the calibration scenario,
#' typically the desired type I error rate when the calibration scenario
#' corresponds to the null hypothesis.
#' @param n.t Sample size for the treatment arm.
#' @param n Sample size for the control arm.
#' @param sigma.t Known sampling standard deviation for the treatment arm.
#' @param sigma Known sampling standard deviation for the control arm.
#' @param theta.t The mean for the treatment arm under the calibration scenario.
#' @param theta The mean for the control arm under the calibration scenario.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be non-negative. Default is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param interval Search interval for the posterior probability cutoff. The
#' default is from 0.5 to 0.999.
#' @param rel.tol Tolerance passed to \code{\link[stats]{uniroot}}.
#' @param oc_rel.tol Relative tolerance passed to scenario-level numerical
#' integration.
#' @param n_sd_int Half-width of the numerical integration region for each arm,
#' expressed as a multiple of the corresponding standard error.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{cutoff}{Calibrated posterior probability cutoff.}
#'   \item{objective}{Value of the root-finding objective at the solution.}
#'   \item{target}{Target rejection probability.}
#'   \item{method}{Borrowing method used.}
#'   \item{alternative}{Direction of the posterior decision.}
#'   \item{margin}{Clinical margin used for inference.}
#'   \item{theta}{True control-arm mean under the calibration scenario.}
#'   \item{theta.t}{True treatment-arm mean under the calibration scenario.}
#'   \item{interval}{Search interval used for calibration.}
#' }
#'
#' @details
#' The function solves for the posterior probability cutoff such that the
#' scenario-level rejection probability returned by
#' \code{\link{eval_scenario_cont_2arm}} matches \code{target}. Under null
#' scenarios, this corresponds to calibration of the type I error.
#'
#' @export
calibrate_cutoff_cont_2arm <- function(if.prior, nf.prior, prior.t = nf.prior,
                                       target = 0.05,
                                       n.t, n,
                                       sigma.t, sigma,
                                       theta.t = NULL,
                                       theta = NULL,
                                       delta,
                                       method = c("SAM", "rMAP", "NP"),
                                       alternative = c("greater", "less"),
                                       margin = 0,
                                       weight_rMAP = 0.5,
                                       method.w = "LRT",
                                       prior.odds = 1,
                                       interval = c(0.5, 0.999),
                                       rel.tol = 1e-5,
                                       oc_rel.tol = 1e-6,
                                       n_sd_int = 8) {
  method <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!is.numeric(target) || length(target) != 1 || !is.finite(target) ||
      target <= 0 || target >= 1) {
    stop("`target` must be a scalar in (0, 1).")
  }
  if (!is.numeric(interval) || length(interval) != 2 ||
      !all(is.finite(interval)) || interval[1] >= interval[2]) {
    stop("`interval` must be an increasing length-2 numeric vector.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  if (!is.numeric(rel.tol) || length(rel.tol) != 1 || !is.finite(rel.tol) || rel.tol <= 0) {
    stop("`rel.tol` must be a positive scalar.")
  }
  if (!is.numeric(oc_rel.tol) || length(oc_rel.tol) != 1 ||
      !is.finite(oc_rel.tol) || oc_rel.tol <= 0) {
    stop("`oc_rel.tol` must be a positive scalar.")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (is.null(theta.t)) theta.t <- summary(if.prior)["mean"]
  if (is.null(theta)) theta <- summary(if.prior)["mean"]

  f <- function(cutoff) {
    out <- eval_scenario_cont_2arm(
      if.prior = if.prior,
      nf.prior = nf.prior,
      prior.t = prior.t,
      n.t = n.t,
      n = n,
      sigma.t = sigma.t,
      sigma = sigma,
      theta.t = theta.t,
      theta = theta,
      cutoff = cutoff,
      delta = delta,
      method = method,
      alternative = alternative,
      margin = margin,
      weight_rMAP = weight_rMAP,
      method.w = method.w,
      prior.odds = prior.odds,
      rel.tol = oc_rel.tol,
      n_sd_int = n_sd_int
    )

    out$reject_prob - target
  }

  f_lo <- f(interval[1])
  f_hi <- f(interval[2])

  if (!is.finite(f_lo) || !is.finite(f_hi)) {
    stop("Calibration failed because the objective could not be evaluated at the interval endpoints.")
  }

  if (f_lo * f_hi > 0) {
    stop(
      sprintf(
        "Root not bracketed on [%.3f, %.3f]: f(lower)=%.6f, f(upper)=%.6f",
        interval[1], interval[2], f_lo, f_hi
      )
    )
  }

  out <- uniroot(f, interval = interval, tol = rel.tol)

  list(
    cutoff = out$root,
    objective = out$f.root,
    target = target,
    method = method,
    alternative = alternative,
    margin = margin,
    theta = theta,
    theta.t = theta.t,
    interval = interval
  )
}

#' Update Control Posterior Under SAM Prior
#'
#' Internal helper function to update the control arm posterior distribution
#' under a self-adapting mixture (SAM) prior for a continuous endpoint.
#'
#' The control prior is formed by mixing an informative prior and a
#' non-informative prior using an adaptive borrowing weight determined by a
#' user-supplied weight function. Each informative component and the
#' non-informative component are updated separately under conjugate
#' normal-normal updating, and the posterior mixture weights are obtained by
#' Bayes theorem.
#'
#' @param if.prior Informative normal mixture prior for the control arm,
#' typically a MAP prior constructed from historical data.
#' @param nf.prior Non-informative normal prior used as the robustifying
#' component in the SAM prior.
#' @param ybar Observed sample mean in the control arm.
#' @param n Number of subjects in the control arm.
#' @param sigma Known sampling standard deviation in the control arm.
#' @param delta Clinically significant difference used in the SAM borrowing rule.
#' @param weight_fun Function used to compute the adaptive borrowing weight for
#' the SAM prior. The function must return a value in \eqn{[0,1]}.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is LRT (Likelihood Ratio Test), the alternative option can
#' be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
#' details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights for the control arm posterior.}
#'   \item{mu}{Posterior means of the updated mixture components.}
#'   \item{var}{Posterior variances of the updated mixture components.}
#'   \item{wSAM}{Adaptive borrowing weight used in the SAM prior.}
#' }
#'
#' @details
#' Let \code{if.prior} denote the informative prior \eqn{\pi_1(\theta)} and
#' \code{nf.prior} denote the non-informative prior \eqn{\pi_0(\theta)}. The
#' SAM prior for the control arm is formed as
#' \deqn{\pi_{\mathrm{SAM}}(\theta) = w \pi_1(\theta) + (1-w)\pi_0(\theta),}
#' where \eqn{w} is the adaptive borrowing weight returned by
#' \code{weight_fun}.
#'
#' Each informative mixture component and the non-informative normal component
#' are updated separately using conjugate normal-normal updating. The posterior
#' mixture weights are proportional to the prior component weights multiplied by
#' the corresponding marginal density of the observed control arm sample mean.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.control_sam_update <- function(if.prior, nf.prior, ybar, n, sigma, delta,
                                weight_fun, method.w = "LRT", prior.odds = 1) {
  if (!is.function(weight_fun)) {
    stop("`weight_fun` must be a function returning the SAM weight w(ybar).")
  }
  if (!is.numeric(ybar) || length(ybar) != 1 || !is.finite(ybar)) {
    stop("`ybar` must be a finite scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(sigma) || length(sigma) != 1 || !is.finite(sigma) || sigma <= 0) {
    stop("`sigma` must be a positive scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 || !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  
  w_sam <- weight_fun(
    ybar = ybar,
    if.prior = if.prior,
    nf.prior = nf.prior,
    n = n,
    sigma = sigma,
    delta = delta,
    method.w = method.w,
    prior.odds = prior.odds
  )
  
  if (!is.finite(w_sam) || w_sam < 0 || w_sam > 1) {
    stop("`weight_fun` must return a finite value in [0, 1].")
  }
  
  inf_mix <- .extract_mixnorm(if.prior)
  nf_mix  <- .extract_mixnorm(nf.prior)
  
  inf_upd <- lapply(seq_len(inf_mix$K), function(k) {
    .normal_update(inf_mix$mu[k], inf_mix$sd[k], ybar = ybar, n = n, sigma = sigma)
  })
  
  nf_upd <- lapply(seq_len(nf_mix$K), function(k) {
    .normal_update(nf_mix$mu[k], nf_mix$sd[k], ybar = ybar, n = n, sigma = sigma)
  })
  
  inf_marg <- vapply(inf_upd, `[[`, numeric(1), "marg")
  nf_marg  <- vapply(nf_upd,  `[[`, numeric(1), "marg")
  
  prior_w_inf <- w_sam * inf_mix$w
  prior_w_nf  <- (1 - w_sam) * nf_mix$w
  
  post_w_unnorm <- c(prior_w_inf * inf_marg, prior_w_nf * nf_marg)
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.control_sam_update()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_mu <- c(
    vapply(inf_upd, `[[`, numeric(1), "mean"),
    vapply(nf_upd,  `[[`, numeric(1), "mean")
  )
  
  post_var <- c(
    vapply(inf_upd, `[[`, numeric(1), "var"),
    vapply(nf_upd,  `[[`, numeric(1), "var")
  )
  
  list(
    w = post_w,
    mu = post_mu,
    var = post_var,
    wSAM = w_sam
  )
}
#' Update Control Posterior Under Fixed-Weight Mixture Prior
#'
#' Internal helper function to update the control arm posterior distribution
#' under a fixed-weight robust MAP prior for a continuous endpoint.
#'
#' The control prior is formed by mixing an informative prior and a
#' non-informative prior using a prespecified fixed borrowing weight. Each
#' informative component and the non-informative component are updated
#' separately under conjugate normal-normal updating, and the posterior mixture
#' weights are obtained by Bayes theorem.
#'
#' @param if.prior Informative normal mixture prior for the control arm,
#' typically a MAP prior constructed from historical data.
#' @param nf.prior Non-informative normal prior used as the robustifying
#' component in the fixed-weight prior.
#' @param ybar Observed sample mean in the control arm.
#' @param n Number of subjects in the control arm.
#' @param sigma Known sampling standard deviation in the control arm.
#' @param weight Fixed borrowing weight assigned to the informative prior.
#' Must be a scalar in \eqn{[0,1]}.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights for the control arm posterior.}
#'   \item{mu}{Posterior means of the updated mixture components.}
#'   \item{var}{Posterior variances of the updated mixture components.}
#'   \item{weight}{Fixed borrowing weight used in the prior.}
#' }
#'
#' @details
#' Let \code{if.prior} denote the informative prior \eqn{\pi_1(\theta)} and
#' \code{nf.prior} denote the non-informative prior \eqn{\pi_0(\theta)}. The
#' fixed-weight prior for the control arm is formed as
#' \deqn{\pi(\theta) = w \pi_1(\theta) + (1-w)\pi_0(\theta),}
#' where \eqn{w} is the prespecified fixed borrowing weight.
#'
#' Each informative mixture component and the non-informative normal component
#' are updated separately using conjugate normal-normal updating. The posterior
#' mixture weights are proportional to the prior component weights multiplied by
#' the corresponding marginal density of the observed control arm sample mean.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.control_fixed_update <- function(if.prior, nf.prior, ybar, n, sigma, weight) {
  if (!is.numeric(weight) || length(weight) != 1 || !is.finite(weight) ||
      weight < 0 || weight > 1) {
    stop("`weight` must be a finite scalar in [0, 1].")
  }
  if (!is.numeric(ybar) || length(ybar) != 1 || !is.finite(ybar)) {
    stop("`ybar` must be a finite scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(sigma) || length(sigma) != 1 || !is.finite(sigma) || sigma <= 0) {
    stop("`sigma` must be a positive scalar.")
  }
  
  inf_mix <- .extract_mixnorm(if.prior)
  nf_mix  <- .extract_mixnorm(nf.prior)
  
  inf_upd <- lapply(seq_len(inf_mix$K), function(k) {
    .normal_update(
      mu = inf_mix$mu[k],
      sd = inf_mix$sd[k],
      ybar = ybar,
      n = n,
      sigma = sigma
    )
  })
  
  nf_upd <- lapply(seq_len(nf_mix$K), function(k) {
    .normal_update(
      mu = nf_mix$mu[k],
      sd = nf_mix$sd[k],
      ybar = ybar,
      n = n,
      sigma = sigma
    )
  })
  
  inf_marg <- vapply(inf_upd, `[[`, numeric(1), "marg")
  nf_marg  <- vapply(nf_upd,  `[[`, numeric(1), "marg")
  
  prior_w_inf <- weight * inf_mix$w
  prior_w_nf  <- (1 - weight) * nf_mix$w
  
  post_w_unnorm <- c(prior_w_inf * inf_marg, prior_w_nf * nf_marg)
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.control_fixed_update()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_mu <- c(
    vapply(inf_upd, `[[`, numeric(1), "mean"),
    vapply(nf_upd,  `[[`, numeric(1), "mean")
  )
  
  post_var <- c(
    vapply(inf_upd, `[[`, numeric(1), "var"),
    vapply(nf_upd,  `[[`, numeric(1), "var")
  )
  
  list(
    w = post_w,
    mu = post_mu,
    var = post_var,
    weight = weight
  )
}
#' Compute SAM Borrowing Weight for Normal Mixture Prior
#'
#' Internal helper function to compute the self-adapting mixture (SAM)
#' borrowing weight for a continuous endpoint using the summary-statistic
#' version of the likelihood ratio test rule.
#'
#' The weight is computed based on the observed control-arm sample mean and the
#' posterior mean of the informative prior. The resulting weight is used to mix
#' the informative prior and the non-informative prior in the SAM prior
#' construction.
#'
#' @param ybar Observed sample mean in the control arm.
#' @param if.prior Informative normal mixture prior for the control arm.
#' @param nf.prior Non-informative prior. Included for interface consistency
#' but not used in the calculation.
#' @param n Number of subjects in the control arm.
#' @param sigma Known sampling standard deviation in the control arm.
#' @param delta Clinically significant difference used in the SAM borrowing rule.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is LRT (Likelihood Ratio Test), the alternative option can
#' be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
#' details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#'
#' @return A scalar borrowing weight in \eqn{[0,1]}.
#'
#' @details
#' Let \eqn{\theta_h} denote the posterior mean estimate from the informative
#' prior. Under the default LRT rule, the SAM borrowing weight is defined as
#' \deqn{w = \frac{1}{1 + R},}
#' where
#' \deqn{R = \exp\{\max(R_1, R_2)\},}
#' with
#' \deqn{R_1 = -\frac{1}{2}\frac{n \delta (\delta - 2 \bar{Y} + 2 \theta_h)}{\sigma^2},}
#' and
#' \deqn{R_2 = -\frac{1}{2}\frac{n \delta (\delta + 2 \bar{Y} - 2 \theta_h)}{\sigma^2}.}
#'
#' Under the PPR rule, the likelihood ratio is additionally divided by
#' \code{prior.odds}. See \code{\link{SAM_weight}} for more details.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
weight_fun_normmix <- function(ybar, if.prior, nf.prior, n, sigma, delta,
                               method.w = "LRT", prior.odds = 1) {
  if (!is.numeric(ybar) || length(ybar) != 1 || !is.finite(ybar)) {
    stop("`ybar` must be a finite scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(sigma) || length(sigma) != 1 || !is.finite(sigma) || sigma <= 0) {
    stop("`sigma` must be a positive scalar.")
  }
  if (!is.numeric(delta) || length(delta) != 1 || !is.finite(delta) || delta < 0) {
    stop("`delta` must be a non-negative scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 || !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  
  theta_h <- summary(if.prior)["mean"]
  
  R1 <- -0.5 * (n * delta * (delta - 2 * ybar + 2 * theta_h)) / sigma^2
  R2 <- -0.5 * (n * delta * (delta + 2 * ybar - 2 * theta_h)) / sigma^2
  
  a <- max(R1, R2)
  
  if (method.w == "PPR") {
    a <- a - log(prior.odds)
  }
  
  if (a > 700) {
    return(0)
  }
  
  1 / (1 + exp(a))
}

#' Evaluate One Scenario for a Two-Arm Comparative Trial with Continuous Endpoint
#'
#' The \code{eval_scenario_cont_2arm} function is designed to evaluate
#' repeated-sampling operating characteristics for a two-arm comparative trial
#' with a continuous endpoint under one borrowing strategy: self-adapting
#' mixture prior (SAM), robust MAP prior with fixed weight (rMAP), or
#' non-informative prior (NP).
#'
#' The treatment effect is defined as \eqn{\tau = \theta_t - \theta},
#' where \eqn{\theta_t} and \eqn{\theta} denote the true means in the
#' treatment and control arms, respectively.
#'
#' For a given true scenario \eqn{(\theta_t, \theta)}, this function computes
#' the repeated-sampling rejection probability, bias, root mean squared error
#' (RMSE), and mean borrowing weight using one-dimensional numerical
#' integration.
#'
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a normal mixture prior.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param n.t Number of subjects in the treatment arm.
#' @param n Number of subjects in the control arm.
#' @param sigma.t Known sampling standard deviation in the treatment arm.
#' @param sigma Known sampling standard deviation in the control arm.
#' @param theta.t True treatment arm mean.
#' @param theta True control arm mean.
#' @param cutoff Posterior probability cutoff used for decision making.
#' Rejection occurs if the posterior probability exceeds \code{cutoff}.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param rel.tol Relative tolerance passed to numerical integration.
#' @param n_sd_int Half-width of the numerical integration region for each arm,
#' expressed as a multiple of the corresponding standard error.
#'
#' @return A one-row data frame with the following columns:
#' \describe{
#'   \item{theta}{True control arm mean.}
#'   \item{theta.t}{True treatment arm mean.}
#'   \item{delta_true}{True treatment effect, \eqn{\tau = \theta_t - \theta}.}
#'   \item{method}{Borrowing method used.}
#'   \item{alternative}{Direction of the posterior decision.}
#'   \item{cutoff}{Posterior probability cutoff used for decision making.}
#'   \item{margin}{Clinical margin used for inference.}
#'   \item{reject_prob}{Repeated-sampling rejection probability.}
#'   \item{bias}{Bias of the posterior mean estimator of \eqn{\theta}.}
#'   \item{rmse}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
#'   \item{mean_weight}{Average borrowing weight under the specified method.}
#' }
#'
#' @details
#' The rejection probability is computed by reducing the repeated-sampling
#' decision rule to a one-dimensional integral over the control-arm sample mean.
#'
#' Bias and RMSE are evaluated for the posterior mean estimator of the control
#' arm mean \eqn{\theta}. Both are computed from one-dimensional first and
#' second moments of the control-arm posterior mean. The mean borrowing weight
#' is computed by one-dimensional integration over the control-arm sample mean.
#'
#' Under null scenarios, \code{reject_prob} corresponds to type I error. Under
#' alternative scenarios, it corresponds to power.
#'
#' @export
eval_scenario_cont_2arm <- function(if.prior, nf.prior, prior.t = nf.prior,
                                    n.t, n,
                                    sigma.t, sigma,
                                    theta.t, theta,
                                    cutoff,
                                    delta,
                                    method = c("SAM", "rMAP", "NP"),
                                    alternative = c("greater", "less"),
                                    margin = 0,
                                    weight_rMAP = 0.5,
                                    method.w = "LRT",
                                    prior.odds = 1,
                                    rel.tol = 1e-6,
                                    n_sd_int = 8) {
  method <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!is.numeric(n.t) || length(n.t) != 1 || !is.finite(n.t) || n.t <= 0) {
    stop("`n.t` must be a positive scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(sigma.t) || length(sigma.t) != 1 || !is.finite(sigma.t) || sigma.t <= 0) {
    stop("`sigma.t` must be a positive scalar.")
  }
  if (!is.numeric(sigma) || length(sigma) != 1 || !is.finite(sigma) || sigma <= 0) {
    stop("`sigma` must be a positive scalar.")
  }
  if (!is.numeric(theta.t) || length(theta.t) != 1 || !is.finite(theta.t)) {
    stop("`theta.t` must be a finite scalar.")
  }
  if (!is.numeric(theta) || length(theta) != 1 || !is.finite(theta)) {
    stop("`theta` must be a finite scalar.")
  }
  if (!is.numeric(cutoff) || length(cutoff) != 1 || cutoff <= 0 || cutoff >= 1) {
    stop("`cutoff` must be a scalar in (0, 1).")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  if (!is.numeric(rel.tol) || length(rel.tol) != 1 || !is.finite(rel.tol) || rel.tol <= 0) {
    stop("`rel.tol` must be a positive scalar.")
  }
  if (!is.numeric(n_sd_int) || length(n_sd_int) != 1 || !is.finite(n_sd_int) || n_sd_int <= 0) {
    stop("`n_sd_int` must be a positive scalar.")
  }
  if (method == "rMAP") {
    if (is.null(weight_rMAP) || length(weight_rMAP) != 1 ||
        !is.finite(weight_rMAP) || weight_rMAP < 0 || weight_rMAP > 1) {
      stop("`weight_rMAP` must be a scalar in [0, 1] when `method = \"rMAP\"`.")
    }
  }

  se_t <- sigma.t / sqrt(n.t)
  se   <- sigma / sqrt(n)

  lower_t <- theta.t - n_sd_int * se_t
  upper_t <- theta.t + n_sd_int * se_t
  lower   <- theta   - n_sd_int * se
  upper   <- theta   + n_sd_int * se

  ## ---- control posterior object given ybar ----
  post_c_fun <- function(ybar) {
    if (method == "SAM") {
      .control_sam_update(
        if.prior = if.prior,
        nf.prior = nf.prior,
        ybar = ybar,
        n = n,
        sigma = sigma,
        delta = delta,
        weight_fun = weight_fun_normmix,
        method.w = method.w,
        prior.odds = prior.odds
      )
    } else if (method == "rMAP") {
      .control_fixed_update(
        if.prior = if.prior,
        nf.prior = nf.prior,
        ybar = ybar,
        n = n,
        sigma = sigma,
        weight = weight_rMAP
      )
    } else {
      post_c <- .posterior_mix_update(
        prior = nf.prior,
        ybar = ybar,
        n = n,
        sigma = sigma
      )
      post_c$weight <- 0
      post_c
    }
  }

  ## ---- posterior mean of theta given ybar ----
  post_mean_fun <- function(ybar) {
    post_c <- post_c_fun(ybar)
    sum(post_c$w * post_c$mu)
  }

  ## ---- borrowing weight given ybar ----
  weight_fun_eval <- function(ybar) {
    if (method == "SAM") {
      weight_fun_normmix(
        ybar = ybar,
        if.prior = if.prior,
        nf.prior = nf.prior,
        n = n,
        sigma = sigma,
        delta = delta,
        method.w = method.w,
        prior.odds = prior.odds
      )
    } else if (method == "rMAP") {
      weight_rMAP
    } else {
      0
    }
  }

  ## ---- posterior probability threshold in ybar_t for fixed ybar ----
  find_ybar_t_cutoff_general <- function(ybar) {
    gfun <- function(ybar_t) {
      ps <- post_summary_cont_2arm(
        ybar_t = ybar_t,
        ybar = ybar,
        if.prior = if.prior,
        nf.prior = nf.prior,
        prior.t = prior.t,
        n.t = n.t,
        n = n,
        sigma.t = sigma.t,
        sigma = sigma,
        delta = delta,
        cutoff = cutoff,
        method = method,
        alternative = alternative,
        margin = margin,
        weight_rMAP = weight_rMAP,
        method.w = method.w,
        prior.odds = prior.odds
      )
      ps$post_prob - cutoff
    }

    gl <- gfun(lower_t)
    gu <- gfun(upper_t)

    if (alternative == "greater") {
      if (gl > 0) return(-Inf)
      if (gu < 0) return( Inf)
    } else {
      if (gl < 0) return(-Inf)
      if (gu > 0) return( Inf)
    }

    uniroot(gfun, interval = c(lower_t, upper_t), tol = 1e-8)$root
  }

  ## ---- 1D reject probability over ybar ----
  reject_integrand <- function(ybar) {
    vapply(ybar, function(x) {
      y_cut <- find_ybar_t_cutoff_general(x)

      pr_reject_given_y <-
        if (is.infinite(y_cut) && y_cut < 0) {
          if (alternative == "greater") 1 else 0
        } else if (is.infinite(y_cut) && y_cut > 0) {
          if (alternative == "greater") 0 else 1
        } else if (alternative == "greater") {
          1 - pnorm(y_cut, mean = theta.t, sd = se_t)
        } else {
          pnorm(y_cut, mean = theta.t, sd = se_t)
        }

      pr_reject_given_y * dnorm(x, mean = theta, sd = se)
    }, numeric(1))
  }

  reject_prob <- integrate(
    f = reject_integrand,
    lower = lower,
    upper = upper,
    rel.tol = rel.tol,
    subdivisions = 200L
  )$value

  ## ---- 1D control moments for bias/RMSE ----
  mean_integrand <- function(ybar) {
    vapply(ybar, function(x) {
      post_mean_fun(x) * dnorm(x, mean = theta, sd = se)
    }, numeric(1))
  }

  mean_sq_integrand <- function(ybar) {
    vapply(ybar, function(x) {
      post_mean_fun(x)^2 * dnorm(x, mean = theta, sd = se)
    }, numeric(1))
  }

  weight_integrand <- function(ybar) {
    vapply(ybar, function(x) {
      weight_fun_eval(x) * dnorm(x, mean = theta, sd = se)
    }, numeric(1))
  }

  mean_post <- integrate(
    f = mean_integrand,
    lower = lower,
    upper = upper,
    rel.tol = rel.tol,
    subdivisions = 200L
  )$value

  mean_sq_post <- integrate(
    f = mean_sq_integrand,
    lower = lower,
    upper = upper,
    rel.tol = rel.tol,
    subdivisions = 200L
  )$value

  mean_weight <- integrate(
    f = weight_integrand,
    lower = lower,
    upper = upper,
    rel.tol = rel.tol,
    subdivisions = 200L
  )$value

  ## ---- bias and RMSE for theta ----
  bias <- mean_post - theta
  rmse <- sqrt(pmax(mean_sq_post - 2 * theta * mean_post + theta^2, 0))

  data.frame(
    theta = theta,
    theta.t = theta.t,
    delta_true = theta.t - theta,
    method = method,
    alternative = alternative,
    cutoff = cutoff,
    margin = margin,
    reject_prob = reject_prob,
    bias = bias,
    rmse = rmse,
    mean_weight = mean_weight
  )
}

#' Calibrate Posterior Probability Cutoff for a Two-Arm Comparative Trial
#'
#' The \code{calibrate_cutoff_2arm} function is designed to calibrate the
#' posterior probability cutoff for a two-arm comparative trial under the
#' specified endpoint type and borrowing strategy: self-adapting mixture
#' prior (SAM), robust MAP prior with fixed weight (rMAP), or non-informative
#' prior (NP).
#'
#' For binary endpoints, the function dispatches to
#' \code{\link{calibrate_cutoff_bin_2arm}}. For continuous endpoints, it
#' dispatches to \code{\link{calibrate_cutoff_cont_2arm}}.
#'
#' @param if.prior Informative prior constructed from historical data,
#' represented (approximately) as a mixture of conjugate distributions.
#' @param nf.prior Non-informative prior used for the mixture and as the
#' robustifying component for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param target Target rejection probability under the calibration scenario,
#' typically the desired type I error rate when the calibration scenario
#' corresponds to the null hypothesis.
#' @param n.t Sample size for the treatment arm.
#' @param n Sample size for the control arm.
#' @param theta.t The response rate (binary endpoints) or mean
#' (continuous endpoints) for the treatment arm under the calibration scenario.
#' @param theta The response rate (binary endpoints) or mean
#' (continuous endpoints) for the control arm under the calibration scenario.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be non-negative. Default is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param interval Search interval for the posterior probability cutoff. The
#' default is from 0.5 to 0.999.
#' @param rel.tol Tolerance passed to \code{\link[stats]{uniroot}}.
#' @param oc_rel.tol Relative tolerance passed to scenario-level calculations.
#' For continuous endpoints this controls numerical integration in the
#' operating-characteristic engine; for binary endpoints this controls numerical
#' integration in posterior probability calculations.
#' @param ... Additional arguments required by the endpoint-specific method.
#' For continuous endpoints, this includes \code{sigma.t}, \code{sigma}, and
#' optionally \code{n_sd_int}.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{cutoff}{Calibrated posterior probability cutoff.}
#'   \item{objective}{Value of the root-finding objective at the solution.}
#'   \item{target}{Target rejection probability.}
#'   \item{method}{Borrowing method used.}
#'   \item{alternative}{Direction of the posterior decision.}
#'   \item{margin}{Clinical margin used for inference.}
#'   \item{theta}{True control-arm parameter under the calibration scenario.}
#'   \item{theta.t}{True treatment-arm parameter under the calibration scenario.}
#'   \item{interval}{Search interval used for calibration.}
#' }
#'
#' @details
#' The function calibrates the posterior probability cutoff \eqn{C} for a
#' two-arm comparative trial so that the repeated-sampling rejection
#' probability under the specified calibration scenario equals
#' \code{target}. Inference is based on the posterior probability of the
#' treatment effect \eqn{\theta_t - \theta}, where \eqn{\theta_t} and
#' \eqn{\theta} denote the treatment-arm and control-arm parameters,
#' respectively.
#'
#' For \code{alternative = "greater"}, the posterior decision rule is based on
#' \deqn{P(\theta_t - \theta > \Delta \mid D) > C,}
#' where \eqn{\Delta} denotes the clinical margin and \eqn{D} denotes the
#' observed data. Under the default settings, calibration is performed under
#' the null scenario \eqn{\theta_t = \theta = \theta_h}, where
#' \eqn{\theta_h = E(\code{if.prior})} is the posterior mean of the
#' informative prior constructed from historical data. This corresponds to a
#' superiority test when \eqn{\Delta = 0}.
#'
#' For \code{alternative = "less"}, the posterior decision rule is based on
#' \deqn{P(\theta_t - \theta < -\Delta \mid D) > C.}
#' Calibration is again performed under the null scenario
#' \eqn{\theta_t = \theta = \theta_h}, where
#' \eqn{\theta_h = E(\code{if.prior})}. This corresponds to an
#' inferiority test when \eqn{\Delta = 0}.
#'
#' This function acts as a generic front-end and dispatches according to the
#' class of \code{if.prior}. For binary endpoints, it calls
#' \code{\link{calibrate_cutoff_bin_2arm}}. For continuous endpoints, it calls
#' \code{\link{calibrate_cutoff_cont_2arm}}.
#'
#' @seealso \code{\link{calibrate_cutoff_bin_2arm}},
#' \code{\link{calibrate_cutoff_cont_2arm}}
#'
#' @examples
#' ## Example: calibrate the posterior probability cutoff for a two-arm
#' ## binary trial using a SAM prior for the control arm
#'
#' ## Informative prior constructed from historical data
#' if.prior <- mixbeta(c(1, 20, 40))
#'
#' ## Sample sizes
#' n.t <- 100
#' n <- 50
#'
#' ## Calibrate the posterior probability cutoff
#' PPC <- calibrate_cutoff_2arm(
#'   if.prior = if.prior,             ## Informative prior from historical data
#'   nf.prior = mixbeta(c(1, 1, 1)),  ## Non-informative prior for SAM prior
#'   prior.t = mixbeta(c(1, 1, 1)),   ## Prior for treatment arm
#'   target = 0.05,                   ## Targeted type I error rate
#'   n.t = n.t,                       ## Sample size for treatment arm
#'   n = n,                           ## Sample size for control arm
#'   theta.t = summary(if.prior)["mean"],  ## True treatment-arm response rate
#'   theta = summary(if.prior)["mean"],    ## True control-arm response rate
#'   method = "SAM",                  ## Borrowing method
#'   delta = 0.2,                     ## Clinically significant difference
#'   alternative = "greater",         ## Superiority test
#'   margin = 0                       ## Clinical margin
#' )
#'
#' PPC
#'
#'
#' @export
calibrate_cutoff_2arm <- function(if.prior, nf.prior, prior.t = nf.prior,
                                  target = 0.05,
                                  n.t, n,
                                  theta.t = NULL,
                                  theta = NULL,
                                  delta,
                                  method = c("SAM", "rMAP", "NP"),
                                  alternative = c("greater", "less"),
                                  margin = 0,
                                  weight_rMAP = 0.5,
                                  method.w = "LRT",
                                  prior.odds = 1,
                                  interval = c(0.5, 0.999),
                                  rel.tol = 1e-5,
                                  oc_rel.tol = 1e-6,
                                  ...) {
  UseMethod("calibrate_cutoff_2arm")
}

#' @export
calibrate_cutoff_2arm.default <- function(if.prior, nf.prior, prior.t = nf.prior,
                                          target = 0.05,
                                          n.t, n,
                                          theta.t = NULL, theta = NULL,
                                          delta,
                                          method = c("SAM", "rMAP", "NP"),
                                          alternative = c("greater", "less"),
                                          margin = 0,
                                          weight_rMAP = 0.5,
                                          method.w = "LRT",
                                          prior.odds = 1,
                                          interval = c(0.5, 0.999),
                                          rel.tol = 1e-5,
                                          oc_rel.tol = 1e-6,
                                          ...) {
  stop("Unsupported prior class for `calibrate_cutoff_2arm()`.")
}

#' @describeIn calibrate_cutoff_2arm
#' Calibrate the posterior probability cutoff for a two-arm binary endpoint.
#' @export
calibrate_cutoff_2arm.betaMix <- function(if.prior, nf.prior, prior.t = nf.prior,
                                          target = 0.05,
                                          n.t, n,
                                          theta.t = NULL, theta = NULL,
                                          delta,
                                          method = c("SAM", "rMAP", "NP"),
                                          alternative = c("greater", "less"),
                                          margin = 0,
                                          weight_rMAP = 0.5,
                                          method.w = "LRT",
                                          prior.odds = 1,
                                          interval = c(0.5, 0.999),
                                          rel.tol = 1e-5,
                                          oc_rel.tol = 1e-8,
                                          ...) {
  calibrate_cutoff_bin_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    theta.t = theta.t,
    theta = theta,
    delta = delta,
    method = method,
    alternative = alternative,
    margin = margin,
    weight_rMAP = weight_rMAP,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol
  )
}

#' @describeIn calibrate_cutoff_2arm
#' Calibrate the posterior probability cutoff for a two-arm continuous endpoint.
#' @param sigma.t Known sampling standard deviation in the treatment arm.
#' @param sigma Known sampling standard deviation in the control arm.
#' @param n_sd_int Half-width of the numerical integration region for each arm,
#' expressed as a multiple of the corresponding standard error.
#' @export
calibrate_cutoff_2arm.normMix <- function(if.prior, nf.prior, prior.t = nf.prior,
                                          target = 0.05,
                                          n.t, n,
                                          theta.t = NULL, theta = NULL,
                                          delta,
                                          method = c("SAM", "rMAP", "NP"),
                                          alternative = c("greater", "less"),
                                          margin = 0,
                                          weight_rMAP = 0.5,
                                          method.w = "LRT",
                                          prior.odds = 1,
                                          interval = c(0.5, 0.999),
                                          rel.tol = 1e-5,
                                          oc_rel.tol = 1e-6,
                                          sigma.t,
                                          sigma,
                                          n_sd_int = 8,
                                          ...) {
  if (missing(sigma.t)) {
    stop("`sigma.t` must be provided for continuous endpoints.")
  }
  if (missing(sigma)) {
    stop("`sigma` must be provided for continuous endpoints.")
  }

  calibrate_cutoff_cont_2arm(
    if.prior = if.prior,
    nf.prior = nf.prior,
    prior.t = prior.t,
    target = target,
    n.t = n.t,
    n = n,
    sigma.t = sigma.t,
    sigma = sigma,
    theta.t = theta.t,
    theta = theta,
    delta = delta,
    method = method,
    alternative = alternative,
    margin = margin,
    weight_rMAP = weight_rMAP,
    method.w = method.w,
    prior.odds = prior.odds,
    interval = interval,
    rel.tol = rel.tol,
    oc_rel.tol = oc_rel.tol,
    n_sd_int = n_sd_int
  )
}

## internal cache for pairwise beta-beta posterior probabilities
.bin_postprob_cache <- new.env(parent = emptyenv())

.beta_mean_vec <- function(a, b) a / (a + b)

.beta_var_vec <- function(a, b) a * b / ((a + b)^2 * (a + b + 1))

## Computes P(theta_t - theta > margin | data)
.post_prob_pair_bin <- function(a.t, b.t, a.c, b.c, margin, rel.tol) {
  key <- paste(a.t, b.t, a.c, b.c, format(margin, digits = 16),
               format(rel.tol, digits = 16), sep = "|")

  if (exists(key, envir = .bin_postprob_cache, inherits = FALSE)) {
    return(get(key, envir = .bin_postprob_cache, inherits = FALSE))
  }

  lower <- max(0, margin)
  upper <- 1

  val <- if (lower >= upper) {
    0
  } else {
    integrand <- function(pt) {
      pbeta(pt - margin, shape1 = a.c, shape2 = b.c) *
        dbeta(pt, shape1 = a.t, shape2 = b.t)
    }

    integrate(
      f = integrand,
      lower = lower,
      upper = upper,
      rel.tol = rel.tol,
      subdivisions = 200L
    )$value
  }

  assign(key, val, envir = .bin_postprob_cache)
  val
}

## Computes P(theta_t - theta > margin | data) from posterior mixtures
.post_prob_bin_from_posts <- function(post_t, post_c, margin, rel.tol) {
  out <- 0

  for (j in seq_along(post_t$w)) {
    wt_j <- post_t$w[j]
    a_tj <- post_t$a[j]
    b_tj <- post_t$b[j]

    for (k in seq_along(post_c$w)) {
      out <- out + wt_j * post_c$w[k] *
        .post_prob_pair_bin(
          a.t = a_tj,
          b.t = b_tj,
          a.c = post_c$a[k],
          b.c = post_c$b[k],
          margin = margin,
          rel.tol = rel.tol
        )
    }
  }

  out
}

#' Posterior Summary for Two-Arm Comparative Trial with Binary Endpoint
#'
#' The \code{post_summary_bin_2arm} function is designed to compute the
#' posterior summary for the treatment effect in a two-arm comparative trial
#' with a binary endpoint under one of three borrowing strategies:
#' self-adapting mixture prior (SAM), robust MAP prior with fixed weight
#' (rMAP), or non-informative prior (NP).
#'
#' The treatment effect is defined as \eqn{\tau = \theta_t - \theta}, where
#' \eqn{\theta_t} and \eqn{\theta} denote the response rates in the treatment
#' and control arms, respectively. Inference is based on the posterior
#' distribution of \eqn{\tau} given the observed response counts from the two
#' arms.
#'
#' @param x.t Observed number of responses in the treatment arm.
#' @param x Observed number of responses in the control arm.
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a beta mixture prior.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param n.t Sample size for the treatment arm.
#' @param n Sample size for the control arm.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param cutoff Posterior probability cutoff used for decision making.
#' The null hypothesis is rejected if the posterior tail probability
#' exceeds \code{cutoff}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param rel.tol Relative tolerance for numerical integration used to evaluate
#' the posterior probability.
#'
#' @details
#' The posterior for the treatment arm is obtained by updating
#' \code{prior.t} using the observed response count \code{x.t}. The posterior
#' for the control arm depends on the selected borrowing strategy:
#'
#' \itemize{
#' \item \code{SAM}: the prior for the control arm is a mixture of
#' \code{if.prior} and \code{nf.prior}, with adaptive mixture weight
#' determined by the SAM borrowing rule.
#' \item \code{rMAP}: the prior for the control arm is a fixed mixture of
#' \code{if.prior} and \code{nf.prior}, with fixed weight
#' \code{weight_rMAP}.
#' \item \code{NP}: the prior for control arm is \code{nf.prior}.
#' }
#'
#' When \code{alternative = "greater"}, inference is based on
#' \eqn{P(\theta_t - \theta > margin \mid X_t, X)}. When
#' \code{alternative = "less"}, inference is based on
#' \eqn{P(\theta_t - \theta < -margin \mid X_t, X)}.
#'
#' The posterior mean and posterior variance of \eqn{\tau} are defined as
#' \deqn{E(\tau \mid X_t, X) = E(\theta_t \mid X_t) - E(\theta \mid X),}
#' and
#' \deqn{\mathrm{Var}(\tau \mid X_t, X) = \mathrm{Var}(\theta_t \mid X_t) + \mathrm{Var}(\theta \mid X),}
#' where independence between the treatment and control arm posteriors is
#' assumed conditional on the current and historical data.
#'
#' @return A list containing the posterior probability in the requested
#' direction, posterior mean and variance of \eqn{\tau},
#' decision indicator, borrowing weight used for the control arm prior,
#' and the corresponding trial data and method information.
#'
#' @seealso \code{\link{SAM_weight}}, \code{\link{SAM_prior}}
#'
#' @export
post_summary_bin_2arm <- function(x.t, x,
                                  if.prior, nf.prior, prior.t = nf.prior,
                                  n.t, n,
                                  delta,
                                  cutoff,
                                  method = c("SAM", "rMAP", "NP"),
                                  alternative = c("greater", "less"),
                                  margin = 0,
                                  weight_rMAP = 0.5,
                                  method.w = "LRT",
                                  prior.odds = 1,
                                  rel.tol = 1e-8) {
  method <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!is.numeric(x.t) || length(x.t) != 1 || !is.finite(x.t) || x.t < 0 || x.t > n.t) {
    stop("`x.t` must be a scalar between 0 and `n.t`.")
  }
  if (!is.numeric(x) || length(x) != 1 || !is.finite(x) || x < 0 || x > n) {
    stop("`x` must be a scalar between 0 and `n`.")
  }
  if (!is.numeric(cutoff) || length(cutoff) != 1 || cutoff <= 0 || cutoff >= 1) {
    stop("`cutoff` must be a scalar in (0, 1).")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (!is.numeric(n.t) || length(n.t) != 1 || !is.finite(n.t) || n.t <= 0) {
    stop("`n.t` must be a positive scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  if (!is.numeric(rel.tol) || length(rel.tol) != 1 ||
      !is.finite(rel.tol) || rel.tol <= 0) {
    stop("`rel.tol` must be a positive scalar.")
  }

  post_t <- .posterior_mixbeta_update(
    prior = prior.t,
    x = x.t,
    n = n.t
  )

  if (method == "SAM") {
    post_c <- .control_sam_update_bin(
      if.prior = if.prior,
      nf.prior = nf.prior,
      x = x,
      n = n,
      delta = delta,
      weight_fun = weight_fun_betamix,
      method.w = method.w,
      prior.odds = prior.odds
    )
    weight_used <- post_c$wSAM

  } else if (method == "rMAP") {
    if (is.null(weight_rMAP) || length(weight_rMAP) != 1 ||
        !is.finite(weight_rMAP) || weight_rMAP < 0 || weight_rMAP > 1) {
      stop("`weight_rMAP` must be a scalar in [0, 1] when `method = \"rMAP\"`.")
    }

    post_c <- .control_fixed_update_bin(
      if.prior = if.prior,
      nf.prior = nf.prior,
      x = x,
      n = n,
      weight = weight_rMAP
    )
    weight_used <- weight_rMAP

  } else {
    post_c <- .posterior_mixbeta_update(
      prior = nf.prior,
      x = x,
      n = n
    )
    weight_used <- 0
  }

  post_prob_greater <- .post_prob_bin_from_posts(
    post_t = post_t,
    post_c = post_c,
    margin = margin,
    rel.tol = rel.tol
  )

  post_prob <- if (alternative == "greater") {
    post_prob_greater
  } else {
    1 - .post_prob_bin_from_posts(
      post_t = post_t,
      post_c = post_c,
      margin = -margin,
      rel.tol = rel.tol
    )
  }

  mean_t <- sum(post_t$w * .beta_mean_vec(post_t$a, post_t$b))
  var_t  <- sum(post_t$w * (.beta_var_vec(post_t$a, post_t$b) +
                              .beta_mean_vec(post_t$a, post_t$b)^2)) - mean_t^2

  mean_c <- sum(post_c$w * .beta_mean_vec(post_c$a, post_c$b))
  var_c  <- sum(post_c$w * (.beta_var_vec(post_c$a, post_c$b) +
                              .beta_mean_vec(post_c$a, post_c$b)^2)) - mean_c^2

  post_mean <- mean_t - mean_c
  post_var  <- var_t + var_c

  list(
    post_prob = post_prob,
    post_prob_greater = post_prob_greater,
    post_mean = post_mean,
    post_var = post_var,
    decision = as.numeric(post_prob > cutoff),
    weight = weight_used,
    method = method,
    alternative = alternative,
    margin = margin,
    x.t = x.t,
    x = x
  )
}

#' Calculating Mixture Weight of SAM Priors
#'
#' The \code{SAM_weight} function is designed to calculate the mixture
#' weight of the SAM priors according to the degree of prior-data
#' conflicts (\emph{Yang, et al., 2023}).
#'
#' @param if.prior Informative prior constructed based on historical data,
#' represented (approximately) as a mixture of conjugate distributions.
#' @param theta.h Estimate of the treatment effect based on historical data.
#' If missing, the default value is set to be the posterior mean estimate from
#' \code{if.prior}.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See Details section for more information.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See Details section for more information.
#' @param data Data of the control arm from the current trial, see Methods
#' section for more details.
#' @param delta Clinically significant difference used for the SAM prior.
#' @param ... Additional parameters required for different endpoints.
#'
#' @details SAM prior is constructed by mixing an informative prior
#' \eqn{\pi_1(\theta)}, constructed based on historical data, with a
#' non-informative prior \eqn{\pi_0(\theta)} using the mixture weight
#' \eqn{w} determined by \code{\link{SAM_weight}} function to achieve the
#' degree of prior-data conflict (\emph{Schmidli et al., 2015, Yang et al., 2023}).
#'
#'  Let \eqn{\theta} and \eqn{\theta_h} denote the treatment effects
#'  associated with the current arm data \eqn{D} and historical data \eqn{D_h},
#'  respectively. Let \eqn{\delta} denote the clinically significant difference
#'  such that if \eqn{|\theta_h - \theta| \ge \delta}, then \eqn{\theta_h} is
#'  regarded as clinically distinct from \eqn{\theta}, and it is therefore
#'  inappropriate to borrow any information from \eqn{D_h}. Consider two
#'  hypotheses:
#'
#'  \deqn{H_0: \theta = \theta_h, ~ H_1: \theta = \theta_h + \delta ~ or ~  \theta = \theta_h - \delta.}
#'  \eqn{H_0} represents that \eqn{D_h} and \eqn{D} are consistent (i.e.,
#'  no prior-data conflict) and thus information borrowing is desirable,
#'  whereas \eqn{H_1} represents that the treatment effect of \eqn{D}
#'  differs from \eqn{D_h} to such a degree that no information should be
#'  borrowed.
#'
#'  The SAM prior uses the likelihood ratio test (LRT) statistics \eqn{R} to
#'  quantify the degree of prior-data conflict and determine the extent of
#'  information borrowing.
#'
#'  \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(D | \theta = \theta_h) / \max(P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta)) ,}
#'  where \eqn{P(D | \cdot)} denotes the likelihood function. An alternative
#'  Bayesian choice is the posterior probability ratio (PPR):
#'
#'  \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(H_0) / P( H_1) \times BF, }
#'  where \eqn{P(H_0)} and \eqn{P(H_1)} is the prior probabilities of \eqn{H_0}
#'  and \eqn{H_1} being true. \eqn{BF} is the Bayes Factor that in this case
#'  is the same as the LRT.
#'
#'  The SAM prior, denoted as \eqn{\pi_{sam}(\theta)}, is then defined
#'  as a mixture of an informative prior \eqn{\pi_1(\theta)}, constructed
#'  based on \eqn{D_h} and a non-informative prior \eqn{\pi_0(\theta)}:
#'
#'  \deqn{\pi_{sam}(\theta) = w\pi_1(\theta) + (1-w)\pi_0(\theta),}
#'  where the mixture weight \eqn{w} is calculated as:
#'
#'  \deqn{w = R / (1 + R).}
#'
#'  As the level of prior-data conflict increases, the likelihood ratio
#'  \eqn{R} decreases, resulting in a decrease in the weight \eqn{w}
#'  assigned to the informative prior and thus a decrease in information
#'  borrowing. As a result, \eqn{\pi_{sam}(\theta)} is data-driven and
#'  has the ability to self-adapt the information borrowing based on the
#'  degree of prior-data conflict.
#'
#' @return The mixture weight of the SAM priors.
#'
#' @references Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
#' SAM: Self-adapting mixture prior to dynamically borrow information from
#' historical data in clinical trials. \emph{Biometrics} 2023; 79(4), 2857-2868.
#'
#' @examples
#' set.seed(123)
#' ## Examples for binary endpoints
#' ## Example 1: no prior-data conflict
#' ## Suppose that the informative prior constructed based on historical data is
#' ## beta(40, 60)
#' prior.historical <- mixbeta(c(1, 40, 60))
#' ## Data of control arm
#' data.control     <- rbinom(60, size = 1, prob = 0.42)
#' ## Calculate the mixture weight of the SAM prior
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.15,        ## Clinically significant difference
#'                    data = data.control  ## Control arm data
#'                    )
#' print(wSAM)
#'
#' ## Example 2: in the presence of prior-data conflict, where the current data
#' ## has 12 responses in 60 patients
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.15,    ## Clinically significant difference
#'                    ## Methods to determine mixture weight for the SAM priors
#'                    ## by Posterior Probability Ratio
#'                    method.w = 'PPR',
#'                    ## Prior odds of favoring no prior-data conflicts to
#'                    ## the presence of prior-data conflict
#'                    prior.odd = 1/9,
#'                    n = 60,          ## Number of patients in the control arm
#'                    r = 12           ## Number of responses in the control arm
#'                    )
#' print(wSAM)
#'
#' ## Example 3: in the presence of prior-data conflict, where the current data
#' ## has 12 responses in 60 patients
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.15, ## Clinically significant difference
#'                    n = 60,       ## Number of patients in the control arm
#'                    r = 12        ## Number of responses in the control arm
#'                    )
#' print(wSAM)
#'
#' ## Examples for continuous endpoints
#' ## Example 1: no prior-data conflict
#' ## Suppose that the informative prior constructed from historical data is
#' ## N(0, 3)
#' sigma      <- 3
#' prior.mean <- 0
#' prior.se   <- sigma/sqrt(100)
#' prior.historical <- mixnorm(c(1, prior.mean, prior.se), sigma = sigma)
#' ## Data of the control arm
#' data.control     <- rnorm(80, mean = 0, sd = sigma)
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.3 * sigma,    ## Clinically significant difference
#'                    data = data.control     ## Control arm data
#'                    )
#' print(wSAM)
#'
#' ## Example 2: in the presence of prior-data conflict, where the current data
#' ## has mean of 0.5
#' data.control     <- rnorm(80, mean = 1, sd = sigma)
#' wSAM  <- SAM_weight(if.prior = prior.historical,
#'                     delta = 0.3 * sigma,    ## Clinically significant difference
#'                     data = data.control     ## Control arm data
#'                     )
#' print(wSAM)
#'
#' ## Examples for survival endpoints
#' ## Example 1: no prior-data conflict
#' ## Suppose the survival times from historical data follows exp(1) distribution
#' ## with random censoring time follows U(0.5, 5) distribution
#' T_hi <- rexp(100, rate = 1)
#' C_hi <- runif(100, min = 0.5, max = 5)
#' ## Indicators of the uncensored events
#' delta_hi <- as.numeric(T_hi < C_hi)
#' ## Observed survival times from historical data
#' U_hi     <- T_hi
#' U_hi[delta_hi == 0] <- C_hi[delta_hi == 0]
#' ## Construct the informative prior based on simulated historical data
#' prior.historical <- mixgamma(c(1, sum(delta_hi), sum(U_hi)),
#'                              param = 'ab', likelihood = 'exp')
#' ## Suppose the survival times from control data follows exp(0.95) distribution
#' ## with random censoring time follows U(0.5, 5) distribution
#' T_ci <- rexp(100, rate = 0.95)
#' C_ci <- runif(100, min = 0.5, max = 5)
#' ## Indicators of the uncensored events
#' delta_ci <- as.numeric(T_ci < C_ci)
#' ## Observed survival times from control data
#' U_ci     <- T_ci
#' U_ci[delta_ci == 0] <- C_ci[delta_ci == 0]
#' ## Data of the control arm
#' data.control     <- rbind(sum(delta_ci), sum(U_ci))
#' wSAM <- SAM_weight(if.prior = prior.historical,
#'                    delta = 0.2,            ## Clinically significant difference
#'                    data = data.control     ## Control arm data
#'                    )
#' print(wSAM)
#'
#' ## Example 2: in the presence of prior-data conflict, where the current survival
#' ## times follows exp(2) distribution with random censoring time follows U(0.5, 5)
#' ## distribution
#' T_ci <- rexp(100, rate = 2)
#' C_ci <- runif(100, min = 0.5, max = 5)
#' ## Indicators of the uncensored events
#' delta_ci <- as.numeric(T_ci < C_ci)
#' ## Observed survival times from control data
#' U_ci     <- T_ci
#' U_ci[delta_ci == 0] <- C_ci[delta_ci == 0]
#' ## Data of the control arm
#' data.control     <- rbind(sum(delta_ci), sum(U_ci))
#' wSAM  <- SAM_weight(if.prior = prior.historical,
#'                     delta = 0.2,            ## Clinically significant difference
#'                     data = data.control     ## Control arm data
#'                     )
#' print(wSAM)
#'
#' @import Metrics
#' @import RBesT
#' @import assertthat
#' @import checkmate
#' @import ggplot2
#' @import stats
#' @export
SAM_weight <- function(if.prior, theta.h, method.w, prior.odds, data, delta, ...) UseMethod("SAM_weight")

#' @export
SAM_weight.default <- function(if.prior, theta.h, method.w, prior.odds, data, delta, ...) "Unknown distribution"

#' @describeIn SAM_weight The function calculates the mixture weight of SAM
#' priors for beta mixture distribution. The input \code{data} can be patient-level
#' data (i.e., a vector of 0 and 1 representing the response status of each
#' patient) or summary statistics (i.e., the number of patients and the number
#' of responses).
#' @param n Number of subjects in the control arm for binary endpoint.
#' @param r Number of responses in the control arm for binary endpoint.
#' @export
SAM_weight.betaMix <- function(if.prior, theta.h, method.w, prior.odds, data, delta, n, r, ...) {
  if(!missing(data)) {
    assertthat::assert_that(all(data %in% c(0,1)))
    r <- sum(data)
    n <- length(data)
  }
  if(!missing(method.w)){
    assertthat::assert_that(all(method.w %in% c('LRT', 'PPR')))
    assertthat::assert_that(length(method.w) == 1)
  }

  if(missing(theta.h)){
    message("Using the posterior mean from informative prior as the estimate of the treatment effect based on historical data.")
    theta.h <- summary(if.prior)['mean']
  }

  if(missing(method.w)){
    message("Using the LRT (Likelihood Ratio Test) as the default method to calculate mixture weight for SAM priors.")
    method.w = 'LRT'
  }

  if(method.w == 'PPR' & missing(prior.odds)){
    message("Missing the prior odds, set as 1.")
    prior.odds = 1
  }

  ## The posterior mean estimation from the MAP prior
  theta_h_hat <- theta.h

  ## Calculate the weight for SAM prior
  R <- max(dbinom(x = r, size = n, prob = min(theta_h_hat + delta, 0.99)),
           dbinom(x = r, size = n, prob = max(theta_h_hat - delta, 0.01)))

  R <- R / dbinom(x = r, size = n, prob = theta_h_hat)

  if(method.w  == 'PPR') R = R / prior.odds

  ## SAM weight
  w = 1 / (1 + R)

  return(w)
}


#' @describeIn SAM_weight The function calculates the mixture weight of SAM
#' priors for normal mixture distribution. The input \code{data} should be
#' a vector of patient-level observations. The input \code{data} can be
#' patient-level data (i.e., a vector of continuous response of each
#' patient) or summary statistics (i.e., the mean estimate, number of subjects,
#' and the standard deviation in the control arm).
#' @param m Mean estimate in the control arm for continuous endpoint.
#' @param n Number of subjects in the control arm for continuous endpoint.
#' @param sigma Standard deviation in the control arm for continuous endpoint.
#' @export
SAM_weight.normMix <- function(if.prior, theta.h, method.w, prior.odds, data, delta, m, n, sigma, ...) {

  if(!missing(data)) {
    m <- mean(data)
    n <- length(data)
    if(n == 1 & missing(sigma)){
      stop("Standard deviation in the control arm must be given.")
    }else if(n > 1 & missing(sigma)){
        sigma <- sd(data)
    }
    ## Try to see if the summary data is provided
    if_summary <- FALSE
  } else {
    # if(missing(data)){
    #   stop("Individual data must be given.")
    # }
    if_summary = TRUE
  }

  if(!missing(method.w)){
    assertthat::assert_that(all(method.w %in% c('LRT', 'PPR')))
    assertthat::assert_that(length(method.w) == 1)
  }

  if(missing(theta.h)){
    message("Using the posterior mean from informative prior as the estimate of the treatment effect based on historical data.")
    theta.h <- summary(if.prior)['mean']
  }

  if(missing(method.w)){
    message("Using the LRT (Likelihood Ratio Test) as the default method to calculate mixture weight for SAM priors.")
    method.w = 'LRT'
  }

  if(method.w == 'PPR' & missing(prior.odds)){
    message("Missing the prior odds, set as 1.")
    prior.odds = 1
  }

  ## The posterior mean estimation from the MAP prior
  theta_h_hat <- theta.h

  ## Calculate the weight for SAM prior
  if(!if_summary){
    ## Based on individual data
    R1 <- sum(dnorm(data, mean = theta_h_hat - delta, sigma, log = T) - dnorm(data, mean = theta_h_hat, sigma, log = T))
    R2 <- sum(dnorm(data, mean = theta_h_hat + delta, sigma, log = T) - dnorm(data, mean = theta_h_hat, sigma, log = T))
    R  <- exp(max(R1, R2))
  }else{
    ## Based on summary data
    R1 <- -1/2 * ( (n*delta * (delta - 2 * m + 2 * theta_h_hat) ) / sigma^2 )
    R2 <- -1/2 * ( (n*delta * (delta + 2 * m - 2 * theta_h_hat) ) / sigma^2 )
    R  <- 1/exp(-max(R1, R2))
  }

  if(method.w  == 'PPR') R = R / prior.odds

  ## SAM weight
  w = 1 / (1 + R)


  return(w)
}


#' @describeIn SAM_weight The function calculates the mixture weight of SAM
#' priors for gamma mixture distribution. The input \code{data} can be
#' patient-level data (i.e., a matrix with the first row as the censoring
#' indicator and the second row recording the observed time) or summary
#' statistics (i.e., the number of uncensored observations \code{u} and
#' total observed time \code{w}).
#' @param u Number of events in the control arm for time-to-event endpoint.
#' @param w Total observed time in the control arm for time-to-event endpoint.
#' @export
SAM_weight.gammaMix <- function(if.prior, theta.h, method.w, prior.odds, data, delta, u, w, ...) {
  if(!missing(data)) {
    u <- sum(data[1,])
    w <- sum(data[2,])
  } else {
    if(missing(u) | missing(w)){
      stop("Individual or summary data must be given.")
    }
  }

  if(!missing(method.w)){
    assertthat::assert_that(all(method.w %in% c('LRT', 'PPR')))
    assertthat::assert_that(length(method.w) == 1)
  }

  if(missing(theta.h)){
    message("Using the posterior mean from informative prior as the estimate of the treatment effect based on historical data.")
    theta.h <- summary(if.prior)['mean']
  }

  if(missing(method.w)){
    message("Using the LRT (Likelihood Ratio Test) as the default method to calculate mixture weight for SAM priors.")
    method.w = 'LRT'
  }

  if(method.w == 'PPR' & missing(prior.odds)){
    message("Missing the prior odds, set as 1.")
    prior.odds = 1
  }

  ## The posterior mean estimation from the MAP prior
  hazard_h_hat <- theta.h

  ## The posterior mean estimation from the MAP prior
  # hazard_h_hat <- summary(if.prior)[theta.h]

  ## Calculate the weight for SAM prior
  R <- max(dgamma(hazard_h_hat + delta, shape = u + 1, rate = w),
           dgamma(hazard_h_hat - delta, shape = u + 1, rate = w))

  R <- R / dgamma(hazard_h_hat, shape = u + 1, rate = w)

  if(method.w  == 'PPR') R = R / prior.odds

  ## SAM weight
  w = 1 / (1 + R)


  return(w)
}

#' Extract Components from a Normal Mixture Prior
#'
#' Internal helper function to extract mixture weights, component means,
#' and component standard deviations from a normal mixture prior object.
#'
#' The function assumes the mixture prior stores its component matrix in
#' \code{prior[[1]]}, with row names \code{"w"}, \code{"m"}, and \code{"s"}
#' corresponding to the mixture weights, component means, and component
#' standard deviations, respectively.
#'
#' @param prior A normal mixture prior object, such as one created by
#' \code{\link[SAMprior]{mixnorm}} or returned by related functions in the
#' \pkg{SAMprior} workflow.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Numeric vector of normalized mixture weights.}
#'   \item{mu}{Numeric vector of component means.}
#'   \item{sd}{Numeric vector of component standard deviations.}
#'   \item{K}{Number of mixture components.}
#' }
#'
#' @details
#' Missing component entries are removed before returning the extracted
#' quantities. The mixture weights are normalized to sum to 1.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.extract_mixnorm <- function(prior) {
  comp <- prior[[1]]
  
  if (is.null(comp)) {
    stop("`prior[[1]]` is NULL.")
  }
  
  comp <- as.matrix(comp)
  
  rn <- rownames(comp)
  if (is.null(rn)) {
    stop("`prior[[1]]` must have row names 'w', 'm', and 's'.")
  }
  
  w_idx <- match("w", rn)
  m_idx <- match("m", rn)
  s_idx <- match("s", rn)
  
  if (any(is.na(c(w_idx, m_idx, s_idx)))) {
    stop("Could not find rows 'w', 'm', 's' in `prior[[1]]`.")
  }
  
  w  <- as.numeric(comp[w_idx, , drop = TRUE])
  mu <- as.numeric(comp[m_idx, , drop = TRUE])
  sd <- as.numeric(comp[s_idx, , drop = TRUE])
  
  keep <- !(is.na(w) | is.na(mu) | is.na(sd))
  w  <- w[keep]
  mu <- mu[keep]
  sd <- sd[keep]
  
  if (length(w) == 0) stop("No mixture components found.")
  if (any(sd <= 0)) stop("All component SDs must be positive.")
  
  w <- w / sum(w)
  
  list(
    w  = w,
    mu = mu,
    sd = sd,
    K  = length(w)
  )
}

#' Update a Normal Prior Component
#'
#' Internal helper function to update a single normal prior component using
#' conjugate normal-normal updating based on an observed sample mean.
#'
#' Suppose
#' \deqn{\theta \sim N(\mu, sd^2)}
#' and
#' \deqn{\bar{Y} \mid \theta \sim N(\theta, \sigma^2 / n).}
#' Then this function returns the posterior mean, posterior variance, and the
#' marginal density of \eqn{\bar{Y}} under the prior component.
#'
#' @param mu Prior mean of the normal component.
#' @param sd Prior standard deviation of the normal component.
#' @param ybar Observed sample mean.
#' @param n Sample size corresponding to \code{ybar}.
#' @param sigma Known sampling standard deviation.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{mean}{Posterior mean under the component.}
#'   \item{var}{Posterior variance under the component.}
#'   \item{marg}{Marginal density of \code{ybar} under the component.}
#' }
#'
#' @details
#' The marginal density of \code{ybar} is computed under the implied sampling
#' model
#' \deqn{\bar{Y} \sim N(\mu, sd^2 + \sigma^2 / n).}
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.normal_update <- function(mu, sd, ybar, n, sigma) {
  v_prior <- sd^2
  v_lik   <- sigma^2 / n
  
  v_post <- 1 / (1 / v_prior + 1 / v_lik)
  m_post <- v_post * (mu / v_prior + ybar / v_lik)
  
  m_y <- dnorm(ybar, mean = mu, sd = sqrt(v_prior + v_lik))
  
  list(mean = m_post, var = v_post, marg = m_y)
}

#' Update a Normal Mixture Prior
#'
#' Internal helper function to update all components of a normal mixture prior
#' using conjugate normal-normal updating based on an observed sample mean.
#'
#' Each mixture component is updated separately, and the posterior mixture
#' weights are obtained by Bayes theorem using the component-specific marginal
#' densities of the observed sample mean.
#'
#' @param prior A normal mixture prior object.
#' @param ybar Observed sample mean.
#' @param n Sample size corresponding to \code{ybar}.
#' @param sigma Known sampling standard deviation.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{w}{Posterior mixture weights.}
#'   \item{mu}{Posterior means of the updated mixture components.}
#'   \item{var}{Posterior variances of the updated mixture components.}
#'   \item{marg}{Marginal densities of \code{ybar} under the prior components.}
#' }
#'
#' @details
#' Let the prior be represented as a finite mixture of normal components. Each
#' component is updated using \code{.normal_update()}, and the posterior mixture
#' weights are proportional to the prior mixture weights times the corresponding
#' marginal densities of \code{ybar}.
#'
#' This is an internal utility and is not intended for direct user access.
#'
#' @noRd
.posterior_mix_update <- function(prior, ybar, n, sigma) {
  mix <- .extract_mixnorm(prior)
  
  upd <- lapply(seq_len(mix$K), function(j) {
    .normal_update(mix$mu[j], mix$sd[j], ybar = ybar, n = n, sigma = sigma)
  })
  
  marg <- vapply(upd, `[[`, numeric(1), "marg")
  post_w_unnorm <- mix$w * marg
  den <- sum(post_w_unnorm)
  
  if (!is.finite(den) || den <= 0) {
    stop("Posterior weight normalization failed in `.posterior_mix_update()`.")
  }
  
  post_w <- post_w_unnorm / den
  
  post_mean <- vapply(upd, `[[`, numeric(1), "mean")
  post_var  <- vapply(upd, `[[`, numeric(1), "var")
  
  list(
    w = post_w,
    mu = post_mean,
    var = post_var,
    marg = marg
  )
}

#' Evaluate One Scenario for a Two-Arm Comparative Trial with Binary Endpoint
#'
#' The \code{eval_scenario_bin_2arm} function is designed to evaluate
#' repeated-sampling operating characteristics for a two-arm comparative trial
#' with a binary endpoint under one borrowing strategy: self-adapting mixture
#' prior (SAM), robust MAP prior with fixed weight (rMAP), or non-informative
#' prior (NP).
#'
#' The treatment effect is defined as \eqn{\tau = \theta_t - \theta},
#' where \eqn{\theta_t} and \eqn{\theta} denote the true response rates in the
#' treatment and control arms, respectively.
#'
#' For a given true scenario \eqn{(\theta_t, \theta)}, this function computes
#' the repeated-sampling rejection probability, bias, root mean squared error
#' (RMSE), and mean borrowing weight. The rejection probability is accelerated
#' by exploiting monotonicity of the posterior decision in the treatment-arm
#' response count for each fixed control-arm response count.
#'
#' @param if.prior Informative prior constructed based on historical data for
#' the control arm, represented (approximately) as a beta mixture prior.
#' @param nf.prior Non-informative prior used as the robustifying component
#' for the control arm prior.
#' @param prior.t Prior used for the treatment arm. If missing, the default
#' value is set to be \code{nf.prior}.
#' @param n.t Number of subjects in the treatment arm.
#' @param n Number of subjects in the control arm.
#' @param theta.t True treatment arm response rate.
#' @param theta True control arm response rate.
#' @param cutoff Posterior probability cutoff used for decision making.
#' Rejection occurs if the posterior probability exceeds \code{cutoff}.
#' @param delta Clinically significant difference used for the SAM prior.
#' This argument is only used when \code{method = "SAM"}.
#' @param method Borrowing strategy for the control arm. Must be one of
#' \code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.
#' @param alternative Direction of the posterior decision. Must be one of
#' \code{"greater"} (for superiority) or \code{"less"} (for inferiority).
#' @param margin Clinical margin. Must be a non-negative scalar. The default
#' value is \code{0}.
#' @param weight_rMAP Weight assigned to the informative prior component
#' (\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
#' This argument is only used when \code{method = "rMAP"}. The default value is
#' 0.5.
#' @param method.w Methods used to determine the mixture weight for SAM priors.
#' The default method is "LRT" (Likelihood Ratio Test), the alternative option
#' is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
#' more details.
#' @param prior.odds The prior probability of \eqn{H_0} being true compared to
#' the prior probability of \eqn{H_1} being true using PPR method. The default
#' value is 1. See \code{\link{SAM_weight}} for more details.
#' @param rel.tol Relative tolerance for numerical integration used in
#' posterior probability calculations.
#'
#' @return A one-row data frame with the following columns:
#' \describe{
#'   \item{theta}{True control arm response rate.}
#'   \item{theta.t}{True treatment arm response rate.}
#'   \item{delta_true}{True treatment effect, \eqn{\tau = \theta_t - \theta}.}
#'   \item{method}{Borrowing method used.}
#'   \item{alternative}{Direction of the posterior decision.}
#'   \item{cutoff}{Posterior probability cutoff used for decision making.}
#'   \item{margin}{Clinical margin used for inference.}
#'   \item{reject_prob}{Repeated-sampling rejection probability.}
#'   \item{bias}{Bias of the posterior mean estimator of \eqn{\theta}.}
#'   \item{rmse}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
#'   \item{mean_weight}{Average borrowing weight under the specified method.}
#' }
#'
#' @export
eval_scenario_bin_2arm <- function(if.prior, nf.prior, prior.t = nf.prior,
                                   n.t, n,
                                   theta.t, theta,
                                   cutoff,
                                   delta,
                                   method = c("SAM", "rMAP", "NP"),
                                   alternative = c("greater", "less"),
                                   margin = 0,
                                   weight_rMAP = 0.5,
                                   method.w = "LRT",
                                   prior.odds = 1,
                                   rel.tol = 1e-8) {
  method <- match.arg(method)
  alternative <- match.arg(alternative)

  if (!is.numeric(n.t) || length(n.t) != 1 || !is.finite(n.t) || n.t <= 0) {
    stop("`n.t` must be a positive scalar.")
  }
  if (!is.numeric(n) || length(n) != 1 || !is.finite(n) || n <= 0) {
    stop("`n` must be a positive scalar.")
  }
  if (!is.numeric(theta.t) || length(theta.t) != 1 || !is.finite(theta.t) ||
      theta.t < 0 || theta.t > 1) {
    stop("`theta.t` must be a scalar in [0, 1].")
  }
  if (!is.numeric(theta) || length(theta) != 1 || !is.finite(theta) ||
      theta < 0 || theta > 1) {
    stop("`theta` must be a scalar in [0, 1].")
  }
  if (!is.numeric(cutoff) || length(cutoff) != 1 || cutoff <= 0 || cutoff >= 1) {
    stop("`cutoff` must be a scalar in (0, 1).")
  }
  if (!is.numeric(margin) || length(margin) != 1 || !is.finite(margin) || margin < 0) {
    stop("`margin` must be a non-negative scalar.")
  }
  if (!method.w %in% c("LRT", "PPR")) {
    stop("`method.w` must be either \"LRT\" or \"PPR\".")
  }
  if (!is.numeric(prior.odds) || length(prior.odds) != 1 ||
      !is.finite(prior.odds) || prior.odds <= 0) {
    stop("`prior.odds` must be a positive scalar.")
  }
  if (!is.numeric(rel.tol) || length(rel.tol) != 1 ||
      !is.finite(rel.tol) || rel.tol <= 0) {
    stop("`rel.tol` must be a positive scalar.")
  }
  if (method == "rMAP") {
    if (is.null(weight_rMAP) || length(weight_rMAP) != 1 ||
        !is.finite(weight_rMAP) || weight_rMAP < 0 || weight_rMAP > 1) {
      stop("`weight_rMAP` must be a scalar in [0, 1] when `method = \"rMAP\"`.")
    }
  }

  if (!exists(".post_prob_bin_from_posts", mode = "function")) {
    stop("Internal helper `.post_prob_bin_from_posts()` not found. Please source `post_summary_bin_2arm.R` first.")
  }

  ## ---- helper: posterior mean from control posterior object ----
  post_mean_from_control <- function(post_c) {
    mean_comp <- post_c$a / (post_c$a + post_c$b)
    sum(post_c$w * mean_comp)
  }

  ## ---- helper: borrowing weight at x ----
  weight_fun_eval <- function(x) {
    if (method == "SAM") {
      weight_fun_betamix(
        x = x,
        if.prior = if.prior,
        nf.prior = nf.prior,
        n = n,
        delta = delta,
        method.w = method.w,
        prior.odds = prior.odds
      )
    } else if (method == "rMAP") {
      weight_rMAP
    } else {
      0
    }
  }

  ## ---- cache treatment posteriors for all x.t ----
  post_t_list <- lapply(0:n.t, function(xt) {
    .posterior_mixbeta_update(
      prior = prior.t,
      x = xt,
      n = n.t
    )
  })

  ## ---- cache control posteriors for all x ----
  post_c_list <- lapply(0:n, function(xc) {
    if (method == "SAM") {
      .control_sam_update_bin(
        if.prior = if.prior,
        nf.prior = nf.prior,
        x = xc,
        n = n,
        delta = delta,
        weight_fun = weight_fun_betamix,
        method.w = method.w,
        prior.odds = prior.odds
      )
    } else if (method == "rMAP") {
      .control_fixed_update_bin(
        if.prior = if.prior,
        nf.prior = nf.prior,
        x = xc,
        n = n,
        weight = weight_rMAP
      )
    } else {
      .posterior_mixbeta_update(
        prior = nf.prior,
        x = xc,
        n = n
      )
    }
  })

  ## ---- posterior probability from cached posteriors ----
  post_prob_from_posts <- function(post_t, post_c) {
    if (alternative == "greater") {
      .post_prob_bin_from_posts(
        post_t = post_t,
        post_c = post_c,
        margin = margin,
        rel.tol = rel.tol
      )
    } else {
      1 - .post_prob_bin_from_posts(
        post_t = post_t,
        post_c = post_c,
        margin = -margin,
        rel.tol = rel.tol
      )
    }
  }

  ## ---- find treatment-count rejection boundary for each fixed x ----
  find_xt_boundary <- function(post_c) {
    decision_at <- function(xt) {
      post_prob_from_posts(post_t_list[[xt + 1L]], post_c) > cutoff
    }

    dec_low <- decision_at(0)
    dec_high <- decision_at(n.t)

    if (alternative == "greater") {
      if (dec_low) return(-1L)
      if (!dec_high) return(n.t + 1L)

      lo <- 0L
      hi <- n.t
      while (lo + 1L < hi) {
        mid <- (lo + hi) %/% 2L
        if (decision_at(mid)) {
          hi <- mid
        } else {
          lo <- mid
        }
      }
      hi
    } else {
      if (!dec_low) return(-1L)
      if (dec_high) return(n.t + 1L)

      lo <- 0L
      hi <- n.t
      while (lo + 1L < hi) {
        mid <- (lo + hi) %/% 2L
        if (decision_at(mid)) {
          lo <- mid
        } else {
          hi <- mid
        }
      }
      lo
    }
  }

  ## ---- rejection probability using thresholded binomial tails ----
  reject_prob <- 0

  for (x in 0:n) {
    p_x <- dbinom(x, size = n, prob = theta)
    post_c <- post_c_list[[x + 1L]]
    xt_boundary <- find_xt_boundary(post_c)

    pr_reject_given_x <- if (alternative == "greater") {
      if (xt_boundary < 0L) {
        1
      } else if (xt_boundary > n.t) {
        0
      } else {
        pbinom(xt_boundary - 1L, size = n.t, prob = theta.t, lower.tail = FALSE)
      }
    } else {
      if (xt_boundary < 0L) {
        0
      } else if (xt_boundary > n.t) {
        1
      } else {
        pbinom(xt_boundary, size = n.t, prob = theta.t, lower.tail = TRUE)
      }
    }

    reject_prob <- reject_prob + pr_reject_given_x * p_x
  }

  ## ---- exact control-arm moments for bias / RMSE / mean weight ----
  mean_post <- 0
  mean_sq_post <- 0
  mean_weight <- 0

  for (x in 0:n) {
    p_x <- dbinom(x, size = n, prob = theta)
    post_c <- post_c_list[[x + 1L]]
    post_mean_x <- post_mean_from_control(post_c)
    weight_x <- weight_fun_eval(x)

    mean_post <- mean_post + post_mean_x * p_x
    mean_sq_post <- mean_sq_post + post_mean_x^2 * p_x
    mean_weight <- mean_weight + weight_x * p_x
  }

  ## ---- bias and RMSE for theta ----
  bias <- mean_post - theta
  rmse <- sqrt(pmax(mean_sq_post - 2 * theta * mean_post + theta^2, 0))

  data.frame(
    theta = theta,
    theta.t = theta.t,
    delta_true = theta.t - theta,
    method = method,
    alternative = alternative,
    cutoff = cutoff,
    margin = margin,
    reject_prob = reject_prob,
    bias = bias,
    rmse = rmse,
    mean_weight = mean_weight
  )
}

---
title: "SAMprior for Binary Endpoints"
author: "Peng Yang and Ying Yuan"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
  html_vignette:
    toc: true
  html_document:
    toc: true
    number_sections: true
    toc_float:
      collapsed: false
      smooth_scroll: false
  pdf_document:
    toc: true
  word_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{Getting started with SAMprior (binary)}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---
  
```{r, include=FALSE}
library(SAMprior)
library(knitr)
knitr::opts_chunk$set(
    fig.width = 1.62*4,
    fig.height = 4
    )
## setup up fast sampling when run on CRAN
is_CRAN <- !identical(Sys.getenv("NOT_CRAN"), "true")
## NOTE: for running this vignette locally, please uncomment the
## following line:
## is_CRAN <- FALSE
.user_mc_options <- list()
if (is_CRAN) {
    .user_mc_options <- options(RBesT.MC.warmup=250, RBesT.MC.iter=500, RBesT.MC.chains=2, RBesT.MC.thin=1, RBesT.MC.control=list(adapt_delta=0.9))
}
```

# Introduction

The self-adapting mixture prior (SAMprior) package is designed to enhance 
the effectiveness and practicality of clinical trials by leveraging historical 
information or real-world data [1]. The package incorporates historical data 
into a new trial using an informative prior constructed based on historical 
data while mixing a non-informative prior to enhance the robustness of 
information borrowing. It utilizes a data-driven way to determine a self-adapting
mixture weight that dynamically favors the informative (non-informative) prior
component when there is little (substantial) evidence of prior-data conflict.
Operating characteristics are evaluated and compared to the robust 
Meta-Analytic-Predictive (rMAP) prior [2], which assigns a fixed weight of 0.5. 

Consider a randomized clinical trial to compare a treatment with a control in 
patients with ankylosing spondylitis. The primary efficacy endpoint is binary, 
indicating whether a patient achieves 20% improvement at week six according 
to the Assessment of SpondyloArthritis International Society criteria [3].
Nine historical data available to the control were used to construct the 
MAP prior:

```{r,results="asis",echo=FALSE}
## Current trial data
ASAS20 <- data.frame(study = c('Baeten (2013)', 'Deodhar (2016)', 'Deodhar (2019)',
                               'Erdes (2019)', 'Huang (2019)', 'Kivitz (2018)',
                               'Pavelka (2017)', 'Sieper (2017)', 'Van der Heijde (2018)'),
                     n = c(6, 122, 104, 23, 153, 117, 76, 74, 87),
                     r = c(1, 35, 31, 10, 56, 55, 28, 21, 35))
kable(ASAS20)
```


# SAM Prior Derivation

SAM prior is constructed by mixing an informative prior $\pi_1(\theta)$, 
constructed based on historical data, with a non-informative prior 
$\pi_0(\theta)$ using the mixture weight $w$ determined by **`SAM_weight`**
function according to the degree of prior-data conflict [1]. The following 
sections describe how to construct SAM prior in details.

## Informative Prior Construction based on Historical Data

To construct informative priors based on the aforementioned nine historical 
data, we apply **`gMAP`** function from RBesT to perform meta-analysis. This 
informative prior results in a representative form from a large MCMC samples, 
and it can be converted to a parametric representation with the 
**`automixfit`** function using expectation-maximization (EM) algorithm [4].
This informative prior is also called MAP prior.

```{r}
# load R packages
library(ggplot2)
theme_set(theme_bw()) # sets up plotting theme
set.seed(22)
map_ASAS20 <- gMAP(cbind(r, n-r) ~ 1 | study,
                   family = binomial,
                   data = ASAS20, 
                   tau.dist = "HalfNormal", 
                   tau.prior = 1,
                   beta.prior = 2)

map_automix <- automixfit(map_ASAS20)
map_automix
plot(map_automix)$mix
```

The resulting MAP prior is approximated by a mixture of conjugate priors, 
given by $\pi_1(\theta) = 0.63 Beta(42.5, 77.2) + 0.37 Beta(7.2, 12.4)$, with 
$\hat{\theta}_h \approx 0.36$.

## SAM Weight Determination

Let $\theta$ and $\theta_h$ denote the response rates associated with the
current arm data $D$ and historical $D_h$, respectively. Let $\delta$ denote
the clinically significant difference such that is $|\theta_h - \theta| \ge \delta$,
then $\theta_h$ is regarded as clinically distinct from $\theta$, and it is 
therefore inappropriate to borrow any information from $D_h$. Consider two
hypotheses:

$$
H_0: \theta = \theta_h, ~~ H_1: \theta = \theta_h + \delta ~ \text{or} ~ \theta = \theta_h - \delta.
$$
$H_0$ represents that $D_h$ and $D$ are consistent (i.e., no prior-data 
conflict) and thus information borrowing is desirable, whereas $H_1$ represents
that the response rate of $D$ differs from $D_h$ to such a degree that no
information should be borrowed.

The SAM prior uses the likelihood ratio test (LRT) statistics $R$ to quantify
the degree of prior-data conflict and determine the extent of information 
borrowing.
$$
R = \frac{P(D | H_0, \theta_h)}{P(D | H_1, \theta_h)} = \frac{P(D | \theta = \theta_h)}{\max \{ P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta) \}} ,
$$
where $P(D | \cdot)$ denotes the likelihood function. An alternative Bayesian
choice is the posterior probability ratio (PPR):
$$
R = \frac{P(D | H_0, \theta_h)}{P(D | H_1, \theta_h)} = \frac{P(H_0)}{P(H_1)} \times BF ,
$$
where $P(H_0)$ and $P(H_1)$ is the prior probabilities of $H_0$ and $H_1$ 
being true. $BF$ is the Bayes Factor that in this case is the same as LRT.

The SAM prior, denoted as $\pi_{\text{sam}}(\theta)$, is then defined as a mixture
of an informative prior $\pi_1(\theta)$, constructed based on $D_h$, with a 
non-informative prior $\pi_0(\theta)$:
$$\pi_{\text{sam}}(\theta) = w \pi_1(\theta) + (1 - w) \pi_0(\theta)$$
where the mixture weight $w$ is calculated as:
$$w = \frac{R}{1 + R}.$$ 
As the level of prior-data conflict increases, the likelihood ratio $R$ 
decreases, resulting in a decrease in the weight $w$ assigned to the 
informative prior and a decrease in information borrowing. As a result,
$\pi_{\text{sam}}(\theta)$ is data-driven and has the ability to self-adapt the 
information borrowing based on the degree of prior-data conflict.

To calculate mixture weight $w$ of the SAM prior, we assume the sample size 
enrolled in the control arm is $n = 35$, with $r = 10$ responses, then we can 
apply function **`SAM_weight`** in SAMprior as follows: 
```{r, message=FALSE}
n <- 35; r = 10 
wSAM_LRT <- SAM_weight(if.prior = map_automix, 
                       delta = 0.2,
                       n = n, r = r)
                   
cat('SAM weight: ', wSAM_LRT)
```

The default method to calculate $w$ is using LRT, which is fully data-driven.
However, if investigators want to incorporate prior information on prior-data
conflict to determine the mixture weight $w$, this can be achieved by using 
PPR method as follows:
```{r, message=FALSE}
wSAM_PPR <- SAM_weight(if.prior = map_automix, 
                       delta = 0.2,
                       method.w = 'PPR',
                       prior.odds = 3/7,
                       n = n, r = r)
cat('SAM weight: ', wSAM_PPR)
```
The **`prior.odds`** indicates the prior probability of $H_0$ over the prior 
probability of $H_1$. In this case (e.g., **`prior.odds = 3/7`**), the prior 
information favors the presence prior-data conflict and it results in a 
decreased mixture weight.

When historical information is congruent with the current control arm, SAM 
weight reaches to the highest peak. As the level of prior-data conflict 
increases, SAM weight decreases. This demonstrates that SAM prior is data-driven
and self-adapting, favoring the informative (non-informative) prior component 
when there is little (substantial) evidence of prior-data conflict.
```{r, echo=FALSE, message=FALSE}
weight_grid <- seq(1, 34, by = 1)
weight_res  <- lapply(1:34, function(x) SAM_weight(if.prior = map_automix, 
                                                   delta = 0.2,
                                                   n = 35, r = x))
df_weight <- data.frame(grid   = weight_grid/35,
                        weight = unlist(weight_res))
qplot(grid, weight, data = df_weight, geom = "line", main= "SAM Weight") +
  xlab('Response Rate from Control trial')+ ylab('Weight') +
  geom_vline(xintercept = summary(map_automix)['mean'], linetype = 2, col = 'blue') 
```



## SAM Prior Construction

To construct the SAM prior, we mix the derived informative prior $\pi_1(\theta)$ 
with a vague prior $\pi_0(\theta)$ using pre-determined mixture weight by 
**`SAM_prior`** function in SAMprior as follows: 
```{r, message=FALSE}
SAM.prior <- SAM_prior(if.prior = map_automix, 
                       nf.prior = mixbeta(nf.prior = c(1,1,1)),
                       weight = wSAM_LRT)
SAM.prior
```
where the non-informative prior $\pi_0(\theta)$ follows a uniform distribution.

## Operating Characteristics

In this section, we aim to investigate the operating characteristics of 
the SAM prior, constructed based on the historical data in the context 
of ankylosing spondylitis trial. The incorporation of historical information 
is expected to be beneficial in reducing the required sample size for the 
current arms. To achieve this, we assume a 1:2 ratio between the control and 
treatment arms.

We compare the operating characteristics of the SAM prior and rMAP prior 
with pre-specified fixed weight under various scenarios. Specifically, 
we will evaluate the relative bias and relative mean squared error (MSE) 
of these methods. The relative bias and relative MSE are defined 
as the differences between the bias/MSE of a given method and the bias/MSE 
obtained when using a non-informative prior for the estimated response rate in 
the control arm.

Additionally, we investigate the Type I error and power of the methods 
under different degrees of prior-data conflicts. The decision regarding 
whether a treatment is superior or inferior to a standard control will be 
based on the condition:
$$\Pr(\theta_t - \theta > \Delta \mid D) > C,$$
where $\Delta$ is the clinical margin, and we let $\Delta = 0$. 
$C$ is the posterior probability cutoff. We calibrated the posterior 
probability cutoff under the null hypothesis, where there is no treatment 
effect difference between the treatment and control arms and the historical
data and current control arm are assumed to arise from the same data-generating
process, to ensure the Type I error is maintained at the nominal level.

In SAMprior, the operating characteristics can be considered in following steps:

1. Specify priors: This step involves constructing informative prior based 
   on historical data and non-informative prior.
   
2. Specify design parameters for the **`get_OC`** function: This step 
   involves defining the design parameters for evaluating the operating 
   characteristics. These parameters include the clinically significant 
   difference (CSD) used in SAM prior calculation, the method used to 
   determine the mixture weight for the SAM prior, the sample sizes for 
   the control and treatment arms, the choice of weight for the robust MAP 
   prior used as a benchmark.
   
3. Scenarios: This step specifies the vectors of response rates for the 
   control and treatment arms. The **`get_OC`** function calibrates the
   posterior probability cutoff under the assumption that the null calibration 
   scenario is given by $\theta =$ **`theta[1]`** and 
   $\theta_t =$ **`theta[1]`** $+ \Delta$ with $\Delta = 0$.
   
   
### Type I Error
To evaluate Type I error, we consider four scenarios. In the first scenario, 
we assume $\theta = \theta_t = \theta_h$, so each method should calibrate 
the Type I error rate to the nominal level. The second and third scenarios 
represent minimal prior-data conflict, whereas the last scenario represents 
substantial prior-data conflict. Overall, the results indicate that both 
SAM and rMAP effectively control the Type I error under minimal prior-data 
conflict, while SAM demonstrates better Type I error control in the presence
of substantial prior-data conflict.

```{r, message=FALSE}
TypeI <- get_OC(if.prior = map_automix,       ## MAP prior from historical data
                nf.prior = mixbeta(c(1,1,1)), ## Non-informative prior for mixture prior
                prior.t = mixbeta(c(1,1,1)),  ## Non-informative prior for treatment arm
                delta    = 0.2,               ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Response rates for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.30, 0.40, 0.60),
                theta.t  = c(summary(map_automix)['mean'], 0.30, 0.38, 0.61))
kable(TypeI)
```

### Power

For power evaluation, we also consider four scenarios. In the first scenario, 
we assume $\theta = \theta_t = \theta_h$, so each method should calibrate the 
Type I error rate to the nominal level. The second and third scenarios represent 
minimal prior-data conflict, whereas the last scenario represents substantial 
prior-data conflict. Overall, the results show that the SAM prior achieves 
performance similar to that of rMAP, and both outperform non-informative 
prior (NP) when prior-data conflict is minimal. However, when prior-data 
conflict is substantial, the SAM prior yields better performance than rMAP.
```{r, message=FALSE}
Power <- get_OC(if.prior = map_automix,       ## MAP prior from historical data
                nf.prior = mixbeta(c(1,1,1)), ## Non-informative prior for mixture prior
                prior.t = mixbeta(c(1,1,1)),  ## Non-informative prior for treatment arm
                delta    = 0.2,               ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin,
                ## Response rates for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.36, 0.42, 0.16),
                theta.t  = c(summary(map_automix)['mean'], 0.56, 0.62, 0.36))
kable(Power)
```


# Decision Making

Finally, we present an example showing how to calibrate the posterior 
probability cutoff and make a final decision on whether the treatment
is superior to a standard control after the trial has been 
completed and the data have been collected.

The calibration step aims to identify the posterior probability cutoff $C$ 
such that the Type I error is controlled at a prespecified level. 
For superiority, this is based on the posterior decision rule
$$
\Pr(\theta_t - \theta > 0 \mid D) > C.
$$


This calibration can be carried out using the **`calibrate_cutoff_2arm`**
function, as illustrated below:
```{r}
## Sample size and number of responses for treatment arm
n_t <- 70; x_t <- 22 
## Calibrate the posterior probability cutoff 
PPC <- calibrate_cutoff_2arm(if.prior = map_automix,        ## MAP prior from historical data
                             nf.prior = mixbeta(c(1,1,1)),  ## Non-informative prior for mixture prior
                             prior.t = mixbeta(c(1,1,1)),   ## Non-informative prior for treatment arm
                             target = 0.05,                 ## Targeted Type I error rate
                             n.t = n_t, n = n,              ## Sample size for treatment and control arms, respectively
                             theta.t = summary(map_automix)['mean'],  ## The true response rate for treatment arm
                             theta = summary(map_automix)['mean'],    ## The true response rate for control arm
                             ## Method to determine the mixture weight for the SAM prior
                             method = 'SAM',    ## Method 
                             delta = 0.2,       ## CSD for SAM prior
                             ## Trial settings
                             alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less".
                             margin = 0      ## Clinical margin
                             )
```


```{r include=F}
cat("Calibrated posterior probability cutoff:", round(PPC$cutoff, 4), "\n")
```

To make the final decision using the calibrated posterior probability cutoff 
$C$, we next update the posterior distributions based on the observed trial 
data. In this example, the final posterior updating step is carried out using 
the **`postmix`** function from RBesT, while the calibrated cutoff $C$ is 
obtained from **`calibrate_cutoff_2arm`** in SAMprior. The treatment is then 
declared superior if the resulting posterior probability exceeds the calibrated 
cutoff. The posterior updating step is illustrated below:

```{r}
## first obtain posterior distributions...
post_SAM <- postmix(priormix = SAM.prior,         ## SAM Prior
                    r = r,   n = n)
post_trt <- postmix(priormix = mixbeta(c(1,1,1)), ## Non-informative prior
                    r = x_t, n = n_t)

## Define the decision function
decision = decision2S(PPC$cutoff, 0, lower.tail=FALSE)

## Decision-making
decision(post_trt, post_SAM)
```


### References

[1] Yang P. et al., _Biometrics_, 2023; 00, 1–12. https://doi.org/10.1111/biom.13927 \
[2] Schmidli H. et al., _Biometrics_ 2014; 70(4):1023-1032. \
[3] Baeten D. et al., _The Lancet_, 2013; (382), 9906, p 1705. \
[4] Neuenschwander B. et al., _Clin Trials_. 2010; 7(1):5-18. 

### R Session Info

```{r}
sessionInfo()
```

```{r,include=FALSE}
options(.user_mc_options)
```

---
title: "SAMprior for Continuous Endpoints"
author: "Peng Yang and Ying Yuan"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
  html_vignette:
    toc: true
  html_document:
    toc: true
    number_sections: true
    toc_float:
      collapsed: false
      smooth_scroll: false
  pdf_document:
    toc: true
  word_document:
    toc: true
vignette: >
  %\VignetteIndexEntry{Getting started with SAMprior (continuous)}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---
  
```{r, include=FALSE}
library(SAMprior)
library(knitr)
knitr::opts_chunk$set(
    fig.width = 1.62*4,
    fig.height = 4
    )
## setup up fast sampling when run on CRAN
is_CRAN <- !identical(Sys.getenv("NOT_CRAN"), "true")
## NOTE: for running this vignette locally, please uncomment the
## following line:
## is_CRAN <- FALSE
.user_mc_options <- list()
if (is_CRAN) {
    .user_mc_options <- options(RBesT.MC.warmup=250, RBesT.MC.iter=500, RBesT.MC.chains=2, RBesT.MC.thin=1, RBesT.MC.control=list(adapt_delta=0.9))
}
```

# Introduction

The self-adapting mixture prior (SAMprior) package is designed to enhance 
the effectiveness and practicality of clinical trials by leveraging historical 
information or real-world data [1]. The package incorporates historical data 
into a new trial using an informative prior constructed based on historical 
data while mixing a non-informative prior to enhance the robustness of 
information borrowing. It utilizes a data-driven way to determine a self-adapting
mixture weight that dynamically favors the informative (non-informative) prior
component when there is little (substantial) evidence of prior-data conflict.
Operating characteristics are evaluated and compared to the robust 
Meta-Analytic-Predictive (rMAP) prior [2], which assigns a fixed weight of 0.5. 

# SAM Prior Derivation

SAM prior is constructed by mixing an informative prior $\pi_1(\theta)$, 
constructed based on historical data, with a non-informative prior 
$\pi_0(\theta)$ using the mixture weight $w$ determined by **`SAM_weight`**
function according to the degree of prior-data conflict [1]. The following 
sections describe how to construct SAM prior in details.

## Informative Prior Construction based on Historical Data

We assume three historical data as follows:
```{r,results="D_h",echo=FALSE}
set.seed(123)
std <- function(x) sd(x)/sqrt(length(x))
df_1 <- rnorm(40, 0, 3); 
df_2 <- rnorm(50, 0, 3); 
df_3 <- rnorm(60, 0, 3); 
dat <- data.frame(study = c(1,2,3),
                  n = c(40, 50, 60),
                  mean = round(c(mean(df_1), mean(df_2), mean(df_3)), 3),
                  se = round(c(std(df_1), std(df_2), std(df_3)), 3))
kable(dat)
```



To construct informative priors based on the aforementioned three historical 
data, we apply **`gMAP`** function from RBesT to perform meta-analysis. This 
informative prior results in a representative form from a large MCMC samples, 
and it can be converted to a parametric representation with the 
**`automixfit`** function using expectation-maximization (EM) algorithm [3].
This informative prior is also called MAP prior.

```{r, message=FALSE}
sigma = 3
# load R packages
library(ggplot2)
theme_set(theme_bw()) # sets up plotting theme
set.seed(22)
map_mcmc <- gMAP(cbind(mean, se) ~ 1 | study, 
                 weights=n,data=dat,
                 family=gaussian,
                 beta.prior=cbind(0, sigma),
                 tau.dist="HalfNormal",tau.prior=cbind(0,sigma/2))

map_automix <- automixfit(map_mcmc)
map_automix
plot(map_automix)$mix
```

The resulting MAP prior is approximated by a mixture of conjugate priors.

## SAM Weight Determination

Let $\theta$ and $\theta_h$ denote the treatment effects associated with the
current arm data $D$ and historical $D_h$, respectively. Let $\delta$ denote
the clinically significant difference such that is $|\theta_h - \theta| \ge \delta$,
then $\theta_h$ is regarded as clinically distinct from $\theta$, and it is 
therefore inappropriate to borrow any information from $D_h$. Consider two
hypotheses:

$$
H_0: \theta = \theta_h, ~~ H_1: \theta = \theta_h + \delta ~ \text{or} ~ \theta = \theta_h - \delta.
$$
$H_0$ represents that $D_h$ and $D$ are consistent (i.e., no prior-data 
conflict) and thus information borrowing is desirable, whereas $H_1$ represents
that the treatment effect of $D$ differs from $D_h$ to such a degree that no
information should be borrowed.

The SAM prior uses the likelihood ratio test (LRT) statistics $R$ to quantify
the degree of prior-data conflict and determine the extent of information 
borrowing.
$$
R = \frac{P(D | H_0, \theta_h)}{P(D | H_1, \theta_h)} = \frac{P(D | \theta = \theta_h)}{\max \{ P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta) \}} ,
$$
where $P(D | \cdot)$ denotes the likelihood function. An alternative Bayesian
choice is the posterior probability ratio (PPR):
$$
R = \frac{P(D | H_0, \theta_h)}{P(D | H_1, \theta_h)} = \frac{P(H_0)}{P(H_1)} \times BF ,
$$
where $P(H_0)$ and $P(H_1)$ is the prior probabilities of $H_0$ and $H_1$ 
being true. $BF$ is the Bayes Factor that in this case is the same as LRT.

The SAM prior, denoted as $\pi_{\text{sam}}(\theta)$, is then defined as a mixture
of an informative prior $\pi_1(\theta)$, constructed based on $D_h$, with a 
non-informative prior $\pi_0(\theta)$:
$$\pi_{\text{sam}}(\theta) = w \pi_1(\theta) + (1 - w) \pi_0(\theta)$$
where the mixture weight $w$ is calculated as:
$$w = \frac{R}{1 + R}.$$ 
As the level of prior-data conflict increases, the likelihood ratio $R$ 
decreases, resulting in a decrease in the weight $w$ assigned to the 
informative prior and a decrease in information borrowing. As a result,
$\pi_{\text{sam}}(\theta)$ is data-driven and has the ability to self-adapt the 
information borrowing based on the degree of prior-data conflict.

To calculate the SAM weight $w$, we first assume the sample size enrolled in 
the control arm is $n = 30$, with $\theta = 0.4$ and $\sigma = 3$. Additionally,
we assume the effective size is $d = \frac{\theta - \theta_h}{\sigma} = 0.5$,
then we can apply 
function **`SAM_weight`** in SAMprior as follows: 
```{r, message=FALSE}
set.seed(234)
sigma        <- 3 ## Standard deviation in the current trial
data.crt     <- rnorm(35, mean = 0.4, sd = sigma)
wSAM_LRT <- SAM_weight(if.prior = map_automix, 
                       delta = 0.5 * sigma,
                       data = data.crt)
                   
cat('SAM weight: ', wSAM_LRT)
```

The default method to calculate $w$ is using LRT, which is fully data-driven.
However, if investigators want to incorporate prior information on prior-data
conflict to determine the mixture weight $w$, this can be achieved by using 
PPR method as follows:
```{r, message=FALSE}
wSAM_PPR <- SAM_weight(if.prior = map_automix, 
                       delta = 0.5 * sigma,
                       method.w = 'PPR',
                       prior.odds = 3/7,
                       data = data.crt)
                       
cat('SAM weight: ', wSAM_PPR)
```
The **`prior.odds`** indicates the prior probability of $H_0$ over the prior 
probability of $H_1$. In this case (e.g., **`prior.odds = 3/7`**), the prior 
information favors the presence prior-data conflict and it results in a 
decreased mixture weight.


When historical information is congruent with the current control arm, SAM 
weight reaches to the highest peak. As the level of prior-data conflict 
increases, SAM weight decreases. This demonstrates that SAM prior is data-driven
and self-adapting, favoring the informative (non-informative) prior component 
when there is little (substantial) evidence of prior-data conflict.
```{r, echo=FALSE, message=FALSE, warning=FALSE}
weight_grid <- seq(-3, 3, by = 0.3)
weight_res  <- lapply(weight_grid, function(x){
  res <- c()
  for(s in 1:300){
    data.control <- rnorm(n = 35, mean = x, sd = sigma)
    res <- c(res, SAM_weight(if.prior = map_automix,
                             delta = 0.5 * sigma,
                             data = data.control))
    
  }
  mean(res)
})
df_weight <- data.frame(grid   = weight_grid,
                        weight = unlist(weight_res))
qplot(grid, weight, data = df_weight, geom = "line", main= "SAM Weight") +
  xlab('Sample mean from control trial')+ ylab('Weight') +
  geom_vline(xintercept = summary(map_automix)['mean'], linetype = 2, col = 'blue') 
```



## SAM Prior Construction

To construct the SAM prior, we mix the derived informative prior $\pi_1(\theta)$ 
with a vague prior $\pi_0(\theta)$ using pre-determined mixture weight by 
**`SAM_prior`** function in SAMprior as follows: 
```{r, message=FALSE}
unit_prior <- mixnorm(nf.prior = c(1, summary(map_automix)['mean'], sigma))
SAM.prior <- SAM_prior(if.prior = map_automix, 
                       nf.prior = unit_prior,
                       weight = wSAM_LRT, sigma = sigma)
SAM.prior
```
where the non-informative prior $\pi_0(\theta)$ follows an unit-information prior.

## Operating Characteristics

In this section, we aim to investigate the operating characteristics of 
the SAM prior, constructed based on the historical data.
The incorporation of historical information is expected to be beneficial 
in reducing the required sample size for the current arms. To achieve this, 
we assume a 1:2 ratio between the control and treatment arms.

We compare the operating characteristics of the SAM prior and rMAP prior 
with pre-specified fixed weight under various scenarios. Specifically, 
we will evaluate the relative bias and relative mean squared error (MSE) 
of these methods. The relative bias and relative MSE are defined 
as the differences between the bias/MSE of a given method and the bias/MSE 
obtained when using a non-informative prior for the estimated effect in the 
control arm.

Additionally, we investigate the Type I error and power of the methods 
under different degrees of prior-data conflicts. The decision regarding 
whether a treatment is superior or inferior to a standard control will be 
based on the condition:
$$\Pr(\theta_t - \theta > 0 \mid D) > C,$$
where $\Delta$ is the clinical margin, and we let $\Delta = 0$.
$C$ is the posterior probability cutoff. We calibrated the posterior 
probability cutoff under the null hypothesis, where there is no treatment 
effect difference between the treatment and control arms and the historical
data and current control arm are assumed to arise from the same data-generating
process, to ensure the Type I error is maintained at the nominal level.

In SAMprior, the operating characteristics can be considered in following steps:

1. Specify priors: This step involves constructing informative prior based 
   on historical data and non-informative prior.
   
2. Specify design parameters for the **`get_OC`** function: This step 
   involves defining the design parameters for evaluating the operating 
   characteristics. These parameters include the clinically significant 
   difference (CSD) used in SAM prior calculation, the method used to 
   determine the mixture weight for the SAM prior, the sample sizes for 
   the control and treatment arms, the choice of weight for the robust MAP 
   prior used as a benchmark.
   
3. Scenarios: This step specifies the vectors of response rates for the 
   control and treatment arms. The **`get_OC`** function calibrates the
   posterior probability cutoff under the assumption that the null calibration 
   scenario is given by $\theta =$ **`theta[1]`** and 
   $\theta_t =$ **`theta[1]`** $+ \Delta$ with $\Delta = 0$.

### Type I Error
To evaluate Type I error, we consider four scenarios. In the first scenario, 
we assume $\theta = \theta_t = \theta_h$, so each method should calibrate 
the Type I error rate to the nominal level. The second and third scenarios 
represent minimal prior-data conflict, whereas the last scenario represents 
substantial prior-data conflict. Overall, the results indicate that both 
SAM and rMAP effectively control the Type I error under minimal prior-data 
conflict, while SAM demonstrates better Type I error control in the presence
of substantial prior-data conflict.
```{r, message=FALSE}
# weak_prior <- mixnorm(c(1, summary(map_automix)[1], 1e4))
TypeI <- get_OC(if.prior = map_automix,         ## MAP prior from historical data
                nf.prior = unit_prior,          ## Unit-informative prior for mixture prior
                prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                delta    = 0.5*sigma,         ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Treatment effect for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0,    -0.2, 2),
                theta.t  = c(summary(map_automix)['mean'], -0.1, -0.2, 2))
kable(TypeI)
```
### Power
For power evaluation, we also consider four scenarios. In the first scenario, 
we assume $\theta = \theta_t = \theta_h$, so each method should calibrate the 
Type I error rate to the nominal level. The second and third scenarios represent 
minimal prior-data conflict, whereas the last scenario represents substantial 
prior-data conflict. Overall, the results show that the SAM prior achieves 
performance similar to that of rMAP, and both outperform non-informative prior 
(NP) when prior-data conflict is minimal. However, when prior-data conflict is 
substantial, the SAM prior yields better performance than rMAP.
```{r, message=FALSE}
Power <- get_OC(if.prior = map_automix,         ## MAP prior from historical data
                nf.prior = unit_prior,          ## Unit-information prior for mixture prior
                prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                delta    = 0.5 * sigma,       ## CSD for SAM prior
                ## Method to determine the mixture weight for the SAM prior
                method.w = 'LRT',             
                n        = 35, n.t = 70,      ## Sample size for control and treatment arms
                if.rMAP   = TRUE,             ## Output robust MAP prior for comparison
                weight   = 0.5,               ## Weight for robust MAP prior
                ## Trial settings
                alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less"
                margin = 0,              ## Clinical margin
                ## Treatment effect for control and treatment arms
                theta    = c(summary(map_automix)['mean'], 0.1, 0.5,  -2),
                theta.t  = c(summary(map_automix)['mean'], 1.1, 2.0,  -0.5))
kable(Power)
```


# Decision Making
Finally, we present an example showing how to calibrate the posterior 
probability cutoff and make a final decision on whether the treatment
is superior to a standard control after the trial has been 
completed and the data have been collected.

The calibration step aims to identify the posterior probability cutoff $C$ 
such that the Type I error is controlled at a prespecified level. 
For superiority, this is based on the posterior decision rule
$$
\Pr(\theta_t - \theta > \Delta \mid D) > C,
$$
where $\Delta$ is the clinical margin, which is often 0.

This calibration can be carried out using the **`calibrate_cutoff_2arm`**
function, as illustrated below:
```{r}
## Calibrate the posterior probability cutoff 
PPC <- calibrate_cutoff_2arm(if.prior = map_automix,         ## MAP prior from historical data
                             nf.prior = unit_prior,          ## Unit-information prior for mixture prior
                             prior.t = mixnorm(c(1,0,1000)), ## Vague prior for treatment arm
                             target = 0.05,                 ## Targeted Type I error rate
                             n.t = 70, n = 35,              ## Sample size for treatment and control arms, respectively
                             theta.t = summary(map_automix)['mean'],  ## The true effect for treatment arm
                             theta = summary(map_automix)['mean'],    ## The true effect for control arm
                             sigma.t = sigma, sigma = sigma, ## Standard deviation in the treatment and control arms, respectively
                             ## Method to determine the mixture weight for the SAM prior
                             method = 'SAM',    ## Method 
                             delta = 0.2,       ## CSD for SAM prior
                             ## Trial settings
                             alternative = "greater", ## Direction of the posterior decision. Must be one of "greater" or "less".
                             margin = 0.     ## Clinical margin.
                             )
```


```{r include=F}
cat("Calibrated posterior probability cutoff:", round(PPC$cutoff, 4), "\n")
```

To make the final decision using the calibrated posterior probability cutoff 
$C$, we next update the posterior distributions based on the observed trial 
data. In this example, the final posterior updating step is carried out using 
the **`postmix`** function from RBesT, while the calibrated cutoff $C$ is 
obtained from **`calibrate_cutoff_2arm`** in SAMprior. The treatment is then 
declared superior if the resulting posterior probability exceeds the calibrated 
cutoff. The posterior updating step is illustrated below:

```{r}
## Simulate data for treatment arm
data.trt <- rnorm(70, mean = 3, sd = sigma)

## first obtain posterior distributions...
post_SAM <- postmix(priormix = SAM.prior,   ## SAM Prior
                    data = data.crt)
post_trt <- postmix(priormix = unit_prior,  ## Non-informative prior
                    data = data.trt)

## Define the decision function
decision = decision2S(PPC$cutoff, 0, lower.tail=FALSE)

## Decision-making
decision(post_trt, post_SAM)
```


### References

[1] Yang P. et al., _Biometrics_, 2023; 00, 1–12. https://doi.org/10.1111/biom.13927 \
[2] Schmidli H. et al., _Biometrics_, 2014; 70(4):1023-1032. \
[3] Neuenschwander B. et al., _Clin Trials_, 2010; 7(1):5-18. 




### R Session Info

```{r}
sessionInfo()
```

```{r,include=FALSE}
options(.user_mc_options)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calibrate_cutoff_2arm.R
\name{calibrate_cutoff_2arm}
\alias{calibrate_cutoff_2arm}
\alias{calibrate_cutoff_2arm.betaMix}
\alias{calibrate_cutoff_2arm.normMix}
\title{Calibrate Posterior Probability Cutoff for a Two-Arm Comparative Trial}
\usage{
calibrate_cutoff_2arm(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  target = 0.05,
  n.t,
  n,
  theta.t = NULL,
  theta = NULL,
  delta,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  interval = c(0.5, 0.999),
  rel.tol = 1e-05,
  oc_rel.tol = 1e-06,
  ...
)

\method{calibrate_cutoff_2arm}{betaMix}(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  target = 0.05,
  n.t,
  n,
  theta.t = NULL,
  theta = NULL,
  delta,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  interval = c(0.5, 0.999),
  rel.tol = 1e-05,
  oc_rel.tol = 1e-08,
  ...
)

\method{calibrate_cutoff_2arm}{normMix}(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  target = 0.05,
  n.t,
  n,
  theta.t = NULL,
  theta = NULL,
  delta,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  interval = c(0.5, 0.999),
  rel.tol = 1e-05,
  oc_rel.tol = 1e-06,
  sigma.t,
  sigma,
  n_sd_int = 8,
  ...
)
}
\arguments{
\item{if.prior}{Informative prior constructed from historical data,
represented (approximately) as a mixture of conjugate distributions.}

\item{nf.prior}{Non-informative prior used for the mixture and as the
robustifying component for the control arm prior.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{target}{Target rejection probability under the calibration scenario,
typically the desired type I error rate when the calibration scenario
corresponds to the null hypothesis.}

\item{n.t}{Sample size for the treatment arm.}

\item{n}{Sample size for the control arm.}

\item{theta.t}{The response rate (binary endpoints) or mean
(continuous endpoints) for the treatment arm under the calibration scenario.}

\item{theta}{The response rate (binary endpoints) or mean
(continuous endpoints) for the control arm under the calibration scenario.}

\item{delta}{Clinically significant difference used for the SAM prior.
This argument is only used when \code{method = "SAM"}.}

\item{method}{Borrowing strategy for the control arm. Must be one of
\code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be non-negative. Default is \code{0}.}

\item{weight_rMAP}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
This argument is only used when \code{method = "rMAP"}. The default value is
0.5.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
more details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}

\item{interval}{Search interval for the posterior probability cutoff. The
default is from 0.5 to 0.999.}

\item{rel.tol}{Tolerance passed to \code{\link[stats]{uniroot}}.}

\item{oc_rel.tol}{Relative tolerance passed to scenario-level calculations.
For continuous endpoints this controls numerical integration in the
operating-characteristic engine; for binary endpoints this controls numerical
integration in posterior probability calculations.}

\item{...}{Additional arguments required by the endpoint-specific method.
For continuous endpoints, this includes \code{sigma.t}, \code{sigma}, and
optionally \code{n_sd_int}.}

\item{sigma.t}{Known sampling standard deviation in the treatment arm.}

\item{sigma}{Known sampling standard deviation in the control arm.}

\item{n_sd_int}{Half-width of the numerical integration region for each arm,
expressed as a multiple of the corresponding standard error.}
}
\value{
A list with the following elements:
\describe{
  \item{cutoff}{Calibrated posterior probability cutoff.}
  \item{objective}{Value of the root-finding objective at the solution.}
  \item{target}{Target rejection probability.}
  \item{method}{Borrowing method used.}
  \item{alternative}{Direction of the posterior decision.}
  \item{margin}{Clinical margin used for inference.}
  \item{theta}{True control-arm parameter under the calibration scenario.}
  \item{theta.t}{True treatment-arm parameter under the calibration scenario.}
  \item{interval}{Search interval used for calibration.}
}
}
\description{
The \code{calibrate_cutoff_2arm} function is designed to calibrate the
posterior probability cutoff for a two-arm comparative trial under the
specified endpoint type and borrowing strategy: self-adapting mixture
prior (SAM), robust MAP prior with fixed weight (rMAP), or non-informative
prior (NP).
}
\details{
For binary endpoints, the function dispatches to
\code{\link{calibrate_cutoff_bin_2arm}}. For continuous endpoints, it
dispatches to \code{\link{calibrate_cutoff_cont_2arm}}.


The function calibrates the posterior probability cutoff \eqn{C} for a
two-arm comparative trial so that the repeated-sampling rejection
probability under the specified calibration scenario equals
\code{target}. Inference is based on the posterior probability of the
treatment effect \eqn{\theta_t - \theta}, where \eqn{\theta_t} and
\eqn{\theta} denote the treatment-arm and control-arm parameters,
respectively.

For \code{alternative = "greater"}, the posterior decision rule is based on
\deqn{P(\theta_t - \theta > \Delta \mid D) > C,}
where \eqn{\Delta} denotes the clinical margin and \eqn{D} denotes the
observed data. Under the default settings, calibration is performed under
the null scenario \eqn{\theta_t = \theta = \theta_h}, where
\eqn{\theta_h = E(\code{if.prior})} is the posterior mean of the
informative prior constructed from historical data. This corresponds to a
superiority test when \eqn{\Delta = 0}.

For \code{alternative = "less"}, the posterior decision rule is based on
\deqn{P(\theta_t - \theta < -\Delta \mid D) > C.}
Calibration is again performed under the null scenario
\eqn{\theta_t = \theta = \theta_h}, where
\eqn{\theta_h = E(\code{if.prior})}. This corresponds to an
inferiority test when \eqn{\Delta = 0}.

This function acts as a generic front-end and dispatches according to the
class of \code{if.prior}. For binary endpoints, it calls
\code{\link{calibrate_cutoff_bin_2arm}}. For continuous endpoints, it calls
\code{\link{calibrate_cutoff_cont_2arm}}.
}
\section{Methods (by class)}{
\itemize{
\item \code{calibrate_cutoff_2arm(betaMix)}: Calibrate the posterior probability cutoff for a two-arm binary endpoint.

\item \code{calibrate_cutoff_2arm(normMix)}: Calibrate the posterior probability cutoff for a two-arm continuous endpoint.

}}
\examples{
## Example: calibrate the posterior probability cutoff for a two-arm
## binary trial using a SAM prior for the control arm

## Informative prior constructed from historical data
if.prior <- mixbeta(c(1, 20, 40))

## Sample sizes
n.t <- 100
n <- 50

## Calibrate the posterior probability cutoff
PPC <- calibrate_cutoff_2arm(
  if.prior = if.prior,             ## Informative prior from historical data
  nf.prior = mixbeta(c(1, 1, 1)),  ## Non-informative prior for SAM prior
  prior.t = mixbeta(c(1, 1, 1)),   ## Prior for treatment arm
  target = 0.05,                   ## Targeted type I error rate
  n.t = n.t,                       ## Sample size for treatment arm
  n = n,                           ## Sample size for control arm
  theta.t = summary(if.prior)["mean"],  ## True treatment-arm response rate
  theta = summary(if.prior)["mean"],    ## True control-arm response rate
  method = "SAM",                  ## Borrowing method
  delta = 0.2,                     ## Clinically significant difference
  alternative = "greater",         ## Superiority test
  margin = 0                       ## Clinical margin
)

PPC


}
\seealso{
\code{\link{calibrate_cutoff_bin_2arm}},
\code{\link{calibrate_cutoff_cont_2arm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PS_SAM_data.R
\docType{data}
\name{PS_SAM_data}
\alias{PS_SAM_data}
\title{Simulated Data for the Construction of Propensity Score-Integrated Informative Priors}
\format{
A data frame with 600 observations.
\itemize{
\item "A" is the treatment assignment (1 = treated, 0 = control).
\item "G" is the study indicator (1 = current, 0 = historical).
\item "\eqn{X_1}" is a binary covariate.
\item "\eqn{X_2}" is a continuous covariate.
\item "\eqn{X_3}" is a continuous covariate.
\item "\eqn{Y_{binary}}" is binary outcome.
\item "\eqn{Y_{continuous}}" is continuous outcome.
}
}
\usage{
PS_SAM_data
}
\description{
This dataset demonstrates the construction of a Propensity
Score-Integrated (PS) SAM prior. It simulates a two-arm randomized
clinical trial (RCT) with a 2:1 randomization ratio between treatment and
control arms, considering both binary and continuous endpoints.
}
\details{
The dataset includes:
\itemize{
  \item Sample size for treatment arm: \eqn{n_t = 200}.
  \item Sample size for control arm: \eqn{n_c = 100}.
  \item Sample size for historical control study: \eqn{n_h = 300}.
}

Covariates for the control arm were generated from

\deqn{X_1 \sim Ber(0.5), ~~ X_2 \sim N(0, 1), ~~ X_3 \sim N(0.5, 1),}

where \eqn{Ber(\cdot)} stands for Bernoulli distribution. Covariates for the
historical controls were generated from a mixture distribution, with half
were generated the same as for the control arm, while the other half were
drawn from

\deqn{X_1 \sim Ber(0.8), ~~ X_2 \sim N(-0.4, 1), ~~ X_3 \sim N(-0.2, 1).}

For the binary endpoint, \eqn{y_i} were generated from the logit model:

\deqn{logit(\Pr(y_i = 1 | X_{1i}, X_{2i}, X_{3i}, A_i)) = -1.4 - 0.5
X_{1i} + X_{2i} + 2 X_{3i} + \lambda A_i,}

where \eqn{\lambda} is the treatment effect size, and we let \eqn{\lambda = 0.9}
to generate a moderate treatment effect size so that they study has a reasonable
power.

For the continuous endpoint, \eqn{y_i} were generated from the following
normal model:

\deqn{y_i = 1.8 X_{1i} + 0.9 X_{2i} - 2 X_{3i} + \lambda A_i + \epsilon_i,}

where we let \eqn{\lambda = 1}, and \eqn{\epsilon_i \sim N(0, 3.5^2)}.

This dataset enables evaluation of the PS-SAM prior's performance in addressing heterogeneity
between the RCT control arm and historical controls.
}
\examples{
# Load the dataset
data(PS_SAM_data)

# View the structure
str(PS_SAM_data)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_summary_cont_2arm.R
\name{post_summary_cont_2arm}
\alias{post_summary_cont_2arm}
\title{Posterior Summary for Two-Arm Comparative Trial with Continuous Endpoint}
\usage{
post_summary_cont_2arm(
  ybar_t,
  ybar,
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  n.t,
  n,
  sigma.t,
  sigma,
  delta,
  cutoff,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1
)
}
\arguments{
\item{ybar_t}{Observed sample mean in the treatment arm.}

\item{ybar}{Observed sample mean in the control arm.}

\item{if.prior}{Informative prior constructed based on historical data for
the control arm, represented (approximately) as a normal mixture prior.}

\item{nf.prior}{Non-informative prior used as the robustifying component
for the control arm prior.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{n.t}{Sample size for the treatment arm.}

\item{n}{Sample size for the control arm.}

\item{sigma.t}{Known standard deviation in the treatment arm.}

\item{sigma}{Known standard deviation in the control arm.}

\item{delta}{Clinically significant difference used for the SAM prior.
This argument is only used when \code{method = "SAM"}.}

\item{cutoff}{Posterior probability cutoff used for decision making.
The null hypothesis is rejected if the posterior tail probability
exceeds \code{cutoff}.}

\item{method}{Borrowing strategy for the control arm. Must be one of
\code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be a non-negative scalar. The default
value is \code{0}.}

\item{weight_rMAP}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
This argument is only used when \code{method = "rMAP"}. The default value is
0.5.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
more details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}
}
\value{
A list containing the posterior probability in the requested
direction, posterior mean and variance of \eqn{\tau}, decision indicator,
borrowing weight used for the control arm prior, and the corresponding
trial data and method information.
}
\description{
The \code{post_summary_cont_2arm} function is designed to compute the
posterior summary for the treatment effect in a two-arm comparative trial
with a continuous endpoint under one of three borrowing strategies:
self-adapting mixture prior (SAM), robust MAP prior with fixed weight
(rMAP), or non-informative prior (NP).
}
\details{
The treatment effect is defined as \eqn{\tau = \theta_t - \theta}, where
\eqn{\theta_t} and \eqn{\theta} denote the means in the treatment and
control arms, respectively. Inference is based on the posterior distribution
of \eqn{\tau} given the observed sample means from the two arms.


The posterior for the treatment arm is obtained by updating
\code{prior.t} using the observed sample mean \code{ybar_t}. The posterior
for the control arm depends on the selected borrowing strategy:

\itemize{
\item \code{SAM}: the control arm prior is a mixture of \code{if.prior}
and \code{nf.prior}, with adaptive mixture weight determined by the
SAM borrowing rule.
\item \code{rMAP}: the control arm prior is a fixed mixture of
\code{if.prior} and \code{nf.prior}, with fixed weight
\code{weight_rMAP}.
\item \code{NP}: the control arm prior is \code{nf.prior} only.
}

When \code{alternative = "greater"}, inference is based on
\eqn{P(\theta_t - \theta > margin \mid \bar{Y}_t, \bar{Y})}. When
\code{alternative = "less"}, inference is based on
\eqn{P(\theta_t - \theta < -margin \mid \bar{Y}_t, \bar{Y})}.

The posterior mean and posterior variance of \eqn{\tau} are defined as
are defined as
\deqn{E(\tau \mid \bar{Y}_t, \bar{Y}) = E(\theta_t \mid \bar{Y}_t) - E(\theta \mid \bar{Y}),}
and
\deqn{\mathrm{Var}(\tau \mid \bar{Y}_t, \bar{Y}) = \mathrm{Var}(\theta_t \mid \bar{Y}_t) + \mathrm{Var}(\theta \mid \bar{Y}),}
where independence between the treatment and control arm posteriors is
assumed conditional on the current and historical data.
}
\seealso{
\code{\link{SAM_weight}}, \code{\link{SAM_prior}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SAM_prior.R
\name{SAM_prior}
\alias{SAM_prior}
\alias{SAM_prior.betaMix}
\alias{SAM_prior.gammaMix}
\alias{SAM_prior.normMix}
\title{Calculating SAM priors}
\usage{
SAM_prior(if.prior, nf.prior, weight, ...)

\method{SAM_prior}{betaMix}(if.prior, nf.prior, weight, ...)

\method{SAM_prior}{gammaMix}(if.prior, nf.prior, weight, ...)

\method{SAM_prior}{normMix}(if.prior, nf.prior, weight, ..., sigma)
}
\arguments{
\item{if.prior}{Informative prior constructed from historical data,
represented (approximately) as a mixture of conjugate distributions.}

\item{nf.prior}{Non-informative prior used for the mixture.}

\item{weight}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight} \eqn{\leq 1}), which should be determined by
\code{\link{SAM_weight}} function.}

\item{...}{Additional parameters required for different endpoints.}

\item{sigma}{Variance used for constructing the non-informative prior for
continuous endpoints.}
}
\value{
Displays the SAM prior as a mixture of an informative prior
(constructed based on the historical data) and a non-informative prior.
}
\description{
The \code{SAM_prior} function is designed to display the SAM prior, given
the informative prior (constructed from historical data), non-informative
prior, and the mixture weight calculated using \code{\link{SAM_weight}}
function (\emph{Yang, et al., 2023}).
}
\details{
SAM prior is constructed by mixing an informative prior
\eqn{\pi_1(\theta)}, constructed based on historical data, with a
non-informative prior \eqn{\pi_0(\theta)} using the mixture weight
\eqn{w} determined by \code{\link{SAM_weight}} function to achieve the
degree of prior-data conflict (\emph{Schmidli et al., 2015, Yang et al., 2023}).

 Let \eqn{\theta} and \eqn{\theta_h} denote the treatment effects
 associated with the current arm data \eqn{D} and historical data \eqn{D_h},
 respectively. Let \eqn{\delta} denote the clinically significant difference
 such that if \eqn{|\theta_h - \theta| \ge \delta}, then \eqn{\theta_h} is
 regarded as clinically distinct from \eqn{\theta}, and it is therefore
 inappropriate to borrow any information from \eqn{D_h}. Consider two
 hypotheses:

 \deqn{H_0: \theta = \theta_h, ~ H_1: \theta = \theta_h + \delta ~ or ~  \theta = \theta_h - \delta.}
 \eqn{H_0} represents that \eqn{D_h} and \eqn{D} are consistent (i.e.,
 no prior-data conflict) and thus information borrowing is desirable,
 whereas \eqn{H_1} represents that the treatment effect of \eqn{D}
 differs from \eqn{D_h} to such a degree that no information should be
 borrowed.

 The SAM prior uses the likelihood ratio test (LRT) statistics \eqn{R} to
 quantify the degree of prior-data conflict and determine the extent of
 information borrowing.

 \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(D | \theta = \theta_h) / \max(P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta)) ,}
 where \eqn{P(D | \cdot)} denotes the likelihood function. An alternative
 Bayesian choice is the posterior probability ratio (PPR):

 \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(H_0) / P( H_1) \times BF, }
 where \eqn{P(H_0)} and \eqn{P(H_1)} is the prior probabilities of \eqn{H_0}
 and \eqn{H_1} being true. \eqn{BF} is the Bayes Factor that in this case
 is the same as the LRT.

 The SAM prior, denoted as \eqn{\pi_{sam}(\theta)}, is then defined
 as a mixture of an informative prior \eqn{\pi_1(\theta)}, constructed
 based on \eqn{D_h} and a non-informative prior \eqn{\pi_0(\theta)}:

 \deqn{\pi_{sam}(\theta) = w\pi_1(\theta) + (1-w)\pi_0(\theta),}
 where the mixture weight \eqn{w} is calculated as:

 \deqn{w = R / (1 + R).}

 As the level of prior-data conflict increases, the likelihood ratio
 \eqn{R} decreases, resulting in a decrease in the weight \eqn{w}
 assigned to the informative prior and thus a decrease in information
 borrowing. As a result, \eqn{\pi_{sam}(\theta)} is data-driven and
 has the ability to self-adapt the information borrowing based on the
 degree of prior-data conflict.
}
\section{Methods (by class)}{
\itemize{
\item \code{SAM_prior(betaMix)}: The function calculates the SAM prior for beta
mixture distribution. The default \code{nf.prior} is set to be
\code{mixbeta(c(1,1,1))} which represents a uniform prior \code{Beta(1,1)}.

\item \code{SAM_prior(gammaMix)}: The function calculates the SAM prior for gamma
mixture distribution. The default \code{nf.prior} is set to be
\code{mixgamma(c(1,0.001,0.001))} which represents a vague gamma prior
\code{Gamma(0.001,0.001)}.

\item \code{SAM_prior(normMix)}: The function calculates the SAM prior for normal
mixture distribution. The default \code{nf.prior} is set to be
\code{mixnorm(c(1,summary(if.prior)['mean'], sigma))} which represents a
unit-information prior.

}}
\examples{
set.seed(123)
## Examples for binary endpoints
## Suppose that the informative prior constructed based on historical data is
## beta(40, 60)
prior.historical <- mixbeta(c(1, 40, 60))
## Data of the control arm
data.control     <- rbinom(60, size = 1, prob = 0.42)
## Calculate the mixture weight of the SAM prior
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.15,        ## Clinically significant difference
                   data = data.control  ## Control arm data
                   )
## Assume beta(1,1) as the non-informative prior used for mixture
nf.prior  <- mixbeta(nf.prior = c(1,1,1))
## Generate the SAM prior
SAM.prior <- SAM_prior(if.prior = prior.historical, ## Informative prior
                       nf.prior = nf.prior,         ## Non-informative prior
                       weight = wSAM                ## Mixture weight of the SAM prior
                       )
plot(SAM.prior)

## Examples for continuous endpoints
## Suppose that the informative prior constructed based on historical data is
## N(0, 3)
sigma      <- 3
prior.mean <- 0
prior.se   <- sigma/sqrt(100)
prior.historical <- mixnorm(c(1, prior.mean, prior.se), sigma = sigma)
## Data of the control arm
data.control <- rnorm(80, mean = 0, sd = sigma)
## Calculate the mixture weight of the SAM prior
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.2 * sigma,    ## Clinically significant difference
                   data = data.control     ## Control arm data
                   )
## Assume unit-information prior N(0,3) as the non-informative prior used
## for the mixture
nf.prior         <- mixnorm(nf.prior = c(1,prior.mean, sigma),
                            sigma = sigma)
## Generate the SAM prior
SAM.prior <- SAM_prior(if.prior = prior.historical, ## Informative prior
                       nf.prior = nf.prior,         ## Non-informative prior
                       weight = wSAM                ## Mixture weight of the SAM prior
                       )
plot(SAM.prior)

}
\references{
Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
SAM: Self-adapting mixture prior to dynamically borrow information from
historical data in clinical trials. \emph{Biometrics} 2023; 79(4), 2857-2868.

Schmidli H, Gsteiger S, Roychoudhury S, O'Hagan A,
Spiegelhalter D, Neuenschwander B.  Robust meta-analytic-predictive
priors in clinical trials with historical control information.
\emph{Biometrics} 2014; 70(4):1023-1032.
}
\seealso{
\code{\link{SAM_weight}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_OC.R
\name{get_OC}
\alias{get_OC}
\title{Generating Operating Characteristics of SAM Priors Using Analytical Engines}
\usage{
get_OC(
  if.prior,
  theta.h,
  method.w,
  prior.odds,
  nf.prior,
  prior.t = nf.prior,
  delta,
  n,
  n.t,
  target = 0.05,
  if.rMAP = FALSE,
  weight.rMAP = 0.5,
  theta,
  theta.t,
  alternative = c("greater", "less"),
  margin = 0,
  rel.tol = 1e-05,
  oc_rel.tol = 1e-06,
  interval = c(0.5, 0.999),
  n_sd_int = 8,
  ...
)
}
\arguments{
\item{if.prior}{Informative prior constructed from historical data,
represented (approximately) as a mixture of conjugate distributions.}

\item{theta.h}{Estimate of the treatment effect based on historical data.
Included for interface compatibility. If missing, the default value is set
to the posterior mean estimate from \code{if.prior}.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is LRT (Likelihood Ratio Test), the alternative option can
be PPR (Posterior Probability Ratio). See \code{\link{SAM_weight}} for more
details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}

\item{nf.prior}{Non-informative prior used as the robustifying component
for the control arm prior.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{delta}{Clinically significant difference used for the SAM prior.}

\item{n}{Sample size for the control arm.}

\item{n.t}{Sample size for the treatment arm.}

\item{target}{Target type I error used to calibrate the posterior probability
cutoff for each method. The default value is typically 0.05.}

\item{if.rMAP}{Whether to evaluate the operating characteristics of the
robust MAP prior for comparison. The default value is \code{FALSE}.}

\item{weight.rMAP}{Weight assigned to the informative prior component
(\eqn{0 \le} \code{weight.rMAP} \eqn{\le 1}) for the robust MAP prior.}

\item{theta}{A vector of the response rate (binary endpoints) or mean
(continuous endpoints) for the control arm.}

\item{theta.t}{A vector of the response rate (binary endpoints) or mean
(continuous endpoints) for the treatment arm.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be a non-negative scalar. The default
value is \code{0}.}

\item{rel.tol}{Tolerance passed to numerical root finding.}

\item{oc_rel.tol}{Relative tolerance passed to operating characteristic
evaluation.}

\item{interval}{Search interval for the posterior probability cutoff.}

\item{n_sd_int}{Half-width of the numerical integration region for each arm,
expressed as a multiple of the corresponding standard error. Used for
continuous endpoints only.}

\item{...}{Additional parameters. For continuous endpoints, this includes
\code{sigma}.}
}
\value{
A data frame with one row per scenario-method combination. The
columns are:
\describe{
  \item{Scenarios}{Scenario index.}
  \item{theta}{True control-arm parameter value.}
  \item{theta.t}{True treatment-arm parameter value.}
  \item{Methods}{Borrowing method, one of \code{"NP"}, \code{"rMAP"}, or \code{"SAM"}.}
  \item{Cutoffs}{Calibrated posterior probability cutoff for the method.}
  \item{Bias of theta}{Bias of the posterior mean estimator of \eqn{\theta}.}
  \item{RMSE of theta}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
  \item{Weight}{Average borrowing weight under the method.}
  \item{Probability of Rejection}{Repeated-sampling rejection probability.}
}
}
\description{
The \code{get_OC} function generates operating characteristics of SAM
priors using the analytical operating characteristic engines for two-arm
trials with binary or continuous endpoints. As an option, the operating
characteristics of robust MAP priors can also be generated for comparison.
}
\details{
Compared with the original simulation-based implementation, this function
does not rely on trial simulation. Instead, it first calibrates the
posterior probability cutoff for each borrowing method to achieve the
target type I error, and then evaluates operating characteristics
analytically across the requested scenarios.


For each borrowing method, the function first calibrates the posterior
probability cutoff so that the repeated-sampling rejection probability under
the boundary null scenario equals the target type I error \code{target}.
Specifically, calibration is based on the first value of \code{theta}. Let
\eqn{\theta = \code{theta}[1]} denote the control-arm parameter under the
calibration scenario. Then the treatment-arm parameter is set to
\eqn{\theta_t = \theta + margin} when \code{alternative = "greater"}, and
to \eqn{\theta_t = \theta - margin} when \code{alternative = "less"}.
Thus, when \code{margin = 0}, calibration is performed under the null
scenario \eqn{\theta_t = \theta}, corresponding to no treatment effect
difference between the treatment and control arms. After the cutoff is
calibrated, the function evaluates the operating characteristics across all
requested scenarios using the analytical two-arm engines.
}
\examples{
## Example: operating characteristics for a two-arm binary trial
## using a SAM prior without rMAP comparison

## Informative prior constructed from historical data
if.prior <- mixbeta(c(1, 20, 40))

## Evaluate operating characteristics
OC <- get_OC(
  if.prior = if.prior,
  nf.prior = mixbeta(c(1, 1, 1)),   ## Non-informative prior for control mixture
  prior.t = mixbeta(c(1, 1, 1)),    ## Prior for treatment arm
  delta = 0.2,                      ## Clinically significant difference for SAM
  n = 50,                           ## Sample size for control arm
  n.t = 100,                        ## Sample size for treatment arm
  target = 0.05,                    ## Target type I error for cutoff calibration
  if.rMAP = FALSE,                  ## Do not include rMAP comparison
  theta = c(summary(if.prior)["mean"], summary(if.prior)["mean"]),
  theta.t = c(summary(if.prior)["mean"], 0.50),
  alternative = "greater",          ## Superiority test
  margin = 0                        ## Clinical margin
)

OC

}
\references{
Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
SAM: Self-adapting mixture prior to dynamically borrow information from
historical data in clinical trials.
\emph{Biometrics} 2023;79(4):2857-2868.

Schmidli H, Gsteiger S, Roychoudhury S, O'Hagan A, Spiegelhalter D,
Neuenschwander B.
Robust meta-analytic-predictive priors in clinical trials with historical
control information.
\emph{Biometrics} 2014;70(4):1023-1032.
}
\seealso{
\code{\link{eval_oc_bin_2arm}},
\code{\link{eval_oc_cont_2arm}}, \code{\link{calibrate_cutoff_bin_2arm}},
\code{\link{calibrate_cutoff_cont_2arm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eval_oc_bin_2arm.R
\name{eval_oc_bin_2arm}
\alias{eval_oc_bin_2arm}
\title{Evaluate Multiple Scenarios for a Two-Arm Comparative Trial with Binary Endpoint}
\usage{
eval_oc_bin_2arm(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  theta,
  theta.t,
  n.t,
  n,
  delta,
  method = c("SAM", "rMAP", "NP"),
  cutoff,
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  rel.tol = 1e-08
)
}
\arguments{
\item{if.prior}{Informative prior constructed based on historical data for
the control arm, represented (approximately) as a beta mixture prior.}

\item{nf.prior}{Non-informative prior used as the robustifying component
for the control arm prior.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{theta}{A vector of true control arm response rates.}

\item{theta.t}{A vector of true treatment arm response rates.}

\item{n.t}{Sample size for the treatment arm.}

\item{n}{Sample size for the control arm.}

\item{delta}{Clinically significant difference used for the SAM prior.
This argument is only used when \code{method = "SAM"}.}

\item{method}{Borrowing methods to evaluate. Any subset of
\code{c("SAM", "rMAP", "NP")}.}

\item{cutoff}{Posterior probability cutoff specification. Either a single
numeric value applied to all methods, or a named numeric vector/list with
method-specific cutoffs, for example
\code{c(SAM = 0.94, rMAP = 0.96, NP = 0.95)}.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be a non-negative scalar. The default
value is \code{0}.}

\item{weight_rMAP}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
This argument is only used when \code{method = "rMAP"}. The default value is
0.5.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
more details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}

\item{rel.tol}{Relative tolerance passed to scenario-level numerical
integration used in \code{\link{post_summary_bin_2arm}}.}
}
\value{
A data frame with one row per scenario-method combination and columns:
\describe{
  \item{scenario}{Scenario index.}
  \item{theta}{True control arm response rate.}
  \item{theta.t}{True treatment arm response rate.}
  \item{delta_true}{True treatment effect, \eqn{\tau = \theta_t - \theta}.}
  \item{method}{Borrowing method used.}
  \item{alternative}{Direction of the posterior decision.}
  \item{cutoff}{Posterior probability cutoff used.}
  \item{margin}{Clinical margin used for inference.}
  \item{reject_prob}{Repeated-sampling rejection probability.}
  \item{bias}{Bias of the posterior mean estimator of \eqn{\theta}.}
  \item{rmse}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
  \item{mean_weight}{Average borrowing weight under the specified method.}
}
}
\description{
The \code{eval_oc_bin_2arm} function is designed to evaluate
repeated-sampling operating characteristics for multiple scenarios in a
two-arm comparative trial with a binary endpoint under one or more
borrowing strategies: self-adapting mixture prior (SAM), robust MAP prior
with fixed weight (rMAP), or non-informative prior (NP).
}
\details{
For each scenario, the function computes the repeated-sampling rejection
probability, bias, RMSE, and mean borrowing weight using
\code{\link{eval_scenario_bin_2arm}}.


The vectors \code{theta} and \code{theta.t} must have the same length.
Each pair \code{(theta[i], theta.t[i])} defines one scenario.

The \code{cutoff} argument may be common across methods or method-specific.
This is useful when each borrowing method is calibrated separately before
operating characteristics are evaluated.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SAM_weight.R
\name{SAM_weight}
\alias{SAM_weight}
\alias{SAM_weight.betaMix}
\alias{SAM_weight.normMix}
\alias{SAM_weight.gammaMix}
\title{Calculating Mixture Weight of SAM Priors}
\usage{
SAM_weight(if.prior, theta.h, method.w, prior.odds, data, delta, ...)

\method{SAM_weight}{betaMix}(if.prior, theta.h, method.w, prior.odds, data, delta, n, r, ...)

\method{SAM_weight}{normMix}(
  if.prior,
  theta.h,
  method.w,
  prior.odds,
  data,
  delta,
  m,
  n,
  sigma,
  ...
)

\method{SAM_weight}{gammaMix}(if.prior, theta.h, method.w, prior.odds, data, delta, u, w, ...)
}
\arguments{
\item{if.prior}{Informative prior constructed based on historical data,
represented (approximately) as a mixture of conjugate distributions.}

\item{theta.h}{Estimate of the treatment effect based on historical data.
If missing, the default value is set to be the posterior mean estimate from
\code{if.prior}.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See Details section for more information.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See Details section for more information.}

\item{data}{Data of the control arm from the current trial, see Methods
section for more details.}

\item{delta}{Clinically significant difference used for the SAM prior.}

\item{...}{Additional parameters required for different endpoints.}

\item{n}{Number of subjects in the control arm for continuous endpoint.}

\item{r}{Number of responses in the control arm for binary endpoint.}

\item{m}{Mean estimate in the control arm for continuous endpoint.}

\item{sigma}{Standard deviation in the control arm for continuous endpoint.}

\item{u}{Number of events in the control arm for time-to-event endpoint.}

\item{w}{Total observed time in the control arm for time-to-event endpoint.}
}
\value{
The mixture weight of the SAM priors.
}
\description{
The \code{SAM_weight} function is designed to calculate the mixture
weight of the SAM priors according to the degree of prior-data
conflicts (\emph{Yang, et al., 2023}).
}
\details{
SAM prior is constructed by mixing an informative prior
\eqn{\pi_1(\theta)}, constructed based on historical data, with a
non-informative prior \eqn{\pi_0(\theta)} using the mixture weight
\eqn{w} determined by \code{\link{SAM_weight}} function to achieve the
degree of prior-data conflict (\emph{Schmidli et al., 2015, Yang et al., 2023}).

 Let \eqn{\theta} and \eqn{\theta_h} denote the treatment effects
 associated with the current arm data \eqn{D} and historical data \eqn{D_h},
 respectively. Let \eqn{\delta} denote the clinically significant difference
 such that if \eqn{|\theta_h - \theta| \ge \delta}, then \eqn{\theta_h} is
 regarded as clinically distinct from \eqn{\theta}, and it is therefore
 inappropriate to borrow any information from \eqn{D_h}. Consider two
 hypotheses:

 \deqn{H_0: \theta = \theta_h, ~ H_1: \theta = \theta_h + \delta ~ or ~  \theta = \theta_h - \delta.}
 \eqn{H_0} represents that \eqn{D_h} and \eqn{D} are consistent (i.e.,
 no prior-data conflict) and thus information borrowing is desirable,
 whereas \eqn{H_1} represents that the treatment effect of \eqn{D}
 differs from \eqn{D_h} to such a degree that no information should be
 borrowed.

 The SAM prior uses the likelihood ratio test (LRT) statistics \eqn{R} to
 quantify the degree of prior-data conflict and determine the extent of
 information borrowing.

 \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(D | \theta = \theta_h) / \max(P(D | \theta = \theta_h + \delta), P(D | \theta = \theta_h - \delta)) ,}
 where \eqn{P(D | \cdot)} denotes the likelihood function. An alternative
 Bayesian choice is the posterior probability ratio (PPR):

 \deqn{R = P(D | H_0, \theta_h) / P(D | H_1, \theta_h) = P(H_0) / P( H_1) \times BF, }
 where \eqn{P(H_0)} and \eqn{P(H_1)} is the prior probabilities of \eqn{H_0}
 and \eqn{H_1} being true. \eqn{BF} is the Bayes Factor that in this case
 is the same as the LRT.

 The SAM prior, denoted as \eqn{\pi_{sam}(\theta)}, is then defined
 as a mixture of an informative prior \eqn{\pi_1(\theta)}, constructed
 based on \eqn{D_h} and a non-informative prior \eqn{\pi_0(\theta)}:

 \deqn{\pi_{sam}(\theta) = w\pi_1(\theta) + (1-w)\pi_0(\theta),}
 where the mixture weight \eqn{w} is calculated as:

 \deqn{w = R / (1 + R).}

 As the level of prior-data conflict increases, the likelihood ratio
 \eqn{R} decreases, resulting in a decrease in the weight \eqn{w}
 assigned to the informative prior and thus a decrease in information
 borrowing. As a result, \eqn{\pi_{sam}(\theta)} is data-driven and
 has the ability to self-adapt the information borrowing based on the
 degree of prior-data conflict.
}
\section{Methods (by class)}{
\itemize{
\item \code{SAM_weight(betaMix)}: The function calculates the mixture weight of SAM
priors for beta mixture distribution. The input \code{data} can be patient-level
data (i.e., a vector of 0 and 1 representing the response status of each
patient) or summary statistics (i.e., the number of patients and the number
of responses).

\item \code{SAM_weight(normMix)}: The function calculates the mixture weight of SAM
priors for normal mixture distribution. The input \code{data} should be
a vector of patient-level observations. The input \code{data} can be
patient-level data (i.e., a vector of continuous response of each
patient) or summary statistics (i.e., the mean estimate, number of subjects,
and the standard deviation in the control arm).

\item \code{SAM_weight(gammaMix)}: The function calculates the mixture weight of SAM
priors for gamma mixture distribution. The input \code{data} can be
patient-level data (i.e., a matrix with the first row as the censoring
indicator and the second row recording the observed time) or summary
statistics (i.e., the number of uncensored observations \code{u} and
total observed time \code{w}).

}}
\examples{
set.seed(123)
## Examples for binary endpoints
## Example 1: no prior-data conflict
## Suppose that the informative prior constructed based on historical data is
## beta(40, 60)
prior.historical <- mixbeta(c(1, 40, 60))
## Data of control arm
data.control     <- rbinom(60, size = 1, prob = 0.42)
## Calculate the mixture weight of the SAM prior
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.15,        ## Clinically significant difference
                   data = data.control  ## Control arm data
                   )
print(wSAM)

## Example 2: in the presence of prior-data conflict, where the current data
## has 12 responses in 60 patients
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.15,    ## Clinically significant difference
                   ## Methods to determine mixture weight for the SAM priors
                   ## by Posterior Probability Ratio
                   method.w = 'PPR',
                   ## Prior odds of favoring no prior-data conflicts to
                   ## the presence of prior-data conflict
                   prior.odd = 1/9,
                   n = 60,          ## Number of patients in the control arm
                   r = 12           ## Number of responses in the control arm
                   )
print(wSAM)

## Example 3: in the presence of prior-data conflict, where the current data
## has 12 responses in 60 patients
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.15, ## Clinically significant difference
                   n = 60,       ## Number of patients in the control arm
                   r = 12        ## Number of responses in the control arm
                   )
print(wSAM)

## Examples for continuous endpoints
## Example 1: no prior-data conflict
## Suppose that the informative prior constructed from historical data is
## N(0, 3)
sigma      <- 3
prior.mean <- 0
prior.se   <- sigma/sqrt(100)
prior.historical <- mixnorm(c(1, prior.mean, prior.se), sigma = sigma)
## Data of the control arm
data.control     <- rnorm(80, mean = 0, sd = sigma)
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.3 * sigma,    ## Clinically significant difference
                   data = data.control     ## Control arm data
                   )
print(wSAM)

## Example 2: in the presence of prior-data conflict, where the current data
## has mean of 0.5
data.control     <- rnorm(80, mean = 1, sd = sigma)
wSAM  <- SAM_weight(if.prior = prior.historical,
                    delta = 0.3 * sigma,    ## Clinically significant difference
                    data = data.control     ## Control arm data
                    )
print(wSAM)

## Examples for survival endpoints
## Example 1: no prior-data conflict
## Suppose the survival times from historical data follows exp(1) distribution
## with random censoring time follows U(0.5, 5) distribution
T_hi <- rexp(100, rate = 1)
C_hi <- runif(100, min = 0.5, max = 5)
## Indicators of the uncensored events
delta_hi <- as.numeric(T_hi < C_hi)
## Observed survival times from historical data
U_hi     <- T_hi
U_hi[delta_hi == 0] <- C_hi[delta_hi == 0]
## Construct the informative prior based on simulated historical data
prior.historical <- mixgamma(c(1, sum(delta_hi), sum(U_hi)),
                             param = 'ab', likelihood = 'exp')
## Suppose the survival times from control data follows exp(0.95) distribution
## with random censoring time follows U(0.5, 5) distribution
T_ci <- rexp(100, rate = 0.95)
C_ci <- runif(100, min = 0.5, max = 5)
## Indicators of the uncensored events
delta_ci <- as.numeric(T_ci < C_ci)
## Observed survival times from control data
U_ci     <- T_ci
U_ci[delta_ci == 0] <- C_ci[delta_ci == 0]
## Data of the control arm
data.control     <- rbind(sum(delta_ci), sum(U_ci))
wSAM <- SAM_weight(if.prior = prior.historical,
                   delta = 0.2,            ## Clinically significant difference
                   data = data.control     ## Control arm data
                   )
print(wSAM)

## Example 2: in the presence of prior-data conflict, where the current survival
## times follows exp(2) distribution with random censoring time follows U(0.5, 5)
## distribution
T_ci <- rexp(100, rate = 2)
C_ci <- runif(100, min = 0.5, max = 5)
## Indicators of the uncensored events
delta_ci <- as.numeric(T_ci < C_ci)
## Observed survival times from control data
U_ci     <- T_ci
U_ci[delta_ci == 0] <- C_ci[delta_ci == 0]
## Data of the control arm
data.control     <- rbind(sum(delta_ci), sum(U_ci))
wSAM  <- SAM_weight(if.prior = prior.historical,
                    delta = 0.2,            ## Clinically significant difference
                    data = data.control     ## Control arm data
                    )
print(wSAM)

}
\references{
Yang P, Zhao Y, Nie L, Vallejo J, Yuan Y.
SAM: Self-adapting mixture prior to dynamically borrow information from
historical data in clinical trials. \emph{Biometrics} 2023; 79(4), 2857-2868.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/support.R
\name{mixlink}
\alias{mixlink}
\title{Support of Distributions}
\usage{
mixlink(mix, x)
}
\description{
Returns the support of a distribution.
}
\details{
takes x and transforms it according to the defined link function of
the mixture
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calibrate_cutoff_cont_2arm.R
\name{calibrate_cutoff_cont_2arm}
\alias{calibrate_cutoff_cont_2arm}
\title{Calibrate Posterior Probability Cutoff for a Two-Arm Comparative Trial with Continuous Endpoint}
\usage{
calibrate_cutoff_cont_2arm(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  target = 0.05,
  n.t,
  n,
  sigma.t,
  sigma,
  theta.t = NULL,
  theta = NULL,
  delta,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  interval = c(0.5, 0.999),
  rel.tol = 1e-05,
  oc_rel.tol = 1e-06,
  n_sd_int = 8
)
}
\arguments{
\item{if.prior}{Informative prior constructed based on historical data for
the control arm, represented (approximately) as a normal mixture prior.}

\item{nf.prior}{Non-informative prior used for the mixture.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{target}{Target rejection probability under the calibration scenario,
typically the desired type I error rate when the calibration scenario
corresponds to the null hypothesis.}

\item{n.t}{Sample size for the treatment arm.}

\item{n}{Sample size for the control arm.}

\item{sigma.t}{Known sampling standard deviation for the treatment arm.}

\item{sigma}{Known sampling standard deviation for the control arm.}

\item{theta.t}{The mean for the treatment arm under the calibration scenario.}

\item{theta}{The mean for the control arm under the calibration scenario.}

\item{delta}{Clinically significant difference used for the SAM prior.
This argument is only used when \code{method = "SAM"}.}

\item{method}{Borrowing strategy for the control arm. Must be one of
\code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be non-negative. Default is \code{0}.}

\item{weight_rMAP}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
This argument is only used when \code{method = "rMAP"}. The default value is
0.5.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
more details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}

\item{interval}{Search interval for the posterior probability cutoff. The
default is from 0.5 to 0.999.}

\item{rel.tol}{Tolerance passed to \code{\link[stats]{uniroot}}.}

\item{oc_rel.tol}{Relative tolerance passed to scenario-level numerical
integration.}

\item{n_sd_int}{Half-width of the numerical integration region for each arm,
expressed as a multiple of the corresponding standard error.}
}
\value{
A list with the following elements:
\describe{
  \item{cutoff}{Calibrated posterior probability cutoff.}
  \item{objective}{Value of the root-finding objective at the solution.}
  \item{target}{Target rejection probability.}
  \item{method}{Borrowing method used.}
  \item{alternative}{Direction of the posterior decision.}
  \item{margin}{Clinical margin used for inference.}
  \item{theta}{True control-arm mean under the calibration scenario.}
  \item{theta.t}{True treatment-arm mean under the calibration scenario.}
  \item{interval}{Search interval used for calibration.}
}
}
\description{
The \code{calibrate_cutoff_cont_2arm} function is designed to calibrate the
posterior probability cutoff for a two-arm comparative trial with a
continuous endpoint under one borrowing strategy: self-adapting mixture prior
(SAM), robust MAP prior with fixed weight (rMAP), or non-informative prior
(NP).
}
\details{
The calibrated cutoff is chosen so that the repeated-sampling rejection
probability under a specified scenario equals the target value.


The function solves for the posterior probability cutoff such that the
scenario-level rejection probability returned by
\code{\link{eval_scenario_cont_2arm}} matches \code{target}. Under null
scenarios, this corresponds to calibration of the type I error.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eval_scenario_cont_2arm.R
\name{eval_scenario_cont_2arm}
\alias{eval_scenario_cont_2arm}
\title{Evaluate One Scenario for a Two-Arm Comparative Trial with Continuous Endpoint}
\usage{
eval_scenario_cont_2arm(
  if.prior,
  nf.prior,
  prior.t = nf.prior,
  n.t,
  n,
  sigma.t,
  sigma,
  theta.t,
  theta,
  cutoff,
  delta,
  method = c("SAM", "rMAP", "NP"),
  alternative = c("greater", "less"),
  margin = 0,
  weight_rMAP = 0.5,
  method.w = "LRT",
  prior.odds = 1,
  rel.tol = 1e-06,
  n_sd_int = 8
)
}
\arguments{
\item{if.prior}{Informative prior constructed based on historical data for
the control arm, represented (approximately) as a normal mixture prior.}

\item{nf.prior}{Non-informative prior used as the robustifying component
for the control arm prior.}

\item{prior.t}{Prior used for the treatment arm. If missing, the default
value is set to be \code{nf.prior}.}

\item{n.t}{Number of subjects in the treatment arm.}

\item{n}{Number of subjects in the control arm.}

\item{sigma.t}{Known sampling standard deviation in the treatment arm.}

\item{sigma}{Known sampling standard deviation in the control arm.}

\item{theta.t}{True treatment arm mean.}

\item{theta}{True control arm mean.}

\item{cutoff}{Posterior probability cutoff used for decision making.
Rejection occurs if the posterior probability exceeds \code{cutoff}.}

\item{delta}{Clinically significant difference used for the SAM prior.
This argument is only used when \code{method = "SAM"}.}

\item{method}{Borrowing strategy for the control arm. Must be one of
\code{"SAM"}, \code{"rMAP"}, or \code{"NP"}.}

\item{alternative}{Direction of the posterior decision. Must be one of
\code{"greater"} (for superiority) or \code{"less"} (for inferiority).}

\item{margin}{Clinical margin. Must be a non-negative scalar. The default
value is \code{0}.}

\item{weight_rMAP}{Weight assigned to the informative prior component
(\eqn{0 \leq} \code{weight_rMAP} \eqn{\leq 1}) for the robust MAP prior.
This argument is only used when \code{method = "rMAP"}. The default value is
0.5.}

\item{method.w}{Methods used to determine the mixture weight for SAM priors.
The default method is "LRT" (Likelihood Ratio Test), the alternative option
is "PPR" (Posterior Probability Ratio). See \code{\link{SAM_weight}} for
more details.}

\item{prior.odds}{The prior probability of \eqn{H_0} being true compared to
the prior probability of \eqn{H_1} being true using PPR method. The default
value is 1. See \code{\link{SAM_weight}} for more details.}

\item{rel.tol}{Relative tolerance passed to numerical integration.}

\item{n_sd_int}{Half-width of the numerical integration region for each arm,
expressed as a multiple of the corresponding standard error.}
}
\value{
A one-row data frame with the following columns:
\describe{
  \item{theta}{True control arm mean.}
  \item{theta.t}{True treatment arm mean.}
  \item{delta_true}{True treatment effect, \eqn{\tau = \theta_t - \theta}.}
  \item{method}{Borrowing method used.}
  \item{alternative}{Direction of the posterior decision.}
  \item{cutoff}{Posterior probability cutoff used for decision making.}
  \item{margin}{Clinical margin used for inference.}
  \item{reject_prob}{Repeated-sampling rejection probability.}
  \item{bias}{Bias of the posterior mean estimator of \eqn{\theta}.}
  \item{rmse}{Root mean squared error of the posterior mean estimator of \eqn{\theta}.}
  \item{mean_weight}{Average borrowing weight under the specified method.}
}
}
\description{
The \code{eval_scenario_cont_2arm} function is designed to evaluate
repeated-sampling operating characteristics for a two-arm comparative trial
with a continuous endpoint under one borrowing strategy: self-adapting
mixture prior (SAM), robust MAP prior with fixed weight (rMAP), or
non-informative prior (NP).
}
\details{
The treatment effect is defined as \eqn{\tau = \theta_t - \theta},
where \eqn{\theta_t} and \eqn{\theta} denote the true means in the
treatment and control arms, respectively.

For a given true scenario \eqn{(\theta_t, \theta)}, this function computes
the repeated-sampling rejection probability, bias, root mean squared error
(RMSE), and mean borrowing weight using one-dimensional numerical
integration.


The rejection probability is computed by reducing the repeated-sampling
decision rule to a one-dimensional integral over the control-arm sample mean.

Bias and RMSE are evaluated for the posterior mean estimator of the control
arm mean \eqn{\theta}. Both are computed from one-dimensional first and
second moments of the control-arm posterior mean. The mean borrowing weight
is computed by one-dimensional integration over the control-arm sample mean.

Under null scenarios, \code{reject_prob} corresponds to type I error. Under
alternative scenarios, it corresponds to power.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PS_prior.R
\name{PS_prior}
\alias{PS_prior}
\alias{PS_prior.default}
\alias{PS_prior.beta}
\alias{PS_prior.norm}
\title{Calculating the Propensity Score-Integrated Informative Priors}
\usage{
PS_prior(
  formula,
  data,
  outcome,
  study,
  treat,
  method,
  distance,
  ratio,
  ps.method,
  trim
)

PS_prior.default(
  formula,
  data,
  outcome,
  study,
  treat,
  method,
  distance,
  ratio,
  ps.method,
  trim
)

PS_prior.beta(
  formula,
  data,
  outcome,
  study,
  treat,
  method,
  distance,
  ratio,
  ps.method,
  trim
)

PS_prior.norm(
  formula,
  data,
  outcome,
  study,
  treat,
  method,
  distance,
  ratio,
  ps.method,
  trim
)
}
\arguments{
\item{formula}{A two-sided \code{\link{formula}} object containing the study
indicator and covariates to be used in creating the distance measure used
in the matching. This formula will be supplied to the functions that estimate
the distance measure. For example, the formula should be specified as
\code{G ~ X1 + X2 + \dots} where \code{G} represents the name of study indicator
 and \code{X1} and \code{X2} are covariates.}

\item{data}{A data frame containing the variables named in \code{formula}
and possible other arguments.}

\item{outcome}{The variable name of the outcome.}

\item{study}{The variable name of the study indicator.}

\item{treat}{The variable name of the treatment indicator.}

\item{method}{The matching method to be used. The allowed methods are
\code{"nearest"} for nearest neighbor matching (on
the propensity score by default), \code{"optimal"} [\link[MatchIt]{method_optimal}]
for optimal pair matching, \code{"full"} [\link[MatchIt]{method_full}] for optimal
full matching, \code{"genetic"} [\link[MatchIt]{method_genetic}] for genetic
matching, \code{"cem"} [\link[MatchIt]{method_cem}] for coarsened exact matching,
\code{"exact"} [\link[MatchIt]{method_exact}] for exact matching,
\code{"cardinality"} [\link[MatchIt]{method_cardinality}] for cardinality and
template matching, and \code{"subclass"} [\link[MatchIt]{method_subclass}] for
subclassification. When set to \code{"NULL"}, no matching will occur, but
propensity score estimation and common support restrictions will still occur
if requested. See the linked pages for each method for more details on what
these methods do, how the arguments below are used by each on, and what
additional arguments are allowed.}

\item{distance}{The distance measure to be used. Can be either the name of a
method of estimating propensity scores (e.g., \code{"glm"}), the name of a
method of computing a distance matrix from the covariates (e.g.,
\code{"mahalanobis"}), a vector of already-computed distance measures, or a
matrix of pairwise distances. See [\link[MatchIt]{distance}] for allowable
options. The default is \code{"glm"} for propensity scores estimated with
logistic regression using \code{glm()}. Ignored for some methods; see individual
methods pages for information on whether and how the distance measure is
used.}

\item{ratio}{For methods that allow it, how many historical control units should be
matched to each current control unit in \eqn{k:1} matching. Should be a single integer
value. See the individual methods pages for information on whether and how
this argument is used. The default is 1 for 1:1 matching.}

\item{ps.method}{PS method utilize to calculate an informative prior
based on historical data. The allowed methods are \code{"Weighting"}
or \code{"Matching"}. The default method is \code{"Weighting"}.}

\item{trim}{Lower and upper bound of trimming used in \code{"Weighting"}.
The default is [0.1,0.9].}
}
\value{
Displays the informative prior calculated from historical data
based on the selected PS method.
}
\description{
The \code{PS_prior} function is designed to calculate the Propensity Score-Integrated
(PS) informative prior constructed based on historical data.
}
\details{
This function aims to calculate informative priors using historical
 data by incorporating covariate information to enhance borrowing strength
 and address prior-data conflicts.

 Let \eqn{G} be the study indicator, where \eqn{G = 1} indicate patient is
 from current control study, and \eqn{G = 0} indicate patient is from
 historical control study. Given the covariates data \eqn{X}, the propensity
 score is defined as follows,
 \deqn{e(X) = \Pr(G = 1 | X),}
 where \link[MatchIt]{distance} allows different methods to estimate the
 propensity scores.

 Calculate informative prior through PS matching is to identify a subset of
 historical data (\eqn{D_h^*}) that have similar PS as current control data
 (\eqn{D}). Various algorithms are available for PS matching, please refer
 to \code{method}. The informative prior can then be calculated based on
 the matched historical dataset.

 Alternative, we can utilize the inverse probability of treatment weighting
 (IPTW) to adjust the distribution of \eqn{X} in historical data \eqn{D_h},
 making it similar to that in \eqn{D}. Specifically, for the \eqn{i}th
 subject, we assign a weight \eqn{\alpha_i} to the outcome \eqn{y_i} in
 \eqn{D_h} based on its PS \eqn{e(X_i)} and a fixed weight \eqn{\alpha_i = 1}
 to \eqn{X_i} in \eqn{D}, as follows:
 \deqn{\alpha_i = G_1 + (1 - G_i) \frac{e(X_i)}{1 - e(X_i)}.}
 To avoid extremely large weights that may compromise IPTW, symmetric
 trimming rule can be used to trim the tails of the PS distribution by
 input \code{trim} with default [0.1,0.9], that is to trim observations
 whose estimated PS is outside of this range.

 To standardized \eqn{\alpha}, we compute the effective sample size (ESS),
 which approximately reflects the level of precision or equivalently its
 sample size, retained in the sample after weight as
 \eqn{n^{*}_h = (\sum \alpha_i)^2 / \sum{\alpha_i^2}}. The standardized weight
 is given by
 \deqn{\alpha_i^{*} = G_i + (1 - G_i)\frac{G_i}{\sum{\alpha_i} / n_h^{*}}.}

 For binary endpoint \eqn{Y \sim Ber(\theta)}, the informative
 prior \eqn{\pi_1(\theta)} can be constructed as follows,
 \deqn{\pi_1(\theta) \propto L(\theta | D_h, \alpha^{*}) \pi_0(\theta)
  = Beta(a + \sum \alpha_i^{*}y_i, b + n_h^* - \sum \alpha_i^{*}y_i )\},}
 where \eqn{\pi_0(\theta)} is a non-informative prior, a natural choice is
 \eqn{Beta(a, b)}, with \eqn{a = b = 1}.

 For continuous endpoint \eqn{Y \sim N(0, \sigma^2)}, suppose \eqn{\sigma^2}
 is unknown, with non-informative prior \eqn{p(\theta, \sigma^2) \propto 1/\sigma^2},
 \eqn{\pi_1(\theta)} follows a student-\eqn{t} distribution with degree of
 freedom \eqn{n_h^{*} - 1}. Given that \eqn{n_h^{*}} is moderate and large,
 it can be approximated by a normal distribution
 \eqn{N(\bar{y}^{*}, {s^{*}}^2 / n_h^{*})} with
 \deqn{\bar{y}^{*} = \sum \alpha_i^* y_i / \alpha_i^*, ~~ {s^{*}}^2 =
 \sum \alpha_i^* (y_i - \bar{y}^{*})^2 / (n_h^{*}  - 1).}
}
\section{Functions}{
\itemize{
\item \code{PS_prior.default()}: The function calculates the Propensity Score-Integrated
informative prior based on historical data for binary and continuous endpoint.

\item \code{PS_prior.beta()}: The function calculates the Propensity Score-Integrated
informative prior based on historical data for binary endpoint.

\item \code{PS_prior.norm()}: The function calculates the Propensity Score-Integrated
informative prior based on historical data for continuous endpoint.

}}
\examples{
## Load example data
data('PS_SAM_data')
## Subset the data to contain historical data and current control
dat <- PS_SAM_data[PS_SAM_data$A == 0, ]
str(dat)

## Examples for binary endpoints
## Generate the informative prior based on historical data using PS Matching
summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
                 data = dat, ps.method = 'Matching', method = 'nearest',
                 outcome = 'Y_binary', study = 'G', treat = 'A'))

## Generate the informative prior based on historical data using PS Weighting
summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
                 data = dat, ps.method = 'Weighting',
                 outcome = 'Y_binary', study = 'G', treat = 'A'))

## Examples for continuous endpoints
## Generate the informative prior based on historical data using PS Matching
summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
                 data = dat, ps.method = 'Matching', method = 'nearest',
                 outcome = 'Y_continuous', study = 'G', treat = 'A'))

## Generate the informative prior based on historical data using PS Weighting
summary(PS_prior(formula = 'G ~ X_1 + X_2 + X_3',
                 data = dat, ps.method = 'Weighting',
                 outcome = 'Y_continuous', study = 'G', treat = 'A'))


}
\references{
Zhao Y, Laird G, Chen J, Yuan Y. PS-SAM: doubly robust
propensity-score-integrated self-adapting mixture prior to dynamically
borrow information from historical data.
}
\seealso{
\link[MatchIt]{matchit}
}
